#!/usr/bin/perl
#
###################################################################################
#	
#	LASTPIECE_P (Local Alignment-STate Probabilities that Insertion-type and dEletion-type gaps Co-Exist, type P):
#	    A package of programs to compute the probabilities of gapped segments in each of which an insertion-type gap and a deletion-type gap co-exist, (which were referred to as case-(iv) gapped segments by Ezawa (2016a?)),
#	    under a stochastic model of sequence evolution with biologically realistic insertions/deletions.
#	   (Written almost exclusively in Perl.)
#
#	Version 0.3: Copyright (C) 2020 Kiyoshi Ezawa
#
#    This package is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This package is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License, "GNU_GPL.txt",
#    along with this package.  If not, see <http://www.gnu.org/licenses/>.
#
#    The author can be contacted by e-mailing to <kezawa dot ezawa3 at gmail dot com>
#    (replace " dot " and " at " with "." and "@", respectively).
#
###################################################################################
#
# lastpiece.alpha.pl:
#	This is the 1st prototype of "lastpiece" (for the "Local Alignment-STate Probabilities that Insertion-type and dEletion-type gaps Co-Exist").
#	It was created by taking some pieces from:
#	'calc_log_mfacs_pars_pwa_dawg.pl' (in "~//Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData"),
#	'iteration_mfac_pwa_case_ii.pl' (in "~/Ezawa_works_UH/Main_Project/Secondary_Projects/ANALYSES/INDEL_LIKELIHOOD/Perturbation_Analyses/PWA_probs/Iteration_Formulas/Case_ii"),
#	('iteration_mfac_pwa_case_iii.pl' (in "~/Ezawa_works_UH/Main_Project/Secondary_Projects/ANALYSES/INDEL_LIKELIHOOD/Perturbation_Analyses/PWA_probs/Iteration_Formulas/Case_iii"),)
#	and '',
#	by embedding the commands to call the "servant-script"s,
#	and also by adding some functions to compile the results
#	of the steps in the whole computation procedure
#	into the final output form.
#	[CREATION STARTED on 2020/01/23, ENDED on 2020/03/02.]
#
#


use strict;
use warnings;

use MyFile_manipu qw(
	open_input_file2
	open_output_file
	list_sessions_ext
	);

	# ADDED on 2020/01/27. (PKG) #
use MyLASTPIECE_IO qw(
	print_indel_length_freqs
	print_indel_params
	print_indel_rates
	);
	# END of "ADDED on 2020/01/27. (PKG)" #
	# ... Added 'print_indel_params' on 2020/01/29. #

	# IMPORTED (w/ modifications) from 'calc_log_mfacs_pars_pwa_dawg.pl' (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/27. (PKG) #
##use MyTreeMap_indels_ML_hs qw(
use MyLASTPIECE_Sppl qw(
	cal_delta_exp_del_5p2
	);
	# END of "IMPORTED from 'calc_log_mfacs_pars_pwa_dawg.pl' (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/27. (PKG)" #


sub create_dir_rw (\$); # Added on 2020/01/29.
##sub print_indel_length_freqs  (\$$$\@\@\@\@); # OBSOLETE as of 2020/01/27.


$| = 1; # Flush the buffer for standard output as soon as it receives data.

my $bin_perl = "/usr/bin/perl"; # The Perl interpreter. # (Added on 2020/01/29). #

my $program_path = $0;
my ($program_name) = ($program_path =~ m!([^/]+)$!);

	# Controlling the level of test outputs. #
	# (0: silent; 1: very quiet; 2: quiet; 3: medium; 4: verbose; 5: very verbose) 
our $TEST_LEVEL = 3;

$MyLASTPIECE_IO::TEST_LEVEL = $TEST_LEVEL ; # Added on 2020/01/27.

	### Parameters important for the iterative calculation. ###
	### (IMPORTED w/ modifications from "iteration_mfac_pwa_case_iii.pl" (at "~/Ezawa_works_UH/Main_Project/Secondary_Projects/ANALYSES/INDEL_LIKELIHOOD/Perturbation_Analyses/PWA_probs/Iteration_Formulas/Case_iii/") on 2020/01/24 (A).)

our $TIME_INIT = 0; # Initial time.
our $TIME_FIN = 1.0; # Final time.
##our $TIME_FIN = 0.5; # Final time.
##our $TIME_FIN = 0.2; # Final time.

##=for the full-scale analysis.
our $UB_CT_SITES_OUT = 100; # Upper-bound of the number of sites in between the PASs, for outputs.
##=cut
our $UB_CT_SITES_CMPT = 2 * $UB_CT_SITES_OUT; # Upper-bound of the number of sites in between the PASs, for computations.

##=for the full-scale analysis.
our $CT_SB_INTVLS = 200; # Number of sub-intervals for the numerical integration.
##our $CT_SB_INTVLS = 50; # Number of sub-intervals for the numerical integration.
##=cut



	### END of "Parameters important for the iterative calculation. ###
	### (IMPORTED w/ modifications from "iteration_mfac_pwa_case_iii.pl" (at "~/Ezawa_works_UH/Main_Project/Secondary_Projects/ANALYSES/INDEL_LIKELIHOOD/Perturbation_Analyses/PWA_probs/Iteration_Formulas/Case_iii/") on 2020/01/24 (A).)"

	### MODEL PARAMETERS ###
	### (IMPORTED w/ modifications from "iteration_mfac_pwa_case_iii.pl" (at "~/Ezawa_works_UH/Main_Project/Secondary_Projects/ANALYSES/INDEL_LIKELIHOOD/Perturbation_Analyses/PWA_probs/Iteration_Formulas/Case_iii/") on 2020/01/24 (B).)
	### (& IMPORTED w/ modifications from "calc_log_mfacs_pars_pwa_dawg.pl" (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/26 (A).)

our $DISTRI = 'PL'; # Power-law distributions.
##our $DISTRI = 'GM'; # Geometric distributions. 


our $TOTRATE_INS = 0.1; # Total insertion rate (per unit time). (lambda_I in Dawg's indel model.)
our $TOTRATE_DEL = 0.1; # Total deletion rate (per unit time). (lambda_D in Dawg's indel model.)

our $EXPONENT_INS = 1.6; # Exponent for the power-law distribution of insertion lengths: f_I (l) is proportional to l^{-$EXPONENT_INS}.
our $EXPONENT_DEL = 1.6; # Exponent for the power-law distribution of deletion lengths: f_D (l) is proportional to l^{-$EXPONENT_DEL}.

our $PROB_EXT_INS = 0.5; # Extention probability for the geometric distribution of insertion lengths: f_I (l) is proportional to $PROB_EXT_INS^{l}.
our $PROB_EXT_DEL = 0.5; # Extention probability for the geometric distribution of deletion lengths: f_D (l) is proportional to $PROB_EXT_DEL^{l}.

=for the full-scale analysis (3lng)
our $CUTOFF_INS_LEN = 500; # Cutoff value for the insertion length.
our $CUTOFF_DEL_LEN = 500; # Cutoff value for the deletion length.
=cut
=for the full-scale analysis (3sht)
our $CUTOFF_INS_LEN = 100; # Cutoff value for the insertion length.
our $CUTOFF_DEL_LEN = 100; # Cutoff value for the deletion length.
=cut
=for TESTING
our $CUTOFF_INS_LEN = 20; # Cutoff value for the insertion length.
our $CUTOFF_DEL_LEN = 20; # Cutoff value for the deletion length.
=cut

##=Default behavior (Added on 2020/02/02). #
our $CUTOFF_INS_LEN = $UB_CT_SITES_OUT; # Cutoff value for the insertion length.
our $CUTOFF_DEL_LEN = $UB_CT_SITES_OUT; # Cutoff value for the deletion length.
##=cut # End of "Default behavior (Added on 2020/02/02)." #


	### END of "MODEL PARAMETERS  ###
	### (IMPORTED w/ modifications from "iteration_mfac_pwa_case_iii.pl" (at "~/Ezawa_works_UH/Main_Project/Secondary_Projects/ANALYSES/INDEL_LIKELIHOOD/Perturbation_Analyses/PWA_probs/Iteration_Formulas/Case_iii/") on 2020/01/24 (B).)
	### (& IMPORTED w/ modifications from "calc_log_mfacs_pars_pwa_dawg.pl" (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/26 (A).)"


		### Necessary Directories and Files. ###

		### The paths to the important directories. ###

our $DIR_SERVANTS = './Servants'; # The directory (i.e., folder) that contains the servant-scripts.
our $DIR_OUTPUTS = './Outputs'; # The directory that will contain the outputs. #


		## The Names of the Sub-directories and sub-sub-directories 
		## (, which can be under each of $DIR_SERVANTS and $DIR_OUTPUTS).

our $NAME_FINAL = 'Final'; # The sub-directory to store the final results.
our $NAME_PREFINAL = 'PreFinal'; # The sub-sub-directory to store the final results on each class of coloring-pattern evolution.
our $NAME_PREPREFINAL = 'PrePreFinal'; # The sub-sub-directory to store the final results on each class (except '2nd_i') of coloring-pattern evolution. # (Added on 2020/02/10. (GL1)) #

			# MODIFIED on 2020/02/10. (GL1) #
our @SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],

	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	##['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber', 'mu_DA2Adel', 'mu_pA2pDcrL2Ber2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (5th element)
	['Summations', [$NAME_PREFINAL]], # (6th element) # Newly ADDED on 2020/02/29. (GL1) #
	);
=OBSOLETE as of 2020/02/10. (GL1)
our @SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],

	####['Basic', ['Insdelrates', 'Dressed_insdelrates', 'muP_case_i', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element)
	##['Basic', ['Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # MODIFIED on 2020/01/26.
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', 'mu_Dcr2AD2D', $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', 'mu_A2ADA2AD2Adel', $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_DcrL2DcrR2Adel', 'mu_A2DA2DAD2Adel', $NAME_PREFINAL]], # ((3+1)th element)
	['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber', 'mu_DA2Adel', 'mu_pA2pDcrL2Ber2Adel', 'mu_A2DA2Ber2Adel', $NAME_PREFINAL]], # (5th element)
	);
=cut # END of "OBSOLETE as of 2020/02/10. (GL1)"
			# END of "MODIFIED on 2020/02/10. (GL1)" #


		# A 2D-array of the servant scripts. #
		# Each script is associated with the corresponding sub-sub-directory
		# in @SUBDIRNAMES. #

			# Modified on 2020/02/14. (GL2c) #
##our $NAME_PL_EXT2INIT = 'extend2init.pl'; # (Added on 2020/02/10. (GL2a))
##our $NAME_PL_EXT2FIN  = 'extend2fin.pl';   # (Added on 2020/02/10. (GL2b))
our $NAME_PL_EXT2INIT = 'graft_convolution.pl';
our $NAME_PL_EXT2FIN  = $NAME_PL_EXT2INIT;
			# End of "Modified on 2020/02/14. (GL2c)" #

			# MODIFIED on 2020/02/10. (GL2) #
our @SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	##['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl', 'cmpt_mu_DA2Adel.pl', 'cmpt_mu_pA2pDcrL2Ber2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (5th element)
	['wgtd_summs_3Darrays_for_mfacs.pl'], # (6th element) # Newly ADDED on 2020/02/29. (GL2) #
	);
=OBSOLETE as of 2020/02/10. (GL2)
our @SERVANTS = (
	####['', '', 'cmpt_muP_case_i.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element)
	##['', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # MODIFIED on 2020/01/26.
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', 'cmpt_mu_Dcr2AD2D.pl', 'cmpt_muP_2nd_ii.pl'], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', 'cmpt_mu_A2ADA2AD2Adel.pl', 'cmpt_muP_3rd_a.pl'], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', 'cmpt_mu_A2DA2DAD2Adel.pl', 'cmpt_muP_3rd_c.pl'], # ((3+1)th element)
	['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl', 'cmpt_mu_DA2Adel.pl', 'cmpt_mu_pA2pDcrL2Ber2Adel.pl', 'cmpt_mu_A2DA2Ber2Adel.pl', 'cmpt_mu_3rd_e.pl'], # (5th element)
	);
=cut # END of "OBSOLETE as of 2020/02/10. (GL2)"
			# END of "MODIFIED on 2020/02/10. (GL2)" #

			### Modify the global variables for Testing. (Added on 2020/02/02) ###

=For Test1 (on 2020/02/02)
$UB_CT_SITES_OUT = 30; # Upper-bound of the number of sites in between the PASs, for outputs.
$UB_CT_SITES_CMPT = 2 * $UB_CT_SITES_OUT; # Upper-bound of the number of sites in between the PASs, for computations.
$CT_SB_INTVLS = 10; # Number of sub-intervals for the numerical integration.

$CUTOFF_INS_LEN = $UB_CT_SITES_OUT; # Cutoff value for the insertion length.
$CUTOFF_DEL_LEN = $UB_CT_SITES_OUT; # Cutoff value for the deletion length.

$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test1'; # The directory that will contain the outputs. #
=cut

=For Test2a,2c (on 2020/02/02)
$UB_CT_SITES_OUT = 500; # Upper-bound of the number of sites in between the PASs, for outputs.
$UB_CT_SITES_CMPT = $UB_CT_SITES_OUT; # Upper-bound of the number of sites in between the PASs, for computations.

##$CT_SB_INTVLS = 100; # Number of sub-intervals for the numerical integration. (For Test2a)
$CT_SB_INTVLS = 200; # Number of sub-intervals for the numerical integration. (For Test2c)

$CUTOFF_INS_LEN = $UB_CT_SITES_OUT; # Cutoff value for the insertion length.
$CUTOFF_DEL_LEN = $UB_CT_SITES_OUT; # Cutoff value for the deletion length.

##$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test2a'; # The directory that will contain the outputs. # (For Test2a)
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test2c'; # The directory that will contain the outputs. # (For Test2c)
=cut

=For Test2b (on 2020/02/02)
$UB_CT_SITES_OUT = 500; # Upper-bound of the number of sites in between the PASs, for outputs.
$UB_CT_SITES_CMPT = 500; # Upper-bound of the number of sites in between the PASs, for computations.
$CT_SB_INTVLS = 100; # Number of sub-intervals for the numerical integration.

$CUTOFF_INS_LEN = 100; # Cutoff value for the insertion length.
$CUTOFF_DEL_LEN = 100; # Cutoff value for the deletion length.

my $nfac_ins1 = my $nfac_del1 = 0;
for (my $l=1; $l<= 100; $l++) { $nfac_ins1 += exp (-$EXPONENT_INS * log ($l)); }
for (my $l=1; $l<= 100; $l++) { $nfac_del1 += exp (-$EXPONENT_DEL * log ($l)); }

my $nfac_ins5 = my $nfac_del5 = 0;
for (my $l=1; $l<= 500; $l++) { $nfac_ins5 += exp (-$EXPONENT_INS * log ($l)); }
for (my $l=1; $l<= 500; $l++) { $nfac_del5 += exp (-$EXPONENT_DEL * log ($l)); }

$TOTRATE_INS *= $nfac_ins1 /$nfac_ins5;
$TOTRATE_DEL *= $nfac_del1 /$nfac_del5;

$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test2b'; # The directory that will contain the outputs. #
=cut

=For Tests 1 & 2{a,b,c} (on 2020/02/02).
@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii']], # (0th element) # Further MODIFIED on 2020/01/29.
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	);
=cut

##=For Test 3a, Test 5, Test 6,  Tests 7,8, Tests 9,10,11, Tests 12-14, Tests 15-20, Tests 21-26, Tests 27-29, 30.
$UB_CT_SITES_OUT = 30; # Upper-bound of the number of sites in between the PASs, for outputs.
$UB_CT_SITES_CMPT = 30; # Upper-bound of the number of sites in between the PASs, for computations.
$CT_SB_INTVLS = 20; # Number of sub-intervals for the numerical integration.

$CUTOFF_INS_LEN = 30; # Cutoff value for the insertion length.
$CUTOFF_DEL_LEN = 30; # Cutoff value for the deletion length.
##=cut
=For Test 3b (on 2020/02/03).
$UB_CT_SITES_OUT = 100; # Upper-bound of the number of sites in between the PASs, for outputs.
$UB_CT_SITES_CMPT = 200; # Upper-bound of the number of sites in between the PASs, for computations.
$CT_SB_INTVLS = 100; # Number of sub-intervals for the numerical integration.

$CUTOFF_INS_LEN = 100; # Cutoff value for the insertion length.
$CUTOFF_DEL_LEN = 100; # Cutoff value for the deletion length.
=cut
=For Tests 10mini, 11mini, 12mini, 13mini, 14mini, 15mini, 16mini, 17mini, 18mini, 19mini, 20mini, 21mini, 22mini, 23mini, 24mini, 25mini, 26mini, 27mini, 28mini, 29mini, 30mini.
$TIME_INIT = 0; # Initial time.
$TIME_FIN = 0.1; # Final time.
$UB_CT_SITES_OUT = 5; # Upper-bound of the number of sites in between the PASs, for outputs.
$UB_CT_SITES_CMPT = 5; # Upper-bound of the number of sites in between the PASs, for computations.
$CT_SB_INTVLS = 5; # Number of sub-intervals for the numerical integration.

$CUTOFF_INS_LEN = 5; # Cutoff value for the insertion length.
$CUTOFF_DEL_LEN = 5; # Cutoff value for the deletion length.
=cut

=For Test 3{a,b} (on 2020/02/03).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test3a'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test3b'; # The directory that will contain the outputs. #

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],

	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0'] ],
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl'],
	);
=cut
=For Test 5 (on 2020/02/06).
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test5'; # The directory that will contain the outputs. #

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr']], # (0th element) # Further MODIFIED on 2020/01/29.
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	);
=cut
=For Test 6 (on 2020/02/07).
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test6'; # The directory that will contain the outputs. #

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	);
=cut
=For Test 7 (on 2020/02/08).
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test7'; # The directory that will contain the outputs. #

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty']], # (1st element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl'], # (1st element)
	);
=cut
=For Test 8 (on 2020/02/11).
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test8'; # The directory that will contain the outputs. #

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	);
=cut
=For Test 9 (on 2020/02/14).
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test9'; # The directory that will contain the outputs. #

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel']], # (2nd element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl'], # (2nd element)
	);
=cut
=For Test 10 (on 2020/02/16).
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test10'; # The directory that will contain the outputs. #
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test10mini'; # For 'Test 10mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL]], # (2nd element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN], # (2nd element)
	);
=cut
=For Test 11 (on 2020/02/17).
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test11'; # The directory that will contain the outputs. #
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test11mini'; # For 'Test 11mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	);
=cut
=For Test 12 (on 2020/02/18).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test12'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test12mini'; # For 'Test 12mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel' ]], # (3rd element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl' ], # (3rd element)
	);
=cut
=For Test 13 (on 2020/02/18).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test13'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test13mini'; # For 'Test 13mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL]], # (3rd element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT], # (3rd element)
	);
=cut
=For Test 14 (on 2020/02/18).
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test14'; # The directory that will contain the outputs. #
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test14mini'; # For 'Test 14mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	);
=cut
=For Test 15 (on 2020/02/20).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test15'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test15mini'; # For 'Test 15mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel' ]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl' ], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	);
=cut
=For Test 16 (on 2020/02/21).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test16'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test16mini'; # For 'Test 16mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel' ]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl' ], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	);
=cut
=For Test 17 (on 2020/02/22).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test17'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test17mini'; # For 'Test 17mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD' ]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl' ], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	);
=cut
=For Test 18 (on 2020/02/23).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test18'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test18mini'; # For 'Test 18mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel' ]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl' ], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	);
=cut
=For Test 19 (on 2020/02/24).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test19'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test19mini'; # For 'Test 19mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL ]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT ], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	);
=cut
=For Test 20 (on 2020/02/24).
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test20'; # The directory that will contain the outputs. #
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test20mini'; # For 'Test 20mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	);
=cut
=For Test 21 (on 2020/02/25).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test21'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test21mini'; # For 'Test 21mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	['3rd_e',  ['mu_Dcr2Berdot' ]], # (5th element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	['cmpt_mu_Dcr2Berdot.pl' ], # (5th element)
	);
=cut
=For Test 22 (on 2020/02/26).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test22'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test22mini'; # For 'Test 22mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber' ]], # (5th element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl' ], # (5th element)
	);
=cut
=For Test 23 (on 2020/02/27).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test23'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test23mini'; # For 'Test 23mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber', 'mu_DA2Adel' ]], # (5th element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl', 'cmpt_mu_DA2Adel.pl' ], # (5th element)
	);
=cut
##=For Test 24 (on 2020/02/28&29).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test24'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test24mini'; # For 'Test 24mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber', 'mu_DA2Adel', 'mu_pA2pDcrL2Ber2Adel']], # (5th element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl', 'cmpt_mu_DA2Adel.pl', 'cmpt_mu_pA2pDcrL2Ber2Adel.pl'], # (5th element)
	);
##=cut
=For Test 25 (on 2020/02/dd). # Actually, it was NOT PERFORMED!!

#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test25'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test25mini'; # For 'Test 25mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber', 'mu_DA2Adel', 'mu_pA2pDcrL2Ber2Adel', $NAME_PREPREFINAL]], # (5th element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl', 'cmpt_mu_DA2Adel.pl', 'cmpt_mu_pA2pDcrL2Ber2Adel.pl', $NAME_PL_EXT2INIT], # (5th element)
	);
=cut
=For Test 26 (on 2020/02/29).
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test26'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test26mini'; # For 'Test 26mini'.

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber', 'mu_DA2Adel', 'mu_pA2pDcrL2Ber2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (5th element)
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl', 'cmpt_mu_DA2Adel.pl', 'cmpt_mu_pA2pDcrL2Ber2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (5th element)
	);
=cut
##=For Tests 27 & 28 (on 2020/03/01) & Tests 29 & 30 (on 2020/03/02).

#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test29'; # The directory that will contain the outputs. #
$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test30'; # The directory that will contain the outputs. #

#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test27mini'; # For 'Test 27mini'. (Testing the script 'wgtd_summs_3Darrays_for_mfacs.pl' at the 2nd perturbation level.)
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test28mini'; # For 'Test 28mini'. (Testing the script 'wgtd_summs_3Darrays_for_mfacs.pl' at the 3rd and 2nd+3rd perturbation levels.)
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test29mini'; # For 'Test 29mini'. (Testing the Final Processes.)
#$DIR_OUTPUTS = '../../TESTs/Test_LASTPIECE/Test30mini'; # For 'Test 30mini'. (Testing the REVISION on the logging of "3rd_e, 3".)

@SUBDIRNAMES = ( # element = [{sub-dir-name}, [{sub-sub-dir-name1}, ...] ],
	['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.
	['2nd_i',  ['mu_A2empty', $NAME_PREFINAL]], # (1st element)
	['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)
	['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber', 'mu_DA2Adel', 'mu_pA2pDcrL2Ber2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (5th element)
	['Summations', [$NAME_PREFINAL]], # (6th element) # Newly ADDED on 2020/02/29. (GL1) #
	);

@SERVANTS = (
	['', '', '', 'cmpt_muP_case_i2iii.pl', 'cmpt_muP0.pl', 'cmpt_mu_Dcr.pl', 'cmpt_mu_Adel.pl'], # (0th element) # Further MODIFIED on 2020/01/29.
	['cmpt_mu_A2empty.pl',	'cmpt_muP_2nd_i.pl'], # (1st element)
	['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)
	['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)
	['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl', 'cmpt_mu_DA2Adel.pl', 'cmpt_mu_pA2pDcrL2Ber2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (5th element)
	['wgtd_summs_3Darrays_for_mfacs.pl'], # (6th element) # Newly ADDED on 2020/02/29. (GL2) #
	);
##=cut

		### RESTART FROM HERE (Tests) ###

			### End of "Modify the global variables for Testing. (Added on 2020/02/02)" ###




	### Examine whether the necessary scripts already exist or not.
	### If not, exit the script while issuing a warning message.

unless (( -d $DIR_SERVANTS ) and ( -r $DIR_SERVANTS ) ) {
	print STDOUT "[ERROR '$program_name'] CanNOT FIND a readable top servant directory, '$DIR_SERVANTS': $? !\n\n";
	print STDERR "[ERROR '$program_name'] $! \n";
	die ;
}

for (my $i = 0; $i < @SUBDIRNAMES; $i++) {

    my ($subdirname, $sbsbdirnames) = @{$SUBDIRNAMES[$i]};
    my $sbdir = "$DIR_SERVANTS/$subdirname";
    unless (( -d $sbdir ) and ( -r $sbdir )) {
	print STDOUT "[ERROR '$program_name'] CanNOT FIND a readable sub-directory, '$sbdir': $? !\n\n";
	print STDERR "[ERROR '$program_name'] $! \n";
	die ;
    }

    my $list_scripts = $SERVANTS[$i];

    for (my $k = 0; $k <@{$sbsbdirnames}; $k++) {
	my $svnt_script_name = $list_scripts->[$k];
	unless ((defined $svnt_script_name) and ($svnt_script_name =~ m/\S/)) { next; } # Skip if the script name is not specified. #

	my $sbsbdirname = $sbsbdirnames->[$k];
	my $sbsbdir  = "$sbdir/$sbsbdirname";
	##unless (( -d $sbsbdir ) and ( -r $sbsbdir )) {
	unless ( (( -d $sbsbdir ) or ( -l $sbsbdir )) and ( -r $sbsbdir )) { # MODIFIED on 2020/02/10. (CHK-A) #
	    print STDOUT "[ERROR '$program_name'] CanNOT FIND a readable sub-sub-directory, '$sbsbdir': $? !\n\n";
	    print STDERR "[ERROR '$program_name'] $! \n";
	    die ;
	}
	my $svnt_script = "$sbsbdir/$svnt_script_name";
	##unless (( -f $svnt_script ) and ( -r $svnt_script )) {
	unless ( (( -f $svnt_script ) or ( -l $svnt_script)) and ( -r $svnt_script )) { # MODIFIED on 2020/02/14. (CHK-B) #
	    print STDOUT "[ERROR '$program_name'] CanNOT FIND a readable servant-script, '$svnt_script': $? !\n\n";
	    print STDERR "[ERROR '$program_name'] $! \n";
	    die ;
	}
    }
}

	## If $DIR_OUTPUTS does NOT exist yet, create one. ##


		# MODIFIED on 2020/01/29. (DR-TOP) #
unless ( create_dir_rw ($DIR_OUTPUTS) ) { 
	print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the TOP output directory, \"$DIR_OUTPUTS\": $?\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$DIR_OUTPUTS/\" ...\n\n") if ($TEST_LEVEL >1); # (Added on 2020/02/18.) #

=REPLACED with the subroutine, 'create_dir_rw'.
unless ( -d $DIR_OUTPUTS ) {
   unless (0 == system ("mkdir -p $DIR_OUTPUTS") ) {
	print STDOUT "[ERROR '$program_name'] FAILED to CREATE the TOP OUTPUT DIRECTORY, '$DIR_OUTPUTS/': $? !\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
   }
}
unless (( -r $DIR_OUTPUTS ) and ( -w $DIR_OUTPUTS )) {
    unless (0 == system ("chmod u+rw $DIR_OUTPUTS") ) {
	print STDOUT "[ERROR '$program_name'] FAILED to CHANGE permission MODES of the top output directory, '$DIR_OUTPUTS': $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
    }
}
=cut # END of "REPLACED with the subroutine, 'create_dir_rw'."
		# End of "MODIFIED on 2020/01/29. (DR-TOP)" #


	### (IMPORTED w/ modifications from "calc_log_mfacs_pars_pwa_dawg.pl" (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/26 (C).)

	### FINAL Output files. ###

my $fin_outdir_name = ($DISTRI eq 'PL') ? "$DISTRI${EXPONENT_INS}_co$CUTOFF_INS_LEN" : "$DISTRI${PROB_EXT_INS}_co$CUTOFF_INS_LEN" ; ## For the full-scale analysis.
##my $outdir = 'TESTs/Test1'; ## For TESTING the script.

my $fin_outdir = "$DIR_OUTPUTS/$NAME_FINAL/$fin_outdir_name";
my $out_deldel = "$fin_outdir/delta_del.txt";

	# For the bulk multiplication factors. #

		# MODIFIED on 2020/03/01. (Pre-FIN1) #
my $name_out_as_dg = 'lmfacs_asites_vs_dgaps.txt';
my $name_out_ag_ds = 'lmfacs_agaps_vs_dsites.txt';
my $name_out_as_ds = 'lmfacs_asites_nhom_dsites.txt';
my $name_out_nosites = 'lmfacs_nosites.txt';

my $out_as_dg = "$fin_outdir/$name_out_as_dg";
my $out_ag_ds = "$fin_outdir/$name_out_ag_ds";
my $out_as_ds = "$fin_outdir/$name_out_as_ds";
my $out_nosites = "$fin_outdir/$name_out_nosites"; # For case-(i) gapped segments, which do NOT have any ancestral or descendant sites in between the PASs. 
=OBSOLETE as of 2020/03/01. (Pre-FIN1)
my $out_as_dg = "$fin_outdir/lmfacs_asites_vs_dgaps.txt";
my $out_ag_ds = "$fin_outdir/lmfacs_agaps_vs_dsites.txt";
my $out_as_ds = "$fin_outdir/lmfacs_asites_nhom_dsites.txt";
my $out_nosites = "$fin_outdir/lmfacs_nosites.txt"; # For case-(i) gapped segments, which do NOT have any ancestral or descendant sites in between the PASs. (ADDED on 2020/01/27.) #
=cut # END of "OBSOLETE as of 2020/03/01. (Pre-FIN1)"
		# END of "MODIFIED on 2020/03/01. (Pre-FIN1)" #

	# For the boundary multiplication factors. #
	# [NOTE: This first proto-type does NOT compute boundary factors;
	#	instead, the following files will be just symbolic links to the files of bulk factors.
	# 	Theoretically, however, it is NOT so difficult to compute boundary factors under some genuine evolution models (will be documented somewhere soon),
	#	and such function will be implemented in the future, if I have enough time.]

		# MODIFIED on 2020/03/01. (Pre-FIN2) #
my $name_out_bd_as_dg = "bd_$name_out_as_dg";
my $name_out_bd_ag_ds = "bd_$name_out_ag_ds";
my $name_out_bd_as_ds = "bd_$name_out_as_ds";
my $name_out_bd_nosites = "bd_$name_out_nosites";

my $out_bd_as_dg = "$fin_outdir/$name_out_bd_as_dg";
my $out_bd_ag_ds = "$fin_outdir/$name_out_bd_ag_ds";
my $out_bd_as_ds = "$fin_outdir/$name_out_bd_as_ds";
my $out_bd_nosites = "$fin_outdir/$name_out_bd_nosites"; # For case-(i) gapped segments, which do NOT have any ancestral or descendant sites in between the PASs. 
=OBSOLETE as of 2020/03/01. (Pre-FIN2)
my $out_bd_as_dg = "$fin_outdir/bd_lmfacs_asites_vs_dgaps.txt";
my $out_bd_ag_ds = "$fin_outdir/bd_lmfacs_agaps_vs_dsites.txt";
my $out_bd_as_ds = "$fin_outdir/bd_lmfacs_asites_nhom_dsites.txt";
my $out_bd_nosites = "$fin_outdir/bd_lmfacs_nosites.txt"; # For case-(i) gapped segments, which do NOT have any ancestral or descendant sites in between the PASs. (ADDED on 2020/01/27.) #
=cut # END of "OBSOLETE as of 2020/03/01. (Pre-FIN2)"
		# END of "MODIFIED on 2020/03/01. (Pre-FIN2)" #

	### Create the FINAL output directory. ###

		# MODIFIED on 2020/01/29. (DR-FIN) #
unless ( create_dir_rw ($fin_outdir) ) { 
	print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the FINAL OUTPUT DIRECTORY, \"$fin_outdir\": $?\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$fin_outdir/\" ...\n\n") if ($TEST_LEVEL >1); # (Added on 2020/02/18.) #

=OBSOLETE as of 2020/01/30.
unless ( -d $fin_outdir ) {
    unless (0 == system ("mkdir -p $fin_outdir") ) {
	print STDOUT "[ERROR '$program_name'] FAILED to CREATE the FINAL OUTPUT DIRECTORY '$fin_outdir': $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
    }
}
unless (( -r $fin_outdir ) and ( -w $fin_outdir )) {
    unless (0 == system ("chmod u+rw $fin_outdir") ) {
	print STDOUT "[ERROR '$program_name'] FAILED to CHANGE permission MODES of the output directory '$fin_outdir': $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
    }
}
=cut # END of "OBSOLETE as of 2020/01/30."
		# END of "MODIFIED on 2020/01/29. (DR-FIN)" #

	### END of "(IMPORTED w/ modifications from "calc_log_mfacs_pars_pwa_dawg.pl" (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/26 (C).)"


	### (IMPORTED w/ modifications from "calc_log_mfacs_pars_pwa_dawg.pl" (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/26 (D).)

	### Declare the Beginning of the Script ###

print STDOUT "\n<<< Program name: $program_name (path: $program_path) >>>\n\n";

print STDOUT "The Perl Interpreter: $bin_perl .\n\n"; # Added on 2020/01/29.

print STDOUT "TOP Output Directory: $DIR_OUTPUTS .\n\n";

print STDOUT "Final Output Directory: $fin_outdir .\n\n";

print STDOUT "Final Output files:\n\n";

print STDOUT "   (Delta_del) $out_deldel ,\n\n";

print STDOUT "   (log-multiplication factors in the bulk)\n";
print STDOUT "     (for ancestral sites vs descendant gaps) $out_as_dg ,\n";
print STDOUT "     (for ancestral gaps vs descendant sites) $out_ag_ds ,\n";
print STDOUT "     (for ancestral sites & non-homologous descendant sites) $out_as_ds ,\n";
print STDOUT "     (for NO ancestral or descendant sites in between the PASs) $out_nosites .\n\n";

print STDOUT "   (log-multiplication factors on the PWA boundaries) # NOTE: in the current version, these are just symbolic links to the bulk factors above. #\n";
print STDOUT "     (for ancestral sites vs descendant gaps) $out_bd_as_dg ,\n";
print STDOUT "     (for ancestral gaps vs descendant sites) $out_bd_ag_ds ,\n";
print STDOUT "     (for ancestral sites & non-homologous descendant sites) $out_bd_as_ds ,\n";
print STDOUT "     (for NO ancestral or descendant sites in between the PASs) $out_bd_nosites .\n\n\n";


print STDOUT "\n", '$TEST_LEVEL = ', $TEST_LEVEL, " .\n\n";

##print STDOUT "\n<< Domains within which the multiplication factors will be calculated >>\n\n";
##print STDOUT '#{Ancestral sites}  = (0,) 1, ..., ', $UB_CT_ANCSITES, " ,\n";
##print STDOUT '#{Descendant sites} = (0,) 1, ..., ', $UB_CT_DECSITES, " .\n\n";

print STDOUT "\n<< Parameters necessary for recursive computations of the multiplication factors: >>\n\n";

print STDOUT '#{Sub-time-intervals} = ', $CT_SB_INTVLS, " .\n\n";
print STDOUT '($TIME_INIT, $TIME_FIN) = ', "($TIME_INIT, $TIME_FIN) .\n\n";

print STDOUT '{Upper-bound of #{sites} for output}      = ', $UB_CT_SITES_OUT, " ,\n"; 
print STDOUT '{Upper-bound of #{sites} for computation} = ', $UB_CT_SITES_CMPT, " .\n\n"; 

print STDOUT "\n<< Model Parameters >>\n\n";

print STDOUT '$TOTRATE_INS = ', $TOTRATE_INS, " ,\n";
print STDOUT '$TOTRATE_DEL = ', $TOTRATE_DEL, " .\n\n";

print STDOUT '$DISTRI = ', $DISTRI, " .\n\n";

if ($DISTRI eq 'PL') {
   print STDOUT '@FREQS_INS[l] prpt l^{', -$EXPONENT_INS, "}   (l=1,...,$CUTOFF_INS_LEN) ,\n";
   print STDOUT '@FREQS_DEL[l] prpt l^{', -$EXPONENT_DEL, "}   (l=1,...,$CUTOFF_DEL_LEN) .\n\n";
} elsif ($DISTRI eq 'GM') {
   print STDOUT '@FREQS_INS[l] prpt ', $PROB_EXT_INS, "^{l}   (l=1,...,$CUTOFF_INS_LEN) ,\n";
   print STDOUT '@FREQS_DEL[l] prpt ', $PROB_EXT_DEL, "^{l}   (l=1,...,$CUTOFF_DEL_LEN) .\n\n";
}

print STDOUT "\n";

	### END of "(IMPORTED w/ modifications from "calc_log_mfacs_pars_pwa_dawg.pl" (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/26 (D).)"


	### Create Sub-Directory System for the Basic Outputs. (ADDED on 2020/01/29) ###

print STDOUT "\n<<< Process DR-Basic: Creating the Sub-Directory System for the Basic Outputs >>>\n\n";

my ($sbdirname_basic, $sbsbdirnames_basic) = @{$SUBDIRNAMES[0]};

my $sbdir_out_basic = "$DIR_OUTPUTS/$sbdirname_basic";

unless ( create_dir_rw ($sbdir_out_basic) ) { 
	print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the sub-directory, \"$sbdir_out_basic\", for the Basic Outputs: $?\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbdir_out_basic/\" ...\n\n") if ($TEST_LEVEL >2); # (Added on 2020/02/18.) #

for (my $k = 0; $k < @{$sbsbdirnames_basic}; $k++) {

	my $sbsbdir = join ('/', $sbdir_out_basic, $sbsbdirnames_basic->[$k]);

	unless ( create_dir_rw ($sbsbdir) ) { 
	    print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the $k th sub-sub-directory, \"$sbsbdir\", for the Basic Outputs: $?\n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}

   (print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbsbdir/\" ...\n\n") if ($TEST_LEVEL >2); # (Added on 2020/02/18.) #

}

(print STDOUT "\n ... Finished Creating the Sub-Directory System ...\n\n") if ($TEST_LEVEL > 1);

	### END of "Create Sub-Directory System for the Basic Outputs. (ADDED on 2020/01/29)" ###


	### (IMPORTED w/ modifications from "calc_log_mfacs_pars_pwa_dawg.pl" (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/26 (B).)


	## Preliminary 00-A: Compute the insertion/deletion length frequencies (in the bulk). ##

my $nfac_ins = my $nfac_del = 0;

if ($DISTRI eq 'PL') {
   for (my $l=1; $l<= $CUTOFF_INS_LEN; $l++) { $nfac_ins += exp (-$EXPONENT_INS * log ($l)); }
   for (my $l=1; $l<= $CUTOFF_DEL_LEN; $l++) { $nfac_del += exp (-$EXPONENT_DEL * log ($l)); }
} elsif ($DISTRI eq 'GM') {
   for (my $l=1; $l<= $CUTOFF_INS_LEN; $l++) { $nfac_ins += exp ( $l * log($PROB_EXT_INS) ); }
   for (my $l=1; $l<= $CUTOFF_DEL_LEN; $l++) { $nfac_ins += exp ( $l * log($PROB_EXT_DEL) ); }
} else {
    print STDOUT "[ERROR '$program_name'] ILLEGAL PARAMETER: ", '$DISTRI = ', "$DISTRI !\n\n";
    print STDERR "[ERROR '$program_name'] $!\n";
    die ;
}


our @FREQS_INS = (0); # Relative frequencies of the insertion lengths.
our @FREQS_DEL = (0); # Relative frequencies of the deletion lengths.

if ($DISTRI eq 'PL') {
   for (my $l=1; $l<= $CUTOFF_INS_LEN; $l++) { $FREQS_INS[$l] = exp (-$EXPONENT_INS * log ($l)) /$nfac_ins; }
   for (my $l=1; $l<= $CUTOFF_DEL_LEN; $l++) { $FREQS_DEL[$l] = exp (-$EXPONENT_DEL * log ($l)) /$nfac_del; }
} elsif ($DISTRI eq 'GM') {
   for (my $l=1; $l<= $CUTOFF_INS_LEN; $l++) { $FREQS_INS[$l] = exp ( $l * log ($PROB_EXT_INS) ) /$nfac_ins; }
   for (my $l=1; $l<= $CUTOFF_DEL_LEN; $l++) { $FREQS_DEL[$l] = exp ( $l * log ($PROB_EXT_DEL) ) /$nfac_del; }
}

	## Preliminary 00-B: Compute the Indel length frequencies on the boundaries ##

print STDOUT "\n<<< Preliminary Process 00-B: Computing the Indel length frequencies on the boundaries >>>\n\n";

our @FREQS_INS_BD = @FREQS_INS; # Under Dawg's model, the insertion length frequency remains the same whether the insertion is in the bulk or on either boundary.

##our @FREQS_DEL_BD = ();
our @FREQS_DEL_BD = (0); # Modified on 2020/01/27.

for (my $l=1; $l<= $CUTOFF_DEL_LEN; $l++) { # Preliminary for-loop (over deletion lenghts). #

   my $freq_bd = 0;
   for (my $lp= $l; $lp <= $CUTOFF_DEL_LEN; $lp++) { $freq_bd += $FREQS_DEL[$lp]; }

   $FREQS_DEL_BD[$l] = $freq_bd;
}				# End of the preliminary for-loop (over deletion lengths). #


	## Preliminary 00-C: Compute Other Important Parameters ##

print STDOUT "\n<<< Preliminary Process 00-C: Computing Other Important Parameters >>>\n\n";

our $TOTRATE_INDEL = $TOTRATE_INS + $TOTRATE_DEL;
our $SIZE_FULL_INTVL = $TIME_FIN - $TIME_INIT;
our $SIZE_SB_INTVL = $SIZE_FULL_INTVL /$CT_SB_INTVLS;


		### MODIFIED on 2020/01/31. (00-C-1) ###

our @TIME_LAPSES_CMPT = (); # Values of the time lapse (= t_F - t_I) at which the factors will be computed.
for (my $i = 0; $i <= $CT_SB_INTVLS; $i++) {
	$TIME_LAPSES_CMPT[$i] = $i * $SIZE_SB_INTVL;
} 

####our @SMPL_TIME_POINTS = ($CT_SB_INTVLS < 4) ? (1 .. $CT_SB_INTVLS) : (1 .. 4);
##our @SMPL_TIME_POINTS = ($CT_SB_INTVLS < 4) ? (1 .. $CT_SB_INTVLS) : (); # MODIFIED on 2020/02/24. #
our @SMPL_TIME_POINTS = ($CT_SB_INTVLS < 4) ? (1 .. $CT_SB_INTVLS -1) : (); # => DEBUGGED on 2020/03/01. #
##for (my $k = 1; $k < int ($CT_SB_INTVLS/5); $k++) {
for (my $k = 1; $k <= int ($CT_SB_INTVLS/5); $k++) { # CORRECTED on 2020/02/24. #
	push @SMPL_TIME_POINTS, 5 * $k;
}
if ($CT_SB_INTVLS % 5) { # $CT_SB_INTVLS is NOT a multiple of 5.
	push @SMPL_TIME_POINTS, $CT_SB_INTVLS;
}

our @TIME_LAPSES_OUT = (); # Values of the time-lapse (= t_F - t_I) at which the factors will be output.
our @E_INDEL_CTS_OUT = (); # Expected counts of indels at which the factors will be output. #
foreach my $i (@SMPL_TIME_POINTS) {
	my $tlapse = $i * $SIZE_SB_INTVL;
	my $ect_indels = $tlapse * $TOTRATE_INDEL;

	push @TIME_LAPSES_OUT, $tlapse;
	push @E_INDEL_CTS_OUT, $ect_indels;
}

=OBSOLETE as of 2020/01/31. (1)
our @TIME_LAPSES = (); # Values of the time lapse (= t_F - t_I) to be examined. #
for (my $i = 0; $i <= $CT_SB_INTVLS; $i++) {
	$TIME_LAPSES[$i] = $i * $SIZE_SB_INTVL;
} 

our @E_INDEL_CTS = (); # Expected counts of indels. #
foreach my $tlapse (@TIME_LAPSES) { push @E_INDEL_CTS, $TOTRATE_INDEL * $tlapse; }
=cut # End of "OBSOLETE as of 2020/01/31. (1)"

		### End of "MODIFIED on 2020/01/31. (00-C-1)" ###


print STDOUT '$TOTRATE_INDEL = ', $TOTRATE_INDEL, " \;\n";
print STDOUT '$SIZE_FULL_INTVL = ', $SIZE_FULL_INTVL, " \;\n";
print STDOUT '$SIZE_SB_INTVL = ', $SIZE_SB_INTVL, " .\n\n";

		### MODIFIED on 2020/01/31. (00-C-2) ###

print STDOUT '@TIME_LAPSES_CMPT = (', join (', ', @TIME_LAPSES_CMPT), ") .\n\n";

print STDOUT '@SMPL_TIME_POINTS = (', join (', ', @SMPL_TIME_POINTS), ") .\n\n";
print STDOUT '@TIME_LAPSES_OUT = (', join (', ', @TIME_LAPSES_OUT), ") .\n\n";
print STDOUT '@E_INDEL_CTS_OUT = (', join (', ', @E_INDEL_CTS_OUT), ") .\n\n";

##print STDOUT '@TIME_LAPSES = (', join (', ', @TIME_LAPSES), ") .\n\n";
##print STDOUT '@E_INDEL_CTS = (', join (', ', @E_INDEL_CTS), ") .\n\n";

		### End of "MODIFIED on 2020/01/31. (00-C-2)" ###

print STDOUT "\n";


	### END of "(IMPORTED w/ modifications from "calc_log_mfacs_pars_pwa_dawg.pl" (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/26 (B).)"


	### (Added on 2020/01/27) (Pre00) ###


	## Preliminary 00-D: Compute the Indel rates. ##

print STDOUT "\n<<< Preliminary Process 00-D: Computing the Indel rates >>>\n\n";

our @RATES_INS = (0);
our @RATES_DEL = (0);
our @RATES_INS_BD  = (0);
our @RATES_DEL_BD = (0);

for (my $l=1; $l<= $CUTOFF_INS_LEN; $l++) {
	$RATES_INS[$l] = $TOTRATE_INS * $FREQS_INS[$l];
	$RATES_INS_BD[$l] = $TOTRATE_INS * $FREQS_INS_BD[$l];
}

for (my $l=1; $l<= $CUTOFF_DEL_LEN; $l++) {
	$RATES_DEL[$l] = $TOTRATE_DEL * $FREQS_DEL[$l];
	$RATES_DEL_BD[$l] = $TOTRATE_DEL * $FREQS_DEL_BD[$l];
}

	## Preliminary 00-E: Compute the DRESSED Indel rates. ##

print STDOUT "\n<<< Preliminary Process 00-E: Computing the DRESSED Indel rates (actually times \$SIZE_SB_INTVL) >>>\n\n";

our @DR_RATES_INS = (0);
our @DR_RATES_DEL = (0);
our @DR_RATES_INS_BD  = (0);
our @DR_RATES_DEL_BD = (0);

for (my $l=1; $l<= $CUTOFF_INS_LEN; $l++) {
	my $dress = (1 - exp (- $SIZE_SB_INTVL * $TOTRATE_INDEL * $l)) / ($TOTRATE_INDEL * $l);
	$DR_RATES_INS[$l] =  $RATES_INS[$l] * $dress ;
	$DR_RATES_INS_BD[$l] = $RATES_INS_BD[$l] * $dress ;
}

for (my $l=1; $l<= $CUTOFF_DEL_LEN; $l++) {
	my $dress = (exp ($SIZE_SB_INTVL * $TOTRATE_INDEL * $l) - 1) / ($TOTRATE_INDEL * $l);
	$DR_RATES_DEL[$l] = $RATES_DEL[$l] * $dress ;
	$DR_RATES_DEL_BD[$l] = $RATES_DEL_BD[$l] * $dress ;
}

	### END of "(Added on 2020/01/27) (Pre00)" ###


	### IMPORTED w/modifications from 'calc_log_mfacs_pars_pwa_dawg.pl' (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/27. (Pre00) ##

##print STDOUT "\n<<< Preliminary Process 00-F: Calculating the 'universal' exponent factor >>>\n\n\n";
print STDOUT "\n<<< Preliminary Process 00-F: Computing the 'universal' exponent factor >>>\n\n\n"; # Modified on 2020/01/29.

my $delta_del = cal_delta_exp_del_5p2 (@FREQS_DEL, $CUTOFF_DEL_LEN);
unless (defined $delta_del) {
	print STDOUT "[ERROR '$program_name'] Subroutine 'cal_delta_exp_dep_5p2' FAILED !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
	### END of "IMPORTED w/modifications from 'calc_log_mfacs_pars_pwa_dawg.pl' (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/27. (Pre00)" ##



	### Print out some parameters computed in the preliminary processes. ###

		# (ADDED on 2020/01/29.) (00-P1) #

print STDOUT "\n<<< Preliminary Process 00-P: Printing out the quantities computed in the \"Preliminary Process 00-X\" >>>\n\n";

## my $sbdir_out_basic = "$DIR_OUTPUTS/$sbdirname_basic";
## my ($sbdirname_basic, $sbsbdirnames_basic) = @{$SUBDIRNAMES[0]};
## ['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.

my $odir_id_params = join ('/', $sbdir_out_basic, $sbsbdirnames_basic->[0]);

my $ofile_id_len_freqs = join ('/', $odir_id_params, 'indel_len_freqs.txt');

		# End of "(ADDED on 2020/01/29.) (00-P1)" #

unless ( print_indel_length_freqs ($ofile_id_len_freqs, $CUTOFF_INS_LEN, $CUTOFF_DEL_LEN, @FREQS_INS, @FREQS_DEL, @FREQS_INS_BD, @FREQS_DEL_BD) ) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'print_indel_length_freqs', FAILED to output indel length frequencies !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Printed out the indel-length frequencies to the file, '$ofile_id_len_freqs' ...\n\n") if ($TEST_LEVEL > 1);


			# (Added on 2020/01/29) (Pre-PO) #

my $ofile_id_prms = join ('/', $odir_id_params, 'indel_params.txt');

unless ( print_indel_params ($ofile_id_prms, $TOTRATE_INS, $TOTRATE_DEL, $CUTOFF_INS_LEN, $CUTOFF_DEL_LEN) ) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'print_indel_params', FAILED to output some important indel parameters !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Printed out some important indel parameters to the file, '$ofile_id_prms' ...\n\n") if ($TEST_LEVEL > 1);

			# End of "(Added on 2020/01/29) (Pre-PO)" #


			# (Added on 2020/01/27) (Pre-PO) #

				# (Added on 2020/01/29) (00-P2) #
my $odir_id_rates = join ('/', $sbdir_out_basic, $sbsbdirnames_basic->[1]);
my $ofile_id_rates = join ('/', $odir_id_rates, 'raw_indel_rates.txt');
				# End of "(Added on 2020/01/29) (00-P2)" #

##unless ( print_indel_rates ($ofile_id_rates, $TOTRATE_INS, $TOTRATE_DEL, $CUTOFF_INS_LEN, $CUTOFF_DEL_LEN, @RATES_INS, @RATES_DEL, @RATES_INS_BD, @RATES_DEL_BD) ) { # OBSOLETE as of 2020/01/29. # }
unless ( print_indel_rates ($ofile_id_rates, 'Raw', $CUTOFF_INS_LEN, $CUTOFF_DEL_LEN, @RATES_INS, @RATES_DEL, @RATES_INS_BD, @RATES_DEL_BD) ) { # MODIFIED on 2020/01/29. #
	print STDOUT "[ERROR '$program_name'] The subroutine, 'print_indel_rates', FAILED to output raw indel rates !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Printed out the raw indel rates to the file, '$ofile_id_rates' ...\n\n") if ($TEST_LEVEL > 1);


				# (Added on 2020/01/29) (00-P3) #
my $odir_dr_id_rates = join ('/', $sbdir_out_basic, $sbsbdirnames_basic->[2]);
my $ofile_dressed_id_rates = join ('/', $odir_dr_id_rates, 'dressed_indel_rates.txt');
				# End of "(Added on 2020/01/29) (00-P3)" #

####unless ( print_indel_rates ($ofile_dressed_id_rates, $TOTRATE_INS, $TOTRATE_DEL, $CUTOFF_INS_LEN, $CUTOFF_DEL_LEN, @DR_RATES_INS, @DR_RATES_DEL, @DR_RATES_INS_BD, @DR_RATES_DEL_BD) ) { # OBSOLETE as of 2020/01/29. #
##unless ( print_indel_rates ($ofile_dressed_id_rates, 'Dressed', $CUTOFF_INS_LEN, $CUTOFF_DEL_LEN, @DR_RATES_INS, @DR_RATES_DEL, @DR_RATES_INS_BD, @DR_RATES_DEL_BD) ) { # MODIFIED on 2020/01/29. #
unless ( print_indel_rates ($ofile_dressed_id_rates, 'Dressed (x $SIZE_SB_INTVL)', $CUTOFF_INS_LEN, $CUTOFF_DEL_LEN, @DR_RATES_INS, @DR_RATES_DEL, @DR_RATES_INS_BD, @DR_RATES_DEL_BD) ) { # Further MODIFIED on 2020/02/02. #
	print STDOUT "[ERROR '$program_name'] The subroutine, 'print_indel_rates', FAILED to output DRESSED indel rates !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Printed out the DRESSED indel rates (actually times \$SIZE_SB_INTVL) to the file, '$ofile_dressed_id_rates' ...\n\n") if ($TEST_LEVEL > 1);

			# End of "(Added on 2020/01/27) (Pre-PO)" #


my $infile_id_prms = $ofile_id_prms; # MOVED from below on 2020/02/04.
my $infile_id_rates = $ofile_id_rates; # MOVED from below on 2020/02/04.
my $infile_dressed_id_rates = $ofile_dressed_id_rates; # Added on 2020/02/04.

	# IMPORTED w/modifications from 'calc_log_mfacs_pars_pwa_dawg.pl' (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/27. (Pre-PO) #

(print STDOUT "< Print out the 'universal' exponent factor >\n\n\n") if ($TEST_LEVEL > 1);

my $fh_out0 = open_output_file ($out_deldel, 0);
unless (defined $fh_out0) {
	print STDOUT "[ERROR '$program_name'] 'open_output_file' FAILED on the output file, '$out_deldel' !\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}

print $fh_out0 "* Delta_del := Sum_{l=1}^{$CUTOFF_DEL_LEN} ", '(l-1) * $FREQ_DEL_LEN[l]', " .\n";
print $fh_out0 "\n";

print $fh_out0 "Delta_del = $delta_del\n\n";

close ($fh_out0);

(print STDOUT "\n ... Finished Printing out the 'universal' exponent factor to the file, '$out_deldel' ...\n\n") if ($TEST_LEVEL > 1); # Added on 2020/01/29. (Pre-PO A) #

	# End of "IMPORTED w/modifications from 'calc_log_mfacs_pars_pwa_dawg.pl' (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/") on 2020/01/27. (Pre-PO)" #


(print STDOUT "\n\n --- FINISHED OutPutting Parameters Computed in the Preliminary Processes ! ---\n\n\n") if ($TEST_LEVEL > 0); # Added on 2020/01/29. (Pre-PO B)

	### END of "Print out some parameters computed in the preliminary processes." ###


	### BASIC: Compute the "Basic" Quantities. ###
	### (Writing Started on 2020/01/29, Ended on 2020/02/06.)

print STDOUT "\n\n<<<<<  BASIC: Compute the \"Basic\" Quantities >>>>>\n\n";

my @servants_basic = @{$SERVANTS[0]};
my $dir_servants_basic = join ('/', $DIR_SERVANTS, $sbdirname_basic);


print STDOUT "\n\n<<< Basic 3: Compute full Multiplication Factors for cases (i), (ii), and (iii) >>>\n\n";

			 # A BUG fixed on 2020/02/02. #
my $svnt_script_muP_case_i2iii = join ('/', $dir_servants_basic, $sbsbdirnames_basic->[3], $servants_basic[3]);
##my $svnt_script_muP_case_i2iii = join ('/', $dir_servants_basic, $servants_basic[3]);
			 # End of "A BUG fixed on 2020/02/02." #

## my ($sbdirname_basic, $sbsbdirnames_basic) = @{$SUBDIRNAMES[0]};

my $odir_muP_case_i2iii = join ('/', $sbdir_out_basic, $sbsbdirnames_basic->[3]);

my $ofile_muP_case_i = join ('/', $odir_muP_case_i2iii, 'case_i.txt');
my $ofile_muP_case_ii = join ('/', $odir_muP_case_i2iii, 'case_ii.txt');
my $ofile_muP_case_iii = join ('/', $odir_muP_case_i2iii, 'case_iii.txt');
my $log_muP_case_i2iii = join ('/', $odir_muP_case_i2iii, 'log.txt');


######unless ( 0 == system ("$bin_perl $svnt_script_muP_case_i2iii $TIME_FIN $TIME_INIT $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_id_rates $ofile_muP_case_i $ofile_muP_case_ii $ofile_muP_case_iii" ) ) {
####unless ( 0 == system ("$bin_perl $svnt_script_muP_case_i2iii $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_id_rates $ofile_muP_case_i $ofile_muP_case_ii $ofile_muP_case_iii" ) ) { # MODIFIED on 2020/01/30.
##unless ( 0 == system ("$bin_perl $svnt_script_muP_case_i2iii $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_id_rates $ofile_muP_case_i $ofile_muP_case_ii $ofile_muP_case_iii > $log_muP_case_i2iii 2>&1" ) ) { # Further MODIFIED on 2020/02/02.
unless ( 0 == system ("(time $bin_perl $svnt_script_muP_case_i2iii $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_id_rates $ofile_muP_case_i $ofile_muP_case_ii $ofile_muP_case_iii) > $log_muP_case_i2iii 2>&1" ) ) { # Yet Further MODIFIED on 2020/02/06.
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_muP_case_i2iii' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_muP_case_i2iii' ...\n\n") if ($TEST_LEVEL > 1);
if ($TEST_LEVEL > 2) { # (Added on 2020/02/18) #

	print STDOUT "\n[ Output Files: ]\n\n";
	print STDOUT "  (for case-(i))     $ofile_muP_case_i ,\n";
	print STDOUT "  (for case-(ii))    $ofile_muP_case_ii ,\n";
	print STDOUT "  (for case-(iii))   $ofile_muP_case_iii .\n";
	print STDOUT "\n\n";

} # End of "if ($TEST_LEVEL > 2) { # (Added on 2020/02/18) # ...}"

my $infile_muP_case_i = $ofile_muP_case_i; # Added on 2020/02/04.
my $infile_muP_case_ii = $ofile_muP_case_ii; # Added on 2020/03/01.
my $infile_muP_case_iii = $ofile_muP_case_iii; # Added on 2020/03/01.


print STDOUT "\n\n<<< Basic 3+1: Compute Multiplication Factors under \"Base\" Rate Operator >>>\n\n";

my $svnt_script_muP0 = join ('/', $dir_servants_basic, $sbsbdirnames_basic->[3+1], $servants_basic[3+1]);

my $odir_muP0 = join ('/', $sbdir_out_basic, $sbsbdirnames_basic->[3+1]);
my @ofiles_muP0 = ();

foreach my $sigmaBE (0, 1, 2) { # 1st main outer foreach-loop (over $sigmaBE) #

   (print STDOUT "\n< sigmaBE = $sigmaBE: >\n\n") if ($TEST_LEVEL > 0);

	my $ofile_muP0 = join ('/', $odir_muP0, "sigmaBE_eq_$sigmaBE.txt");
	$ofiles_muP0[$sigmaBE] = $ofile_muP0;

	my $log_muP0 = join ('/', $odir_muP0, "log_sigmaBE_eq_$sigmaBE.txt");

	##unless ( 0 == system ("$bin_perl $svnt_script_muP0 $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $sigmaBE $infile_id_prms $infile_id_rates $ofile_muP0 > $log_muP0 2>&1" ) ) { 
	unless ( 0 == system ("(time $bin_perl $svnt_script_muP0 $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $sigmaBE $infile_id_prms $infile_id_rates $ofile_muP0) > $log_muP0 2>&1" ) ) { # Modified on 2020/02/06. #
	    print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_muP0' w/ \$sigmaBE = $sigmaBE: $? \n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
    (print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_muP0' w/ \$sigmaBE = $sigmaBE ...\n\n") if ($TEST_LEVEL > 1);
    (print STDOUT "[ Output File: ] $ofile_muP0 .\n\n") if ($TEST_LEVEL > 2); # (Added on 2020/02/18.) #

} # End of the 1st main outer foreach-loop (over $sigmaBE). #


		# Modified on 2020/02/06. (Basic 3+1) #
my $rf_infiles_muP0 = \@ofiles_muP0;
##my ($infile_muP0_sigmaBE0, $infile_muP0_sigmaBE1, $infile_muP0_sigmaBE2) = @ofiles_muP0;
	# The paths to the files storing muP0 w/ $sigmaBE = 0, 1, and 2, respectively.
		# End of "Modified on 2020/02/06. (Basic 3+1)" #


print STDOUT "\n\n<<< Basic 5: Compute Multiplication Factors for the creation of a \"D\"-region >>>\n\n";

my $svnt_script_mu_Dcr = join ('/', $dir_servants_basic, $sbsbdirnames_basic->[5], $servants_basic[5]);

my $odir_mu_Dcr = join ('/', $sbdir_out_basic, $sbsbdirnames_basic->[5]);

my $ofile_mu_Dcr = join ('/', $odir_mu_Dcr, 'sigmaBE_eq_2.txt');

my $log_mu_Dcr = join ('/', $odir_mu_Dcr, 'log_sigmaBE_eq_2.txt');

##unless ( 0 == system ("$bin_perl $svnt_script_mu_Dcr $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 2 $infile_id_prms $infile_id_rates $infile_muP0_sigmaBE2 $ofile_mu_Dcr --RAW > $log_mu_Dcr 2>&1" ) ) {
##unless ( 0 == system ("(time $bin_perl $svnt_script_mu_Dcr $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 2 $infile_id_prms $infile_id_rates $infile_muP0_sigmaBE2 $ofile_mu_Dcr --RAW) > $log_mu_Dcr 2>&1" ) ) { # Modified on 2020/02/06. #
unless ( 0 == system ("(time $bin_perl $svnt_script_mu_Dcr $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 2 $infile_id_prms $infile_id_rates $rf_infiles_muP0->[2] $ofile_mu_Dcr --RAW) > $log_mu_Dcr 2>&1" ) ) { # Further Modified on 2020/02/06. #
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_Dcr' w/ \$sigmaBE = 2: $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_Dcr' w/ \$sigmaBE = 2 ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_mu_Dcr .\n\n") if ($TEST_LEVEL > 2); # (Added on 2020/02/18.) #


my $infile_mu_Dcr = $ofile_mu_Dcr;



print STDOUT "\n\n<<< Basic 6: Compute Multiplication Factors for the deletion of an \"A\"-region >>>\n\n";

my $svnt_script_mu_Adel = join ('/', $dir_servants_basic, $sbsbdirnames_basic->[6], $servants_basic[6]);

my $odir_mu_Adel = join ('/', $sbdir_out_basic, $sbsbdirnames_basic->[6]);
my @ofiles_mu_Adel = ();

foreach my $sigmaBE (0, 1, 2) { # 2nd main outer foreach-loop (over $sigmaBE) #

   (print STDOUT "\n< sigmaBE = $sigmaBE: >\n\n") if ($TEST_LEVEL > 0);

	my $ofile_mu_Adel = join ('/', $odir_mu_Adel, "sigmaBE_eq_$sigmaBE.txt");
	$ofiles_mu_Adel[$sigmaBE] = $ofile_mu_Adel;

	my $log_mu_Adel = join ('/', $odir_mu_Adel, "log_sigmaBE_eq_$sigmaBE.txt");

	unless ( 0 == system ("(time $bin_perl $svnt_script_mu_Adel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $sigmaBE $infile_id_prms $infile_id_rates $rf_infiles_muP0->[$sigmaBE] $ofile_mu_Adel --RAW) > $log_mu_Adel 2>&1" ) ) {
	    print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_Adel' w/ \$sigmaBE = $sigmaBE: $? \n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
    (print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_Adel' w/ \$sigmaBE = $sigmaBE ...\n\n") if ($TEST_LEVEL > 1);
    (print STDOUT "[ Output File: ] $ofile_mu_Adel .\n\n") if ($TEST_LEVEL > 2); # (Added on 2020/02/18.) #

} # End of the 2nd main outer foreach-loop (over $sigmaBE). #


my $rf_infiles_mu_Adel = \@ofiles_mu_Adel;
	# The paths to the files storing muP0 w/ $sigmaBE = 0, 1, and 2, respectively.

## my $sbdir_out_basic = "$DIR_OUTPUTS/$sbdirname_basic";
#
## my ($sbdirname_basic, $sbsbdirnames_basic) = @{$SUBDIRNAMES[0]};
#
## ['Basic', ['Insdelparams', 'Insdelrates', 'Dressed_insdelrates', 'muP_case_i2iii', 'muP0', 'mu_Dcr', 'mu_Adel']], # (0th element) # Further MODIFIED on 2020/01/29.


print STDOUT "\n\n<<<<< All BASIC Processes Ended />/>/>/>/>\n\n\n";

	### END of "BASIC: Compute the "Basic" Quantities. ###
	### (Writing Started on 2020/01/29, Ended on 2020/02/06.)"


	### Create Sub-Directory System for the Outputs of the Perturbation Class, '2nd_i'. (ADDED on 2020/02/07) ###

print STDOUT "\n<<< Process DR-2nd_i: Creating the Sub-Directory System for the Outputs of the Perturbation Class, '2nd_i' >>>\n\n";

my ($sbdirname_2nd_i, $sbsbdirnames_2nd_i) = @{$SUBDIRNAMES[1]};

my $sbdir_out_2nd_i = "$DIR_OUTPUTS/$sbdirname_2nd_i";

unless ( create_dir_rw ($sbdir_out_2nd_i) ) { 
	print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the sub-directory, \"$sbdir_out_2nd_i\", for the Outputs of the Perturbation Class, '2nd_i': $?\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbdir_out_2nd_i/\" ...\n\n") if ($TEST_LEVEL >2); # (Added on 2020/02/18.) #

for (my $k = 0; $k < @{$sbsbdirnames_2nd_i}; $k++) {

	my $sbsbdir = join ('/', $sbdir_out_2nd_i, $sbsbdirnames_2nd_i->[$k]);

	unless ( create_dir_rw ($sbsbdir) ) { 
	    print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the $k th sub-sub-directory, \"$sbsbdir\", for the Outputs of the Perturbation Class, '2nd_i': $?\n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbsbdir/\" ...\n\n") if ($TEST_LEVEL >2); # (Added on 2020/02/18.) #

}

(print STDOUT "\n ... Finished Creating the Sub-Directory System ...\n\n") if ($TEST_LEVEL > 1);

	### END of "Create Sub-Directory System for the Outputs of the Perturbation Class, '2nd_i'. (ADDED on 2020/02/07)" ###


	### 2nd_i: Compute the Quantities involved in the Perturbation Class, '2nd_i'. ###
	### (Writing Started on 2020/02/07, Ended on 2020/02/11.)

print STDOUT "\n\n<<<<<  2nd_i: Compute the Quantities involved in the Perturbation Class, '2nd_i'  >>>>>\n\n";

my @servants_2nd_i = @{$SERVANTS[1]};
my $dir_servants_2nd_i = join ('/', $DIR_SERVANTS, $sbdirname_2nd_i);


print STDOUT "\n\n<<< 2nd_i, 0: Compute Multiplication Factors for 'A -> {emptyset}' >>>\n\n";

my $svnt_script_mu_A2empty = join ('/', $dir_servants_2nd_i, $sbsbdirnames_2nd_i->[0], $servants_2nd_i[0]);

## my ($sbdirname_2nd_i, $sbsbdirnames_2nd_i) = @{$SUBDIRNAMES[1]};

my $odir_mu_A2empty = join ('/', $sbdir_out_2nd_i, $sbsbdirnames_2nd_i->[0]);

my $ofile_mu_A2empty = join ('/', $odir_mu_A2empty, 'sigmaBE_eq_2.txt');
## my $ofile_muP_case_i = join ('/', $odir_muP_case_i2iii, 'case_i.txt');
my $log_mu_A2empty = join ('/', $odir_mu_A2empty, 'log_sigmaBE_eq_2.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_mu_A2empty $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 2 $rf_infiles_mu_Adel->[2] $infile_muP_case_i $ofile_mu_A2empty) > $log_mu_A2empty 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_A2empty' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_A2empty' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_mu_A2empty .\n\n") if ($TEST_LEVEL > 2); # (Added on 2020/02/18.) #


my $infile_mu_A2empty = $ofile_mu_A2empty;


print STDOUT "\n\n<<< 2nd_i, 1 ($NAME_PREFINAL): Compute Sub-Total Multiplication Factors for the Class, '2nd_i'(, divided by the exponentiated total-rate-increment) >>>\n\n";

my $svnt_script_muP_2nd_i = join ('/', $dir_servants_2nd_i, $sbsbdirnames_2nd_i->[1], $servants_2nd_i[1]);

## my ($sbdirname_2nd_i, $sbsbdirnames_2nd_i) = @{$SUBDIRNAMES[1]};

my $odir_muP_2nd_i = join ('/', $sbdir_out_2nd_i, $sbsbdirnames_2nd_i->[1]);

my $ofile_muP_2nd_i = join ('/', $odir_muP_2nd_i, 'case_iv_2nd_i.txt');
		# A BUG fixed on 2020/02/11. #
##my $log_muP_2nd_i = join ('/', $odir_mu_A2empty, 'log_case_iv_2nd_i.txt');
my $log_muP_2nd_i = join ('/', $odir_muP_2nd_i, 'log_case_iv_2nd_i.txt');
		# End of "A BUG fixed on 2020/02/11." #


		# my $infile_mu_Dcr = $ofile_mu_Dcr;

##unless ( 0 == system ("(time $bin_perl $svnt_script_muP_2nd_i $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 2 $infile_id_prms $infile_mu_A2empty $infile_mu_Dcr $ofile_muP_2nd_i) > $log_muP_2nd_i 2>&1" ) ) {
unless ( 0 == system ("(time $bin_perl $svnt_script_muP_2nd_i $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_mu_A2empty $infile_mu_Dcr $ofile_muP_2nd_i) > $log_muP_2nd_i 2>&1" ) ) { # MODIFIED on 2020/02/10. #
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_muP_2nd_i' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_muP_2nd_i' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_muP_2nd_i .\n\n") if ($TEST_LEVEL > 2); # (Added on 2020/02/18.) #

my $infile_muP_2nd_i = $ofile_muP_2nd_i;

	### END of "2nd_i: Compute the Quantities involved in the Perturbation Class, '2nd_i'. ###
	### (Writing Started on 2020/02/07, Ended on 2020/02/11.)"



	### Create Sub-Directory System for the Outputs of the Perturbation Class, '2nd_ii'. (ADDED on 2020/02/11) ###

print STDOUT "\n<<< Process DR-2nd_ii: Creating the Sub-Directory System for the Outputs of the Perturbation Class, '2nd_ii' >>>\n\n";

my ($sbdirname_2nd_ii, $sbsbdirnames_2nd_ii) = @{$SUBDIRNAMES[2]};

my $sbdir_out_2nd_ii = "$DIR_OUTPUTS/$sbdirname_2nd_ii";

if ($TEST_LEVEL > 3) { # (DR-2nd_ii) # (Added on 2020/02/14). #

	print STDOUT "\n";
	print STDOUT '$sbdirname_2nd_ii = ', $sbdirname_2nd_ii, " .\n\n";
	print STDOUT '@{$sbsbdirnames_2nd_ii} = (', join (', ', @{$sbsbdirnames_2nd_ii}), ") .\n\n\n";

} # End of "if ($TEST_LEVEL > 3) { # (DR-2nd_ii) # ...}"


unless ( create_dir_rw ($sbdir_out_2nd_ii) ) { 
	print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the sub-directory, \"$sbdir_out_2nd_ii\", for the Outputs of the Perturbation Class, '2nd_ii': $?\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbdir_out_2nd_ii/\" ...\n\n") if ($TEST_LEVEL >2); # (Added on 2020/02/18.) #

for (my $k = 0; $k < @{$sbsbdirnames_2nd_ii}; $k++) {

	my $sbsbdir = join ('/', $sbdir_out_2nd_ii, $sbsbdirnames_2nd_ii->[$k]);

	unless ( create_dir_rw ($sbsbdir) ) { 
	    print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the $k th sub-sub-directory, \"$sbsbdir\", for the Outputs of the Perturbation Class, '2nd_ii': $?\n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbsbdir/\" ...\n\n") if ($TEST_LEVEL >2); # (Added on 2020/02/18.) #
}

(print STDOUT "\n ... Finished Creating the Sub-Directory System ...\n\n") if ($TEST_LEVEL > 1);

	### END of "Create Sub-Directory System for the Outputs of the Perturbation Class, '2nd_ii'. (ADDED on 2020/02/11)" ###


	### 2nd_ii: Compute the Quantities involved in the Perturbation Class, '2nd_ii'. ###
	### (Writing Started on 2020/02/11, Ended on 2020/02/16.)

print STDOUT "\n\n<<<<<  2nd_ii: Compute the Quantities involved in the Perturbation Class, '2nd_ii'  >>>>>\n\n";

my @servants_2nd_ii = @{$SERVANTS[2]};
my $dir_servants_2nd_ii = join ('/', $DIR_SERVANTS, $sbdirname_2nd_ii);


if ($TEST_LEVEL > 3) { # (2nd_ii) # (Added on 2020/02/14). #

	print STDOUT "\n";
	print STDOUT '@servants_2nd_ii = (', join (', ', @servants_2nd_ii), ") .\n\n\n";

} # End of "if ($TEST_LEVEL > 3) { # (2nd_ii) # (Added on 2020/02/14). # ...}"


print STDOUT "\n\n<<< 2nd_ii, 0: Compute Multiplication Factors for 'Dcr -> Adel' >>>\n\n"; # (Added on 2020/02/11) #

my $svnt_script_mu_Dcr2Adel = join ('/', $dir_servants_2nd_ii, $sbsbdirnames_2nd_ii->[0], $servants_2nd_ii[0]);

## my ($sbdirname_2nd_ii, $sbsbdirnames_2nd_ii) = @{$SUBDIRNAMES[2]};

my $odir_mu_Dcr2Adel = join ('/', $sbdir_out_2nd_ii, $sbsbdirnames_2nd_ii->[0]);

my $ofile_mu_Dcr2Adel = join ('/', $odir_mu_Dcr2Adel, 'sigmaBE_eq_1.txt');
my $log_mu_Dcr2Adel = join ('/', $odir_mu_Dcr2Adel, 'log_sigmaBE_eq_1.txt');


		# The Revision COMPLETED at "Fri Feb 14 18:32:55 JST 2020"!! #
unless ( 0 == system ("(time $bin_perl $svnt_script_mu_Dcr2Adel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 1 $infile_id_prms $infile_mu_Dcr $rf_infiles_mu_Adel->[1] $ofile_mu_Dcr2Adel) > $log_mu_Dcr2Adel 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_Dcr2Adel' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_Dcr2Adel' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_mu_Dcr2Adel .\n\n") if ($TEST_LEVEL > 2); # (Added on 2020/02/18.) #

		# End of "The Revision COMPLETED at "Fri Feb 14 18:32:55 JST 2020"!!" #
=The revision was INCOMPLETE until "Fri Feb 14 18:28:14 JST 2020!!"
unless ( 0 == system ("(time $bin_perl $svnt_script_mu_Dcr2Adel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 1 $infile_id_prms $infile_mu_Dcr $rf_infiles_mu_Adel->[1] $ofile_mu_Dcr2Adel) > $log_mu_A2empty 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_A2empty' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_A2empty' ...\n\n") if ($TEST_LEVEL > 1);
=cut # END of "The revision was INCOMPLETE until "Fri Feb 14 18:28:14 JST 2020"!!"


my $infile_mu_Dcr2Adel = $ofile_mu_Dcr2Adel;


			# (Added on 2020/02/14) #

##print STDOUT "\n\n<<< 2nd_ii, 1: Compute Multiplication Factors for 'D-cr -> AD -> D' >>>\n\n"; # (Added on 2020/02/11) #
print STDOUT "\n\n<<< 2nd_ii, 1 ($NAME_PREPREFINAL): Compute Multiplication Factors for 'D-cr -> AD -> D' >>>\n\n"; # (=> Modified on 2020/02/15) #

my $svnt_script_mu_Dcr2AD2D = join ('/', $dir_servants_2nd_ii, $sbsbdirnames_2nd_ii->[1], $servants_2nd_ii[1]);

## my ($sbdirname_2nd_ii, $sbsbdirnames_2nd_ii) = @{$SUBDIRNAMES[2]};

my $odir_mu_Dcr2AD2D = join ('/', $sbdir_out_2nd_ii, $sbsbdirnames_2nd_ii->[1]);

my $ofile_mu_Dcr2AD2D = join ('/', $odir_mu_Dcr2AD2D, 'sigmaBE_eq_1.txt');
my $log_mu_Dcr2AD2D = join ('/', $odir_mu_Dcr2AD2D, 'log_sigmaBE_eq_1.txt');

##unless ( 0 == system ("(time $bin_perl $svnt_script_mu_Dcr2AD2D $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 1 $infile_id_prms $infile_mu_Dcr2Adel $rf_infiles_muP0->[2] $ofile_mu_Dcr2AD2D) > $log_mu_Dcr2AD2D 2>&1" ) ) {
unless ( 0 == system ("(time $bin_perl $svnt_script_mu_Dcr2AD2D $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_mu_Dcr2Adel $rf_infiles_muP0->[2] $ofile_mu_Dcr2AD2D) > $log_mu_Dcr2AD2D 2>&1" ) ) { # REMOVED the argument, $SIGMA_BE (always = 1), on 2020/02/15. #
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_Dcr2AD2D' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_Dcr2AD2D' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_mu_Dcr2AD2D .\n\n") if ($TEST_LEVEL > 2); # (Added on 2020/02/18.) #

my $infile_mu_Dcr2AD2D = $ofile_mu_Dcr2AD2D;

			# END of "(Added on 2020/02/14)" #

			# (Added on 2020/02/15) #

print STDOUT "\n\n<<< 2nd_ii, 2 ($NAME_PREFINAL): Compute Sub-Total Multiplication Factors for the Class, '2nd_ii'(, divided by the exponentiated total-rate-increment) >>>\n\n";

my $svnt_script_muP_2nd_ii = join ('/', $dir_servants_2nd_ii, $sbsbdirnames_2nd_ii->[2], $servants_2nd_ii[2]);

## my ($sbdirname_2nd_ii, $sbsbdirnames_2nd_ii) = @{$SUBDIRNAMES[2]};

my $odir_muP_2nd_ii = join ('/', $sbdir_out_2nd_ii, $sbsbdirnames_2nd_ii->[2]);

my $ofile_muP_2nd_ii = join ('/', $odir_muP_2nd_ii, 'case_iv_2nd_ii.txt');
my $log_muP_2nd_ii = join ('/', $odir_muP_2nd_ii, 'log_case_iv_2nd_ii.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_muP_2nd_ii $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $rf_infiles_muP0->[2] $infile_mu_Dcr2AD2D $ofile_muP_2nd_ii) > $log_muP_2nd_ii 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_muP_2nd_ii' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_muP_2nd_ii' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_muP_2nd_ii .\n\n") if ($TEST_LEVEL > 2); # (Added on 2020/02/18.) #

my $infile_muP_2nd_ii = $ofile_muP_2nd_ii;

			# END of "(Added on 2020/02/15)" #


	# ['2nd_ii', ['mu_Dcr2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (2nd element)

	# ['cmpt_mu_Dcr2Adel.pl', $NAME_PL_EXT2FIN, $NAME_PL_EXT2INIT], # (2nd element)


	### END of "2nd_ii: Compute the Quantities involved in the Perturbation Class, '2nd_ii'. ###
	### (Writing Started on 2020/02/11, Ended on 2020/02/16.)"



	### Create Sub-Directory System for the Outputs of the Perturbation Class, '3rd_a'. (ADDED on 2020/02/16) ###

print STDOUT "\n<<< Process DR-3rd_a: Creating the Sub-Directory System for the Outputs of the Perturbation Class, '3rd_a' >>>\n\n";

my ($sbdirname_3rd_a, $sbsbdirnames_3rd_a) = @{$SUBDIRNAMES[3]};

my $sbdir_out_3rd_a = "$DIR_OUTPUTS/$sbdirname_3rd_a";

if ($TEST_LEVEL > 5) { # (DR-3rd_a) # 

	print STDOUT "\n";
	print STDOUT '$sbdirname_3rd_a = ', $sbdirname_3rd_a, " .\n\n";
	print STDOUT '@{$sbsbdirnames_3rd_a} = (', join (', ', @{$sbsbdirnames_3rd_a}), ") .\n\n\n";

} # End of "if ($TEST_LEVEL > 5) { # (DR-3rd_a) # ...}"


##unless ( create_dir_rw ($sbdir_out_2nd_ii) ) {  # A BUG fixed on 2020/02/18.
unless ( create_dir_rw ($sbdir_out_3rd_a) ) { 
	print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the sub-directory, \"$sbdir_out_3rd_a\", for the Outputs of the Perturbation Class, '3rd_a': $?\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbdir_out_3rd_a/\" ...\n\n") if ($TEST_LEVEL >2); # (Added on 2020/02/18.) #

for (my $k = 0; $k < @{$sbsbdirnames_3rd_a}; $k++) {

	my $sbsbdir = join ('/', $sbdir_out_3rd_a, $sbsbdirnames_3rd_a->[$k]);

	unless ( create_dir_rw ($sbsbdir) ) { 
	    print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the $k th sub-sub-directory, \"$sbsbdir\", for the Outputs of the Perturbation Class, '3rd_a': $?\n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbsbdir/\" ...\n\n") if ($TEST_LEVEL >2); # (Added on 2020/02/18.) #
}

(print STDOUT "\n ... Finished Creating the Sub-Directory System ...\n\n") if ($TEST_LEVEL > 1);

	### END of "Create Sub-Directory System for the Outputs of the Perturbation Class, '3rd_a'. (ADDED on 2020/02/16)" ###


	### 3rd_a: Compute the Quantities involved in the Perturbation Class, '3rd_a'. ###
	### (Writing Started on 2020/02/16, Ended on 2020/02/18.)

##print STDOUT "\n\n<<<<<  2nd_ii: Compute the Quantities involved in the Perturbation Class, '2nd_ii'  >>>>>\n\n";
print STDOUT "\n\n<<<<<  3rd_a: Compute the Quantities involved in the Perturbation Class, '3rd_a'  >>>>>\n\n"; # Corrected on 2020/02/18.

my @servants_3rd_a = @{$SERVANTS[3]};
my $dir_servants_3rd_a = join ('/', $DIR_SERVANTS, $sbdirname_3rd_a);


if ($TEST_LEVEL > 5) { # (3rd_a) #

	print STDOUT "\n";
	print STDOUT '@servants_3rd_a = (', join (', ', @servants_3rd_a), ") .\n\n\n";

} # End of "if ($TEST_LEVEL > 5) { # (3rd_a) # ...}"



##print STDOUT "\n\n<<< 3rd_a, 0: Compute Multiplication Factors for 'Dcr -> Adel' >>>\n\n"; # (Added on 2020/02/17) #
print STDOUT "\n\n<<< 3rd_a, 0: Compute Multiplication Factors for 'Dcr -> Adel -> Adel' >>>\n\n"; # (=> Corrected on 2020/02/18) #

my $svnt_script_mu_Dcr2Adel2Adel = join ('/', $dir_servants_3rd_a, $sbsbdirnames_3rd_a->[0], $servants_3rd_a[0]);

## my ($sbdirname_3rd_a, $sbsbdirnames_3rd_a) = @{$SUBDIRNAMES[3]};

my $odir_mu_Dcr2Adel2Adel = join ('/', $sbdir_out_3rd_a, $sbsbdirnames_3rd_a->[0]);

			# CORRECTED on 2020/02/18. #
##my $ofile_mu_Dcr2Adel2Adel = join ('/', $odir_mu_Dcr2Adel, 'sigmaBER_eq_sigmaBEL_eq_1.txt');
##my $log_mu_Dcr2Adel2Adel = join ('/', $odir_mu_Dcr2Adel, 'log_sigmaBER_eq_sigmaBEL_eq_1.txt');
my $ofile_mu_Dcr2Adel2Adel = join ('/', $odir_mu_Dcr2Adel2Adel, 'sigmaBER_eq_sigmaBEL_eq_1.txt');
my $log_mu_Dcr2Adel2Adel = join ('/', $odir_mu_Dcr2Adel2Adel, 'log_sigmaBER_eq_sigmaBEL_eq_1.txt');
			# END of "CORRECTED on 2020/02/18." #


unless ( 0 == system ("(time $bin_perl $svnt_script_mu_Dcr2Adel2Adel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 1 1 $infile_id_prms $infile_mu_Dcr2AD2D $rf_infiles_mu_Adel->[1] $ofile_mu_Dcr2Adel2Adel) > $log_mu_Dcr2Adel2Adel 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_Dcr2Adel2Adel' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_Dcr2Adel2Adel' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_mu_Dcr2Adel2Adel .\n\n") if ($TEST_LEVEL > 2); # (Added on 2020/02/18.) #

my $infile_mu_Dcr2Adel2Adel = $ofile_mu_Dcr2Adel2Adel;


			# (Added on 2020/02/18) #

print STDOUT "\n\n<<< 3rd_a, 1 ($NAME_PREPREFINAL): Compute 'Amp.&Ext.-I' Multiplication Factors for the Class, '3rd_a' >>>\n\n"; 

my $svnt_script_mu_AEI_3rd_a = join ('/', $dir_servants_3rd_a, $sbsbdirnames_3rd_a->[1], $servants_3rd_a[1]);

## my ($sbdirname_3rd_a, $sbsbdirnames_3rd_a) = @{$SUBDIRNAMES[3]};

my $odir_mu_AEI_3rd_a = join ('/', $sbdir_out_3rd_a, $sbsbdirnames_3rd_a->[1]);

my $ofile_mu_AEI_3rd_a = join ('/', $odir_mu_AEI_3rd_a, 'mu_AmpExtI.txt');
my $log_mu_AEI_3rd_a = join ('/', $odir_mu_AEI_3rd_a, 'log_mu_AmpExtI.txt');

unless ( 0 == system ("(time $bin_perl $svnt_script_mu_AEI_3rd_a $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $rf_infiles_muP0->[2] $infile_mu_Dcr2Adel2Adel $ofile_mu_AEI_3rd_a) > $log_mu_AEI_3rd_a 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_AEI_3rd_a' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_AEI_3rd_a' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_mu_AEI_3rd_a .\n\n") if ($TEST_LEVEL > 2);

my $infile_mu_AEI_3rd_a = $ofile_mu_AEI_3rd_a;



print STDOUT "\n\n<<< 3rd_a, 2 ($NAME_PREFINAL): Compute Sub-Total Multiplication Factors for the Class, '3rd_a'(, divided by the exponentiated total-rate-increment) >>>\n\n";

my $svnt_script_muP_3rd_a = join ('/', $dir_servants_3rd_a, $sbsbdirnames_3rd_a->[2], $servants_3rd_a[2]);

## my ($sbdirname_3rd_a, $sbsbdirnames_3rd_a) = @{$SUBDIRNAMES[3]};

my $odir_muP_3rd_a = join ('/', $sbdir_out_3rd_a, $sbsbdirnames_3rd_a->[2]);

my $ofile_muP_3rd_a = join ('/', $odir_muP_3rd_a, 'case_iv_3rd_a.txt');
my $log_muP_3rd_a = join ('/', $odir_muP_3rd_a, 'log_case_iv_3rd_a.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_muP_3rd_a $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_mu_AEI_3rd_a $rf_infiles_muP0->[2] $ofile_muP_3rd_a) > $log_muP_3rd_a 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_muP_3rd_a' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_muP_3rd_a' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_muP_3rd_a .\n\n") if ($TEST_LEVEL > 2);

my $infile_muP_3rd_a = $ofile_muP_3rd_a;

			# END of "(Added on 2020/02/18)" #

	# ['3rd_a',  ['mu_Dcr2Adel2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (3rd element)
	# ['cmpt_mu_Dcr2Adel2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (3rd element)

	### END of "3rd_a: Compute the Quantities involved in the Perturbation Class, '3rd_a'. ###
	### (Writing Started on 2020/02/16, Ended on 2020/02/18.)"



	### Create Sub-Directory System for the Outputs of the Perturbation Class, '3rd_c'. (ADDED on 2020/02/19) ###

print STDOUT "\n<<< Process DR-3rd_c: Creating the Sub-Directory System for the Outputs of the Perturbation Class, '3rd_c' >>>\n\n";

my ($sbdirname_3rd_c, $sbsbdirnames_3rd_c) = @{$SUBDIRNAMES[3+1]};

my $sbdir_out_3rd_c = "$DIR_OUTPUTS/$sbdirname_3rd_c";

if ($TEST_LEVEL > 5) { # (DR-3rd_c) # 

	print STDOUT "\n";
	print STDOUT '$sbdirname_3rd_c = ', $sbdirname_3rd_c, " .\n\n";
	print STDOUT '@{$sbsbdirnames_3rd_c} = (', join (', ', @{$sbsbdirnames_3rd_c}), ") .\n\n\n";

} # End of "if ($TEST_LEVEL > 5) { # (DR-3rd_c) # ...}"


unless ( create_dir_rw ($sbdir_out_3rd_c) ) { 
	print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the sub-directory, \"$sbdir_out_3rd_c\", for the Outputs of the Perturbation Class, '3rd_c': $?\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbdir_out_3rd_c/\" ...\n\n") if ($TEST_LEVEL >2);

for (my $k = 0; $k < @{$sbsbdirnames_3rd_c}; $k++) {

	my $sbsbdir = join ('/', $sbdir_out_3rd_c, $sbsbdirnames_3rd_c->[$k]);

	unless ( create_dir_rw ($sbsbdir) ) { 
	    print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the $k th sub-sub-directory, \"$sbsbdir\", for the Outputs of the Perturbation Class, '3rd_c': $?\n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbsbdir/\" ...\n\n") if ($TEST_LEVEL >2);
}

(print STDOUT "\n ... Finished Creating the Sub-Directory System ...\n\n") if ($TEST_LEVEL > 1);

	### END of "Create Sub-Directory System for the Outputs of the Perturbation Class, '3rd_c'. (ADDED on 2020/02/19)" ###


	### 3rd_c: Compute the Quantities involved in the Perturbation Class, '3rd_c'. ###
	### (Writing Started on 2020/02/19, Ended on 2020/02/24.)

print STDOUT "\n\n<<<<<  3rd_c: Compute the Quantities involved in the Perturbation Class, '3rd_c'  >>>>>\n\n";

##my @servants_3rd_c = @{$SERVANTS[3]}; # A BUG fixed on 2020/02/20. #
my @servants_3rd_c = @{$SERVANTS[3+1]};
my $dir_servants_3rd_c = join ('/', $DIR_SERVANTS, $sbdirname_3rd_c);


if ($TEST_LEVEL > 5) { # (3rd_c) #

	print STDOUT "\n";
	print STDOUT '@servants_3rd_c = (', join (', ', @servants_3rd_c), ") .\n\n\n";

} # End of "if ($TEST_LEVEL > 5) { # (3rd_c) # ...}"


print STDOUT "\n\n<<< 3rd_c, 0: Compute Multiplication Factors for 'Dcr(R) -> .Adel' >>>\n\n";

my $svnt_script_mu_DcrR2dotAdel = join ('/', $dir_servants_3rd_c, $sbsbdirnames_3rd_c->[0], $servants_3rd_c[0]);

## my ($sbdirname_3rd_c, $sbsbdirnames_3rd_c) = @{$SUBDIRNAMES[3+1]};

my $odir_mu_DcrR2dotAdel = join ('/', $sbdir_out_3rd_c, $sbsbdirnames_3rd_c->[0]);


 		# (MODIFIED on 2020/02/25.) #
my $pref_ofile_fxd_LDR = 'out_LDR_eq_'; # The Prefix for the output file names; the suffix should always be '.txt'. # MOVED from within '3rd_c, 1' (on 2020/02/25).
my $pref_ofile_mu_DcrR2dotAdel = join ('/', $odir_mu_DcrR2dotAdel, $pref_ofile_fxd_LDR); # The Prefix for the output files; the suffix should always be '.txt'. #
##my $pref_ofile_mu_DcrR2dotAdel = join ('/', $odir_mu_DcrR2dotAdel, 'out_LDR_eq_'); # The Prefix for the output files; the suffix should always be '.txt'. #
 		# End of "(MODIFIED on 2020/02/25.)" #

my $log_mu_DcrR2dotAdel = join ('/', $odir_mu_DcrR2dotAdel, 'log_mu_DcrR2dotAdel.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_mu_DcrR2dotAdel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 0 $infile_id_prms $infile_mu_Dcr $rf_infiles_mu_Adel->[0] $pref_ofile_mu_DcrR2dotAdel) > $log_mu_DcrR2dotAdel 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_DcrR2dotAdel' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_DcrR2dotAdel' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output Files: ] $pref_ofile_mu_DcrR2dotAdel\$LDR.txt   (w/ \$LDR = 1, ..., $UB_CT_SITES_CMPT) .\n\n") if ($TEST_LEVEL > 2);

my $pref_infile_mu_DcrR2dotAdel = $pref_ofile_mu_DcrR2dotAdel;


			# (Added on 2020/02/20.) #

print STDOUT "\n\n<<< 3rd_c, 1: Compute Multiplication Factors for '.A -> .AD -> .Adel' >>>\n\n";

		### NOTE: This is the 1st RATE-LIMITING STEP!! ###

my $svnt_script_mu_dotA2dotAD2dotAdel = join ('/', $dir_servants_3rd_c, $sbsbdirnames_3rd_c->[1], $servants_3rd_c[1]);

## my ($sbdirname_3rd_c, $sbsbdirnames_3rd_c) = @{$SUBDIRNAMES[3+1]};

my $odir_mu_dotA2dotAD2dotAdel = join ('/', $sbdir_out_3rd_c, $sbsbdirnames_3rd_c->[1]);

			# (Modified on 2020/02/21.) (1) #
##my $pref_ofile_fxd_LDR = 'out_LDR_eq_'; # The Prefix for the output file names; the suffix should always be '.txt'. # MOVED to within '3rd_c, 0' (on 2020/02/25).
my $pref_ofile_mu_dotA2dotAD2dotAdel = join ('/', $odir_mu_dotA2dotAD2dotAdel, $pref_ofile_fxd_LDR); # The Prefix for the output file paths; the suffix should always be '.txt'. #
##my $pref_ofile_mu_dotA2dotAD2dotAdel = join ('/', $odir_mu_dotA2dotAD2dotAdel, 'out_LDR_eq_'); # The Prefix for the output files; the suffix should always be '.txt'. #
			# End of "(Modified on 2020/02/21.) (1)" #

my $pref_log_mu_dotA2dotAD2dotAdel = join ('/', $odir_mu_dotA2dotAD2dotAdel, 'log_LDR_eq_'); # The Prefix for the log files; the suffix should always be '.txt'. #

for (my $LDR = 1; $LDR <= $UB_CT_SITES_CMPT; $LDR++) { # Main for-loop (over $LDR). (3rd_c, 1) #

		### NOTE: This Part Could Be Parallelized (or Distributed)!! ###

   (print STDOUT "\n<  \$LDR = $LDR >\n\n") if ($TEST_LEVEL >2);

	my $infile_fxd_LDR = join ('', $pref_infile_mu_DcrR2dotAdel, $LDR, '.txt');
	my $ofile_fxd_LDR = join ('', $pref_ofile_mu_dotA2dotAD2dotAdel, $LDR, '.txt');
	my $logfile_fxd_LDR = join ('', $pref_log_mu_dotA2dotAD2dotAdel, $LDR, '.txt');

	unless ( 0 == system ("(time $bin_perl $svnt_script_mu_dotA2dotAD2dotAdel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $LDR $infile_id_prms $rf_infiles_muP0->[1] $infile_fxd_LDR $ofile_fxd_LDR) > $logfile_fxd_LDR 2>&1" ) ) {
	    print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_dotA2dotAD2dotAdel' for \$LDR = $LDR: $? \n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_dotA2dotAD2dotAdel', for \$LDR = $LDR ...\n\n") if ($TEST_LEVEL > 1);
   (print STDOUT "[ Output File: ] $ofile_fxd_LDR .\n\n") if ($TEST_LEVEL > 2);

} # End of the Main for-loop (over $LDR). (3rd_c, 1) #

print STDOUT "\n <<< '3rd_c, 1' ENDED />/>/>\n\n"; # Added on 2020/02/21. #

			# (Modified on 2020/02/21.) (2) #
my $indir_mu_dotA2dotAD2dotAdel = $odir_mu_dotA2dotAD2dotAdel;
my $pref_infile_fxd_LDR = $pref_ofile_fxd_LDR; # 
##my $pref_infile_mu_dotA2dotAD2dotAdel = $pref_ofile_mu_dotA2dotAD2dotAdel;
			# End of "(Modified on 2020/02/21.) (2)" #


			# End of "(Added on 2020/02/20.)" #


			# (Added on 2020/02/21.) #

print STDOUT "\n\n<<< 3rd_c, 2: SWAP the INDEXES of the Files Storing  Multiplication Factors for '.A -> .AD -> .Adel' >>>\n\n";


my $svnt_script_swap_indexes_A = join ('/', $dir_servants_3rd_c, $sbsbdirnames_3rd_c->[2], $servants_3rd_c[2]);

## my ($sbdirname_3rd_c, $sbsbdirnames_3rd_c) = @{$SUBDIRNAMES[3+1]};

my $odir_mu_dotA2dotAD2dotAdel_SWP_INDX = join ('/', $sbdir_out_3rd_c, $sbsbdirnames_3rd_c->[2]);

my $pref_ofile_fxd_LA = 'out_LA_eq_'; # The Prefix for the output file names; the suffix should always be '.txt'. #
my $pref_ofile_mu_dotA2dotAD2dotAdel_SWP_INDX = join ('/', $odir_mu_dotA2dotAD2dotAdel_SWP_INDX, $pref_ofile_fxd_LA); # The Prefix for the output file paths; the suffix should always be '.txt'. #

my $pref_log_mu_dotA2dotAD2dotAdel_SWP_INDX = join ('/', $odir_mu_dotA2dotAD2dotAdel_SWP_INDX, 'log_LA_eq_'); # The Prefix for the log files; the suffix should always be '.txt'. #


for (my $LA = 1; $LA <= $UB_CT_SITES_CMPT; $LA++) { # Main for-loop (over $LA). (3rd_c, 1) #


   (print STDOUT "\n<  \$LA = $LA >\n\n") if ($TEST_LEVEL >2);

	my $ofile_fxd_LA = join ('', $pref_ofile_mu_dotA2dotAD2dotAdel_SWP_INDX, $LA, '.txt');
	my $logfile_fxd_LA = join ('', $pref_log_mu_dotA2dotAD2dotAdel_SWP_INDX, $LA, '.txt');
	my $label_corner = '#{sites_in_D(R)}_Fin:#{deleted_sites_in_D(L)}\Time_Lapse';
	my $suff_infile = '.txt';

	unless ( 0 == system ("(time $bin_perl $svnt_script_swap_indexes_A '$label_corner' 1 $LA  $indir_mu_dotA2dotAD2dotAdel $pref_infile_fxd_LDR $suff_infile $ofile_fxd_LA) > $logfile_fxd_LA 2>&1" ) ) { # (NOTE) The argument "1" specifies that the 1st index (in the row-label) MUST be swapped with the file-index. #
	    print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_swap_indexes_A' for \$LA = $LA: $? \n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_swap_indexes_A', for \$LA = $LA ...\n\n") if ($TEST_LEVEL > 1);
   (print STDOUT "[ Output File: ] $ofile_fxd_LA .\n\n") if ($TEST_LEVEL > 2);

} # End of the Main for-loop (over $LA). (3rd_c, 2) #

print STDOUT "\n <<< '3rd_c, 2' ENDED />/>/>\n\n";

my $indir_mu_dotA2dotAD2dotAdel_SWP_INDX = $odir_mu_dotA2dotAD2dotAdel_SWP_INDX;
my $pref_infile_fxd_LA = $pref_ofile_fxd_LA; 

			# End of "(Added on 2020/02/21.)" #


			# (Added on 2020/02/22.) #

print STDOUT "\n\n<<< 3rd_c, 3: Compute Multiplication Factors for 'Dcr(L) -> Dcr(R) -> Adel' >>>\n\n";
	# mu_DcrL2DcrR2Adel

my $svnt_script_mu_DcrL2DcrR2Adel = join ('/', $dir_servants_3rd_c, $sbsbdirnames_3rd_c->[3], $servants_3rd_c[3]);

## my ($sbdirname_3rd_c, $sbsbdirnames_3rd_c) = @{$SUBDIRNAMES[3+1]};

my $odir_mu_DcrL2DcrR2Adel = join ('/', $sbdir_out_3rd_c, $sbsbdirnames_3rd_c->[3]);

my $pref_ofile_mu_DcrL2DcrR2Adel_fxd_LA = join ('/', $odir_mu_DcrL2DcrR2Adel, $pref_ofile_fxd_LA); # The Prefix for the output file paths; the suffix should always be '.txt'. #

my $ofile_mu_DcrL2DcrR2Adel_ALL_LAs = join ('/', $odir_mu_DcrL2DcrR2Adel, 'out_ALL_LAs.txt');

my $pref_log_mu_DcrL2DcrR2Adel_fxd_LA = join ('/', $odir_mu_DcrL2DcrR2Adel, 'log_LA_eq_'); # The Prefix for the log files; the suffix should always be '.txt'. #

my $log_mu_DcrL2DcrR2Adel_ALL_LAs = join ('/', $odir_mu_DcrL2DcrR2Adel, 'log_ALL_LAs.txt'); 


for (my $LA = 1; $LA <= $UB_CT_SITES_CMPT; $LA++) { # Main for-loop (over $LA). (3rd_c, 3) #

   (print STDOUT "\n<  \$LA = $LA >\n\n") if ($TEST_LEVEL >2);

	my $infile_fxd_LA = join ('', $indir_mu_dotA2dotAD2dotAdel_SWP_INDX, '/', $pref_infile_fxd_LA, $LA, '.txt');
	my $ofile_fxd_LA = join ('', $pref_ofile_mu_DcrL2DcrR2Adel_fxd_LA, $LA, '.txt');
	my $logfile_fxd_LA = join ('', $pref_log_mu_DcrL2DcrR2Adel_fxd_LA, $LA, '.txt');

	unless ( 0 == system ("(time $bin_perl $svnt_script_mu_DcrL2DcrR2Adel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $LA $infile_id_prms $infile_mu_Dcr $infile_fxd_LA $ofile_fxd_LA) > $logfile_fxd_LA 2>&1" ) ) {
	    print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_DcrL2DcrR2Adel' for \$LA = $LA: $? \n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_DcrL2DcrR2Adel', for \$LA = $LA ...\n\n") if ($TEST_LEVEL > 1);
   (print STDOUT "[ Output File: ] $ofile_fxd_LA .\n\n") if ($TEST_LEVEL > 2);

	if ($LA == 1) {
	    unless ( 0 == system ("(cp $ofile_fxd_LA $ofile_mu_DcrL2DcrR2Adel_ALL_LAs) 2> $log_mu_DcrL2DcrR2Adel_ALL_LAs" ) ) {
	        print STDOUT "[ERROR '$program_name'] FAILED to Copy '$ofile_fxd_LA' to '$ofile_mu_DcrL2DcrR2Adel_ALL_LAs' : $? \n\n";
	        print STDERR "[ERROR '$program_name'] $!\n";
	        die ;
	    }
   (print STDOUT "\n ... Finished Copying '$ofile_fxd_LA' to '$ofile_mu_DcrL2DcrR2Adel_ALL_LAs' ...\n\n") if ($TEST_LEVEL > 1);

	} else  {
	    unless ( 0 == system ("(grep -v '^#' $ofile_fxd_LA >> $ofile_mu_DcrL2DcrR2Adel_ALL_LAs ) 2>> $log_mu_DcrL2DcrR2Adel_ALL_LAs" ) ) {
	        print STDOUT "[ERROR '$program_name'] FAILED to Append (the bulk of) '$ofile_fxd_LA' to '$ofile_mu_DcrL2DcrR2Adel_ALL_LAs' : $? \n\n";
	        print STDERR "[ERROR '$program_name'] $!\n";
	        die ;
	    }
   (print STDOUT "\n ... Finished Appending  (the bulk of) '$ofile_fxd_LA' to '$ofile_mu_DcrL2DcrR2Adel_ALL_LAs' ...\n\n") if ($TEST_LEVEL > 1);

	} # End of "if ($LA == 1) {...} else {...}".


			# (Added on 2020/02/29.) (A) #

	unless ( 0 == system ( "(rm -f $ofile_fxd_LA) 2>> $log_mu_DcrL2DcrR2Adel_ALL_LAs") ) {
	        print STDOUT "[ERROR '$program_name'] FAILED to Remove '$ofile_fxd_LA' : $? \n\n";
	        print STDERR "[ERROR '$program_name'] $!\n";
	        die ;
	}
   (print STDOUT "\n ... Finished Removing '$ofile_fxd_LA' ...\n\n") if ($TEST_LEVEL > 1);
			# End of (Added on 2020/02/29.) (A) #

} # End of the Main for-loop (over $LA). (3rd_c, 3) #

print STDOUT "\n <<< '3rd_c, 3' ENDED (w/ output file containing ALL results: \"$ofile_mu_DcrL2DcrR2Adel_ALL_LAs\") />/>/>\n\n";

my $indir_mu_DcrL2DcrR2Adel = $odir_mu_DcrL2DcrR2Adel;
my $infile_mu_DcrL2DcrR2Adel_ALL_LAs = $ofile_mu_DcrL2DcrR2Adel_ALL_LAs;

			# End of "(Added on 2020/02/22.)" #


			# (Added on 2020/02/24) (1) #

print STDOUT "\n\n<<< 3rd_c, 3+1 ($NAME_PREPREFINAL): Compute 'Amp.&Ext.-I' Multiplication Factors for the Class, '3rd_c' >>>\n\n"; 

my $svnt_script_mu_AEI_3rd_c = join ('/', $dir_servants_3rd_c, $sbsbdirnames_3rd_c->[3+1], $servants_3rd_c[3+1]);

## my ($sbdirname_3rd_c, $sbsbdirnames_3rd_c) = @{$SUBDIRNAMES[3+1]};

my $odir_mu_AEI_3rd_c = join ('/', $sbdir_out_3rd_c, $sbsbdirnames_3rd_c->[3+1]);

my $ofile_mu_AEI_3rd_c = join ('/', $odir_mu_AEI_3rd_c, 'mu_AmpExtI.txt');
my $log_mu_AEI_3rd_c = join ('/', $odir_mu_AEI_3rd_c, 'log_mu_AmpExtI.txt');

unless ( 0 == system ("(time $bin_perl $svnt_script_mu_AEI_3rd_c $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $rf_infiles_muP0->[2] $infile_mu_DcrL2DcrR2Adel_ALL_LAs $ofile_mu_AEI_3rd_c) > $log_mu_AEI_3rd_c 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_AEI_3rd_c' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_AEI_3rd_c' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_mu_AEI_3rd_c .\n\n") if ($TEST_LEVEL > 2);

my $infile_mu_AEI_3rd_c = $ofile_mu_AEI_3rd_c;


print STDOUT "\n\n<<< 3rd_c, 5 ($NAME_PREFINAL): Compute Sub-Total Multiplication Factors for the Class, '3rd_c'(, divided by the exponentiated total-rate-increment) >>>\n\n";

my $svnt_script_muP_3rd_c = join ('/', $dir_servants_3rd_c, $sbsbdirnames_3rd_c->[5], $servants_3rd_c[5]);

## my ($sbdirname_3rd_c, $sbsbdirnames_3rd_c) = @{$SUBDIRNAMES[3+1]};

my $odir_muP_3rd_c = join ('/', $sbdir_out_3rd_c, $sbsbdirnames_3rd_c->[5]);

my $ofile_muP_3rd_c = join ('/', $odir_muP_3rd_c, 'case_iv_3rd_c.txt');
my $log_muP_3rd_c = join ('/', $odir_muP_3rd_c, 'log_case_iv_3rd_c.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_muP_3rd_c $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_mu_AEI_3rd_c $rf_infiles_muP0->[2] $ofile_muP_3rd_c) > $log_muP_3rd_c 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_muP_3rd_c' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_muP_3rd_c' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_muP_3rd_c .\n\n") if ($TEST_LEVEL > 2);

my $infile_muP_3rd_c = $ofile_muP_3rd_c;

			# END of "(Added on 2020/02/24) (1)" #


	## ['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_dotA2dotAD2dotAdel_INDX_SWPD', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL1) #
	## ['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'swap_indexes_A.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) # MODIFIED on 2020/02/20. (GL2) #
	## (  ['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_DcrL2DcrR2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # ((3+1)th element) ) ... OLD VERSION!!
	## (  ['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # ((3+1)th element) ) ... OLD VERSION!!
	##
	## (  ['3rd_c',  ['mu_DcrR2dotAdel', 'mu_dotA2dotAD2dotAdel', 'mu_DcrL2DcrR2Adel', 'mu_A2DA2DAD2Adel', $NAME_PREFINAL]], # ((3+1)th element) ) ... OLD-OLD VERSION!!
	## (  ['cmpt_mu_DcrR2dotAdel.pl', 'cmpt_mu_dotA2dotAD2dotAdel.pl', 'cmpt_mu_DcrL2DcrR2Adel.pl', 'cmpt_mu_A2DA2DAD2Adel.pl', 'cmpt_muP_3rd_c.pl'], # ((3+1)th element) ) ... OLD-OLD VERSION!!

	### END of "3rd_c: Compute the Quantities involved in the Perturbation Class, '3rd_c'. ###
	### (Writing Started on 2020/02/19, Ended on 2020/02/24.)"


	### Create Sub-Directory System for the Outputs of the Perturbation Class, '3rd_e'. (ADDED on 2020/02/24) ###

print STDOUT "\n<<< Process DR-3rd_e: Creating the Sub-Directory System for the Outputs of the Perturbation Class, '3rd_e' >>>\n\n";

my ($sbdirname_3rd_e, $sbsbdirnames_3rd_e) = @{$SUBDIRNAMES[5]};

my $sbdir_out_3rd_e = "$DIR_OUTPUTS/$sbdirname_3rd_e";

if ($TEST_LEVEL > 5) { # (DR-3rd_e) # 

	print STDOUT "\n";
	print STDOUT '$sbdirname_3rd_e = ', $sbdirname_3rd_e, " .\n\n";
	print STDOUT '@{$sbsbdirnames_3rd_e} = (', join (', ', @{$sbsbdirnames_3rd_e}), ") .\n\n\n";

} # End of "if ($TEST_LEVEL > 5) { # (DR-3rd_e) # ...}"


unless ( create_dir_rw ($sbdir_out_3rd_e) ) { 
	print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the sub-directory, \"$sbdir_out_3rd_e\", for the Outputs of the Perturbation Class, '3rd_e': $?\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbdir_out_3rd_e/\" ...\n\n") if ($TEST_LEVEL >2);

for (my $k = 0; $k < @{$sbsbdirnames_3rd_e}; $k++) {

	my $sbsbdir = join ('/', $sbdir_out_3rd_e, $sbsbdirnames_3rd_e->[$k]);

	unless ( create_dir_rw ($sbsbdir) ) { 
	    print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the $k th sub-sub-directory, \"$sbsbdir\", for the Outputs of the Perturbation Class, '3rd_e': $?\n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbsbdir/\" ...\n\n") if ($TEST_LEVEL >2);
}

(print STDOUT "\n ... Finished Creating the Sub-Directory System ...\n\n") if ($TEST_LEVEL > 1);

	### END of "Create Sub-Directory System for the Outputs of the Perturbation Class, '3rd_e'. (ADDED on 2020/02/24)" ###


	### 3rd_e: Compute the Quantities involved in the Perturbation Class, '3rd_e'. ###
	### (Writing Started on 2020/02/24, Ended on 2020/02/29.)

print STDOUT "\n\n<<<<<  3rd_e: Compute the Quantities involved in the Perturbation Class, '3rd_e'  >>>>>\n\n";

my @servants_3rd_e = @{$SERVANTS[5]};
my $dir_servants_3rd_e = join ('/', $DIR_SERVANTS, $sbdirname_3rd_e);


if ($TEST_LEVEL > 5) { # (3rd_e) #

	print STDOUT "\n";
	print STDOUT '@servants_3rd_e = (', join (', ', @servants_3rd_e), ") .\n\n\n";

} # End of "if ($TEST_LEVEL > 5) { # (3rd_e) # ...}"


print STDOUT "\n\n<<< 3rd_e, 0: Compute Multiplication Factors for 'Dcr -> Ber.' >>>\n\n";

my $svnt_script_mu_Dcr2Berdot = join ('/', $dir_servants_3rd_e, $sbsbdirnames_3rd_e->[0], $servants_3rd_e[0]);

## my ($sbdirname_3rd_e, $sbsbdirnames_3rd_e) = @{$SUBDIRNAMES[5]};

my $odir_mu_Dcr2Berdot = join ('/', $sbdir_out_3rd_e, $sbsbdirnames_3rd_e->[0]);
 
my $ofile_mu_Dcr2Berdot = join ('/', $odir_mu_Dcr2Berdot, 'mu_Dcr2Berdot.txt'); 
##my $log_mu_DcrR2dotAdel = join ('/', $odir_mu_Dcr2Berdot, 'log_mu_Dcr2Berdot.txt');
my $log_mu_Dcr2Berdot = join ('/', $odir_mu_Dcr2Berdot, 'log_mu_Dcr2Berdot.txt'); # CORRECTED on 2020/02/25.


##unless ( 0 == system ("(time $bin_perl $svnt_script_mu_DcrR2dotAdel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_id_rates $infile_mu_Dcr $ofile_mu_Dcr2Berdot  --RAW) > $log_mu_Dcr2Berdot 2>&1" ) ) { # A BUG fixed on 2020/02/25. #
unless ( 0 == system ("(time $bin_perl $svnt_script_mu_Dcr2Berdot $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_id_rates $infile_mu_Dcr $ofile_mu_Dcr2Berdot  --RAW) > $log_mu_Dcr2Berdot 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_Dcr2Berdot' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_Dcr2Berdot' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_mu_Dcr2Berdot .\n\n") if ($TEST_LEVEL > 2);

##my $pref_infile_mu_DcrR2dotAdel = $pref_ofile_mu_DcrR2dotAdel;
my $infile_mu_Dcr2Berdot = $ofile_mu_Dcr2Berdot; # CORRECTED on 2020/02/25.


		# (Added on 2020/02/25.) #

print STDOUT "\n\n<<< 3rd_e, 1: Compute Multiplication Factors for '(A ->) Dcr(L) -> Ber' >>>\n\n";

my $svnt_script_mu_pA2pDcrL2Ber = join ('/', $dir_servants_3rd_e, $sbsbdirnames_3rd_e->[1], $servants_3rd_e[1]);

## my ($sbdirname_3rd_e, $sbsbdirnames_3rd_e) = @{$SUBDIRNAMES[5]};

my $odir_mu_pA2pDcrL2Ber = join ('/', $sbdir_out_3rd_e, $sbsbdirnames_3rd_e->[1]);

my $pref_ofile_fxd_LAI = 'out_LAI_eq_'; # The Prefix for the output file names; the suffix should always be '.txt'. #
my $pref_ofile_mu_pA2pDcrL2Ber = join ('/', $odir_mu_pA2pDcrL2Ber, $pref_ofile_fxd_LAI); # The Prefix for the output files; the suffix should always be '.txt'. #

my $log_mu_pA2pDcrL2Ber = join ('/', $odir_mu_pA2pDcrL2Ber, 'log_mu_pA2pDcrL2Ber.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_mu_pA2pDcrL2Ber $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 1 $infile_id_prms $infile_mu_Dcr2Berdot $rf_infiles_muP0->[1] $pref_ofile_mu_pA2pDcrL2Ber) > $log_mu_pA2pDcrL2Ber 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_pA2pDcrL2Ber' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_pA2pDcrL2Ber' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output Files: ] $pref_ofile_mu_pA2pDcrL2Ber\$LAI.txt   (w/ \$LAI = 1, ..., $UB_CT_SITES_CMPT) .\n\n") if ($TEST_LEVEL > 2);

my $pref_infile_mu_pA2pDcrL2Ber = $pref_ofile_mu_pA2pDcrL2Ber;

		# End of "(Added on 2020/02/25.)" #


		# (Added on 2020/02/27.) (1) #

print STDOUT "\n\n<<< 3rd_e, 2: Compute Multiplication Factors for 'DA -> Adel' >>>\n\n";

my $svnt_script_mu_DA2Adel = join ('/', $dir_servants_3rd_e, $sbsbdirnames_3rd_e->[2], $servants_3rd_e[2]);

## my ($sbdirname_3rd_e, $sbsbdirnames_3rd_e) = @{$SUBDIRNAMES[5]};

my $odir_mu_DA2Adel = join ('/', $sbdir_out_3rd_e, $sbsbdirnames_3rd_e->[2]);

my $pref_ofile_fxd_LDF = 'out_LDF_eq_'; # The Prefix for the output file names; the suffix should always be '.txt'. #
my $pref_ofile_mu_DA2Adel = join ('/', $odir_mu_DA2Adel, $pref_ofile_fxd_LDF); # The Prefix for the output files; the suffix should always be '.txt'. #

my $log_mu_DA2Adel = join ('/', $odir_mu_DA2Adel, 'log_mu_DA2Adel.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_mu_DA2Adel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 1 $infile_id_prms $rf_infiles_mu_Adel->[1] $rf_infiles_muP0->[2] $pref_ofile_mu_DA2Adel) > $log_mu_DA2Adel 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_DA2Adel' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_DA2Adel' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output Files: ] $pref_ofile_mu_DA2Adel\$LDF.txt   (w/ \$LDF = 1, ..., $UB_CT_SITES_CMPT) .\n\n") if ($TEST_LEVEL > 2);

my $pref_infile_mu_DA2Adel = $pref_ofile_mu_DA2Adel;

		# End of "(Added on 2020/02/27.) (1)" #

		# (Added on 2020/02/27.) (2) #

print STDOUT "\n\n<<< 3rd_e, 3: Compute Multiplication Factors for '(A ->) Dcr(L) -> Ber -> Adel' >>>\n\n";

		### NOTE: This is the 2nd RATE-LIMITING STEP!! ###

	# mu_pA2pDcrL2Ber2Adel
	# cmpt_mu_pA2pDcrL2Ber2Adel.pl

my $svnt_script_mu_pA2pDcrL2Ber2Adel = join ('/', $dir_servants_3rd_e, $sbsbdirnames_3rd_e->[3], $servants_3rd_e[3]);

## my ($sbdirname_3rd_e, $sbsbdirnames_3rd_e) = @{$SUBDIRNAMES[5]};

my $odir_mu_pA2pDcrL2Ber2Adel = join ('/', $sbdir_out_3rd_e, $sbsbdirnames_3rd_e->[3]);

my $pref_ofile_fxd_LAI_LDF = 'out_LAI_LDF_eq_'; # The Prefix for the output file names; the suffix should always be '.txt'. #
my $pref_ofile_mu_pA2pDcrL2Ber2Adel_fxd_LAI_LDF = join ('/', $odir_mu_pA2pDcrL2Ber2Adel, $pref_ofile_fxd_LAI_LDF); # The Prefix for the output file paths; the suffix should always be '.txt'. #

my $ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs = join ('/', $odir_mu_pA2pDcrL2Ber2Adel, 'out_ALL_LAIs_LDFs.txt');

##my $pref_log_mu_pA2pDcrL2Ber2Adel_fxd_LAI_LDF = join ('/', $odir_mu_pA2pDcrL2Ber2Adel, 'log_LAI_LDF_eq_'); # The Prefix for the log files; the suffix should always be '.txt'. #
my $log_mu_pA2pDcrL2Ber2Adel_fxd_LAI_LDF = join ('/', $odir_mu_pA2pDcrL2Ber2Adel, 'log_fxd_LAI_LDF.txt');  # MODIFIED in REVISION (on 2020/03/02). (1) #

my $log_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs = join ('/', $odir_mu_pA2pDcrL2Ber2Adel, 'log_ALL_LAIs_LDFs.txt'); 


for (my $LAI = 1; $LAI <= $UB_CT_SITES_CMPT; $LAI++) { # Main Outer for-loop (over $LAI). (3rd_e, 3) #

	my $infile_mu_pA2pDcrL2Ber_fxd_LAI = join ('', $pref_infile_mu_pA2pDcrL2Ber , $LAI, '.txt');

    for (my $LDF = 1; $LDF <= $UB_CT_SITES_CMPT; $LDF++) { # Main Inner for-loop (over $LDF).

		### NOTE: This Part Could Be Parallelized (or Distributed)!! ###

   (print STDOUT "\n<  (\$LAI, \$LDF) = ($LAI, $LDF) >\n\n") if ($TEST_LEVEL >2);

	my $infile_mu_DA2Adel_fxd_LDF =  join ('', $pref_infile_mu_DA2Adel, $LDF, '.txt');


	my $ofile_fxd_LAI_LDF = join ('', $pref_ofile_mu_pA2pDcrL2Ber2Adel_fxd_LAI_LDF, $LAI, '_', $LDF, '.txt');
	##my $logfile_fxd_LAI_LDF = join ('', $pref_log_mu_pA2pDcrL2Ber2Adel_fxd_LAI_LDF, $LAI, '_', $LDF, '.txt');
	my $logfile_fxd_LAI_LDF = $log_mu_pA2pDcrL2Ber2Adel_fxd_LAI_LDF; ## MODIFIED in REVISION (on 2020/03/02). (2) #

	##unless ( 0 == system ("(time $bin_perl $svnt_script_mu_pA2pDcrL2Ber2Adel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $LAI $LDF $infile_id_prms $infile_mu_pA2pDcrL2Ber_fxd_LAI $infile_mu_DA2Adel_fxd_LDF  $ofile_fxd_LAI_LDF) > $logfile_fxd_LAI_LDF 2>&1" ) ) {
	unless ( 0 == system ("(time $bin_perl $svnt_script_mu_pA2pDcrL2Ber2Adel $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $LAI $LDF $infile_id_prms $infile_mu_pA2pDcrL2Ber_fxd_LAI $infile_mu_DA2Adel_fxd_LDF  $ofile_fxd_LAI_LDF) >> $logfile_fxd_LAI_LDF 2>&1" ) ) { # MODIFIED in REVISION (on 2020/03/02). (3) #
	    print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_pA2pDcrL2Ber2Adel' for (\$LAI, \$LDF) = ($LAI, $LDF): $? \n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_pA2pDcrL2Ber2Adel', for (\$LAI, \$LDF) = ($LAI, $LDF) ...\n\n") if ($TEST_LEVEL > 1);
   (print STDOUT "[ Output File: ] $ofile_fxd_LAI_LDF .\n\n") if ($TEST_LEVEL > 2);

	if (($LAI == 1) and ($LDF == 1)) {
	    unless ( 0 == system ("(cp $ofile_fxd_LAI_LDF $ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs) 2> $log_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs" ) ) {
	        print STDOUT "[ERROR '$program_name'] FAILED to Copy '$ofile_fxd_LAI_LDF' to '$ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs' : $? \n\n";
	        print STDERR "[ERROR '$program_name'] $!\n";
	        die ;
	    }
   (print STDOUT "\n ... Finished Copying '$ofile_fxd_LAI_LDF' to '$ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs' ...\n\n") if ($TEST_LEVEL > 1);

	} else  {
	    ##unless ( 0 == system ("(grep -v '^#' $ofile_fxd_LAI_LDF | grep -v '^$' >> $ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs ) 2>> $log_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs" ) ) { # A BUG fixed on 2020/02/28. (NOTE: In Perl, the "$'" is short for a special variable, $POSTMATCHm.)
	    unless ( 0 == system ("(grep -v '^#' $ofile_fxd_LAI_LDF | grep -v '^\$' >> $ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs ) 2>> $log_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs" ) ) {
	        print STDOUT "[ERROR '$program_name'] FAILED to Append (the bulk of) '$ofile_fxd_LAI_LDF' to '$ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs' : $? \n\n";
	        print STDERR "[ERROR '$program_name'] $!\n";
	        die ;
	    }
   (print STDOUT "\n ... Finished Appending  (the bulk of) '$ofile_fxd_LAI_LDF' to '$ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs' ...\n\n") if ($TEST_LEVEL > 1);

	} # End of "if (($LAI == 1) and ($LDF == 1)) {...} else {...}".

			# (Added on 2020/02/29.) (B) #

	unless ( 0 == system ( "(rm -f $ofile_fxd_LAI_LDF) 2>> $log_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs ") ) {
	        print STDOUT "[ERROR '$program_name'] FAILED to Remove '$ofile_fxd_LAI_LDF' : $? \n\n";
	        print STDERR "[ERROR '$program_name'] $!\n";
	        die ;
	}
   (print STDOUT "\n ... Finished Removing '$ofile_fxd_LAI_LDF' ...\n\n") if ($TEST_LEVEL > 1);
			# End of (Added on 2020/02/29.) (B) #

    } # End of the Main Inner for-loop (over $LDF).

} # End of the Main Outer for-loop (over $LAI). (3rd_e, 3) #

print STDOUT "\n <<< '3rd_e, 3' ENDED (w/ output file containing ALL results: \"$ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs\") />/>/>\n\n";

my $indir_mu_pA2pDcrL2Ber2Adel = $odir_mu_pA2pDcrL2Ber2Adel;
my $infile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs = $ofile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs;

		# End of "(Added on 2020/02/27.) (2)" #


			# (Added on 2020/02/29) (C) #

print STDOUT "\n\n<<< 3rd_e, 3+1 ($NAME_PREPREFINAL): Compute 'Amp.&Ext.-I' Multiplication Factors for the Class, '3rd_e' >>>\n\n"; 

my $svnt_script_mu_AEI_3rd_e = join ('/', $dir_servants_3rd_e, $sbsbdirnames_3rd_e->[3+1], $servants_3rd_e[3+1]);

## my ($sbdirname_3rd_e, $sbsbdirnames_3rd_e) = @{$SUBDIRNAMES[5]};

my $odir_mu_AEI_3rd_e = join ('/', $sbdir_out_3rd_e, $sbsbdirnames_3rd_e->[3+1]);

my $ofile_mu_AEI_3rd_e = join ('/', $odir_mu_AEI_3rd_e, 'mu_AmpExtI.txt');
my $log_mu_AEI_3rd_e = join ('/', $odir_mu_AEI_3rd_e, 'log_mu_AmpExtI.txt');

unless ( 0 == system ("(time $bin_perl $svnt_script_mu_AEI_3rd_e $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $rf_infiles_muP0->[2] $infile_mu_pA2pDcrL2Ber2Adel_ALL_LAIs_LDFs $ofile_mu_AEI_3rd_e) > $log_mu_AEI_3rd_e 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_mu_AEI_3rd_e' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_mu_AEI_3rd_e' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_mu_AEI_3rd_e .\n\n") if ($TEST_LEVEL > 2);

my $infile_mu_AEI_3rd_e = $ofile_mu_AEI_3rd_e;


print STDOUT "\n\n<<< 3rd_e, 5 ($NAME_PREFINAL): Compute Sub-Total Multiplication Factors for the Class, '3rd_e'(, divided by the exponentiated total-rate-increment) >>>\n\n";

my $svnt_script_muP_3rd_e = join ('/', $dir_servants_3rd_e, $sbsbdirnames_3rd_e->[5], $servants_3rd_e[5]);

## my ($sbdirname_3rd_e, $sbsbdirnames_3rd_e) = @{$SUBDIRNAMES[5]};

my $odir_muP_3rd_e = join ('/', $sbdir_out_3rd_e, $sbsbdirnames_3rd_e->[5]);

my $ofile_muP_3rd_e = join ('/', $odir_muP_3rd_e, 'case_iv_3rd_e.txt');
my $log_muP_3rd_e = join ('/', $odir_muP_3rd_e, 'log_case_iv_3rd_e.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_muP_3rd_e $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT $infile_id_prms $infile_mu_AEI_3rd_e $rf_infiles_muP0->[2] $ofile_muP_3rd_e) > $log_muP_3rd_e 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_muP_3rd_e' : $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_muP_3rd_e' ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_muP_3rd_e .\n\n") if ($TEST_LEVEL > 2);

my $infile_muP_3rd_e = $ofile_muP_3rd_e;

			# END of "(Added on 2020/02/29) (C)" #


	## ['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber', 'mu_DA2Adel', 'mu_pA2pDcrL2Ber2Adel', $NAME_PREPREFINAL, $NAME_PREFINAL]], # (5th element)
	## ['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl', 'cmpt_mu_DA2Adel.pl', 'cmpt_mu_pA2pDcrL2Ber2Adel.pl', $NAME_PL_EXT2INIT, $NAME_PL_EXT2FIN], # (5th element)
	##
	## ( ['3rd_e',  ['mu_Dcr2Berdot', 'mu_pA2pDcrL2Ber', 'mu_DA2Adel', 'mu_pA2pDcrL2Ber2Adel', 'mu_A2DA2Ber2Adel', $NAME_PREFINAL]], # (5th element) ) ... OLD
	## ( ['cmpt_mu_Dcr2Berdot.pl', 'cmpt_mu_pA2pDcrL2Ber.pl', 'cmpt_mu_DA2Adel.pl', 'cmpt_mu_pA2pDcrL2Ber2Adel.pl', 'cmpt_mu_A2DA2Ber2Adel.pl', 'cmpt_mu_3rd_e.pl'], # (5th element) ) ... OLD

	### END of "3rd_e: Compute the Quantities involved in the Perturbation Class, '3rd_e'. ###
	### (Writing Started on 2020/02/24, Ended on 2020/02/29.)"



	### Create Sub-Directory System for the Outputs of 'Summations'. (ADDED on 2020/02/29) ###

print STDOUT "\n<<< Process DR-Sum: Creating the Sub-Directory System for the Outputs of 'Summations' >>>\n\n";

my ($sbdirname_sum, $sbsbdirnames_sum) = @{$SUBDIRNAMES[6]};

my $sbdir_out_sum = "$DIR_OUTPUTS/$sbdirname_sum";

if ($TEST_LEVEL > 5) { # (DR-Sum) # 

	print STDOUT "\n";
	print STDOUT '$sbdirname_sum = ', $sbdirname_sum, " .\n\n";
	print STDOUT '@{$sbsbdirnames_sum} = (', join (', ', @{$sbsbdirnames_sum}), ") .\n\n\n";

} # End of "if ($TEST_LEVEL > 5) { # (DR-Sum) # ...}"


unless ( create_dir_rw ($sbdir_out_sum) ) { 
	print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the sub-directory, \"$sbdir_out_sum\", for the Outputs of 'Summations': $?\n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbdir_out_sum/\" ...\n\n") if ($TEST_LEVEL >2);

for (my $k = 0; $k < @{$sbsbdirnames_sum}; $k++) {

	my $sbsbdir = join ('/', $sbdir_out_sum, $sbsbdirnames_sum->[$k]);

	unless ( create_dir_rw ($sbsbdir) ) { 
	    print STDOUT "[ERROR '$program_name'] The subroutine, 'create_dir_rw', FAILED on the $k th sub-sub-directory, \"$sbsbdir\", for the Outputs of 'Summations': $?\n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
   (print STDOUT "\n ... The subroutine, 'create_dir_rw', Ended on \"$sbsbdir/\" ...\n\n") if ($TEST_LEVEL >2);
}

(print STDOUT "\n ... Finished Creating the Sub-Directory System ...\n\n") if ($TEST_LEVEL > 1);

	### END of "Create Sub-Directory System for the Outputs of 'Summations'. (ADDED on 2020/02/29)" ###


	### Summations: Compute the weighted summations of the PreFinal multiplication factors computed thus far.
	### (Writing Started on 2020/02/29, Ended on 2020/03/01.)

print STDOUT "\n\n<<<<<  Summations: Compute the weighted summations of the PreFinal multiplication factors computed thus far  >>>>>\n\n";

##my @servants_sum = @{$SERVANTS[5]}; # A BUG fixed on 2020/03/01. #
my @servants_sum = @{$SERVANTS[6]};
my $dir_servants_sum = join ('/', $DIR_SERVANTS, $sbdirname_sum);

## my ($sbdirname_sum, $sbsbdirnames_sum) = @{$SUBDIRNAMES[6]};

my $svnt_script_sum = join ('/', $dir_servants_sum, $sbsbdirnames_sum->[0], $servants_sum[0]);

my $odir_sum = join ('/', $sbdir_out_sum, $sbsbdirnames_sum->[0]);


if ($TEST_LEVEL > 5) { # (Summations) #

	print STDOUT "\n";
	print STDOUT '@servants_sum = (', join (', ', @servants_sum), ") .\n\n";

	print STDOUT '$svnt_script_sum = ', "$svnt_script_sum .\n\n";

	print STDOUT '$odir_sum = ', "$odir_sum .\n\n\n";

} # End of "if ($TEST_LEVEL > 5) { # (Summations) # ...}"


print STDOUT "\n\n<<< Summations, 2nd ($NAME_PREFINAL): Compute Summations of $NAME_PREFINAL Multiplication Factors in the 2nd Perturbation Level >>>\n\n";

##my $ofile_muP_3rd_e = join ('/', $odir_muP_3rd_e, 'case_iv_3rd_e.txt');
##my $log_muP_3rd_e = join ('/', $odir_muP_3rd_e, 'log_case_iv_3rd_e.txt');

my $ofile_sum_2nd = join ('/', $odir_sum, 'case_iv_2nd.txt'); 
my $log_sum_2nd = join ('/', $odir_sum, 'log_case_iv_2nd.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_sum $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 1,2 $infile_muP_2nd_i $infile_muP_2nd_ii $ofile_sum_2nd) > $log_sum_2nd 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_sum' , for the 2nd perturbation level: $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_sum', for the 2nd perturbation level ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_sum_2nd .\n\n") if ($TEST_LEVEL > 2);

my $infile_sum_2nd = $ofile_sum_2nd;


			# (Added on 2020/03/01) (1) #

print STDOUT "\n\n<<< Summations, 3rd ($NAME_PREFINAL): Compute Summations of $NAME_PREFINAL Multiplication Factors in the 3rd Perturbation Level >>>\n\n";

my $ofile_sum_3rd = join ('/', $odir_sum, 'case_iv_3rd.txt'); 
my $log_sum_3rd = join ('/', $odir_sum, 'log_case_iv_3rd.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_sum $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 2,2,2 $infile_muP_3rd_a $infile_muP_3rd_c $infile_muP_3rd_e $ofile_sum_3rd) > $log_sum_3rd 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_sum' , for the 3rd perturbation level: $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_sum', for the 3rd perturbation level ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_sum_3rd .\n\n") if ($TEST_LEVEL > 2);

my $infile_sum_3rd = $ofile_sum_3rd;

print STDOUT "\n\n<<< Summations, 2nd+3rd ($NAME_PREFINAL): Compute Summations of $NAME_PREFINAL Multiplication Factors in the 2nd \& 3rd Perturbation Levels >>>\n\n";

my $ofile_sum_2nd_3rd = join ('/', $odir_sum, 'case_iv_2nd_plus_3rd.txt'); 
my $log_sum_2nd_3rd = join ('/', $odir_sum, 'log_case_iv_2nd_plus_3rd.txt');


unless ( 0 == system ("(time $bin_perl $svnt_script_sum $TIME_INIT $TIME_FIN $CT_SB_INTVLS $UB_CT_SITES_CMPT 1,1 $infile_sum_2nd $infile_sum_3rd $ofile_sum_2nd_3rd) > $log_sum_2nd_3rd 2>&1" ) ) {
	print STDOUT "[ERROR '$program_name'] FAILED to run the Perl script, '$svnt_script_sum' , for the 2nd \& 3rd perturbation levels: $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... Finished Running the Perl script, '$svnt_script_sum', for the 2nd \& 3rd perturbation levels ...\n\n") if ($TEST_LEVEL > 1);
(print STDOUT "[ Output File: ] $ofile_sum_2nd_3rd .\n\n") if ($TEST_LEVEL > 2);

##my $infile_sum_2nd_3rd = $ofile_sum_2nd_3rd;
my $infile_muP_case_iv = $ofile_sum_2nd_3rd; # MODIFIED on 2020/03/01. #

			# END of "(Added on 2020/03/01) (1)" #

	## ['Summations', [$NAME_PREFINAL]], # (6th element) # Newly ADDED on 2020/02/29. (GL1) #
	## ['wgtd_summs_3Darrays_for_mfacs.pl'], # (6th element) # Newly ADDED on 2020/02/29. (GL2) #

	### END of "Summations: Compute the weighted summations of the PreFinal multiplication factors computed thus far.
	### (Writing Started on 2020/02/29, Ended on 2020/03/01.)"



	### FINAL: Compute & Output the log-multiplication factors for cases (i)-(iv). ###
	### (Writing Started on 2020/03/01, Ended on 2020/03/dd.)

print STDOUT "\n\n<<<<<  FINAL: Compute & Output the log-multiplication factors for cases (i)-(iv) >>>>>\n\n";

local $/ = "\n"; # Use the newline character as the input-record-separator.

my $ct_tlapses_cmpt = @TIME_LAPSES_CMPT;

print STDOUT "\n\n<<< Final, 1: Output log-factors for case (i)  >>>\n\n";

my $fh_in1 = open_input_file2 ($infile_muP_case_i);
unless (defined $fh_in1) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'open_input_file2', FAILED to open a file-handle for the file \"$infile_muP_case_i\": $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
 
my $fh_out1 = open_output_file ($out_nosites, 0);
unless (defined $fh_out1) {
	print STDOUT "[ERROR '$program_name'] The subroutine 'open_output_file' FAILED to open a file-handle for the file \"$out_nosites\": $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}

print $fh_out1 join ("\t", '#{Anc_sites}:#{dec_sites}\E[#{indels}]', @E_INDEL_CTS_OUT), "\n\n";

while (my $line = <$fh_in1>) { # 1st Main outer while-loop (over input lines). #

	unless ($line =~ m/^0\:0\t/) { next; }

	$line =~ s/\s+$//;

	my @in_flds = split /\t/, $line;
	my $row_label = shift @in_flds;

	unless (@in_flds == $ct_tlapses_cmpt) {
	    my $ct_flds = @in_flds;
	    print STDOUT "[WARNING '$program_name'] ILLEGAL INPUT: The row labelled '$row_label' has $ct_flds fields (excluding the label), while it SHOULD HAVE $ct_tlapses_cmpt fields: $! !! ... I will SKIP this row ...\n\n";
	    next;
	}

	my @out_flds = ();

	foreach my $k (@SMPL_TIME_POINTS) { 
	    my $lmfac = log ($in_flds[$k]) ;
	    push @out_flds, $lmfac;
	}

	print $fh_out1 join ("\t", '0:0', @out_flds), "\n";

} # End of the 1st Main outer while-loop (over input lines). #

close ($fh_in1);
close ($fh_out1);


print STDOUT "\n\n<<< Final, 2: (Rescale &) Output log-factors for case (ii)  >>>\n\n";

my $fh_in2 = open_input_file2 ($infile_muP_case_ii);
unless (defined $fh_in1) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'open_input_file2', FAILED to open a file-handle for the file \"$infile_muP_case_ii\": $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
 
my $fh_out2 = open_output_file ($out_as_dg, 0);
unless (defined $fh_out2) {
	print STDOUT "[ERROR '$program_name'] The subroutine 'open_output_file' FAILED to open a file-handle for the file \"$out_as_dg\": $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}

print $fh_out2 join ("\t", '#{Anc_sites}\E[#{indels}]', @E_INDEL_CTS_OUT), "\n\n";

while (my $line = <$fh_in2>) { # 2nd Main outer while-loop (over input lines). #

	my ($LA) = ($line =~ m/^(\d+)\t/);

	unless (defined $LA) { next; } # Skip lines not providing the multiplication factors.

	unless ((1 <= $LA) and ($LA <= $UB_CT_SITES_OUT)) { next; } # Skip lines whose $LA's are outside of the range for the output.

	$line =~ s/\s+$//;

	my @in_flds = split /\t/, $line;
	my $row_label = shift @in_flds;

	unless (@in_flds == $ct_tlapses_cmpt) {
	    my $ct_flds = @in_flds;
	    print STDOUT "[WARNING '$program_name'] ILLEGAL INPUT: The row labelled '$row_label' has $ct_flds fields (excluding the label), while it SHOULD HAVE $ct_tlapses_cmpt fields: $! !! ... I will SKIP this row ...\n\n";
	    next;
	}

	my @out_flds = ();

	foreach my $k (@SMPL_TIME_POINTS) { 
	    my $lmfac = $k * $SIZE_SB_INTVL * $TOTRATE_INDEL * $LA + log ($in_flds[$k]) ;
	    push @out_flds, $lmfac;
	}

	print $fh_out2 join ("\t", $LA, @out_flds), "\n";

} # End of the 2nd Main outer while-loop (over input lines). #

close ($fh_in2);
close ($fh_out2);



print STDOUT "\n\n<<< Final, 3: Output log-factors for case (iii)  >>>\n\n";

my $fh_in3 = open_input_file2 ($infile_muP_case_iii);
unless (defined $fh_in3) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'open_input_file2', FAILED to open a file-handle for the file \"$infile_muP_case_iii\": $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
 
my $fh_out3 = open_output_file ($out_ag_ds, 0);
unless (defined $fh_out3) {
	print STDOUT "[ERROR '$program_name'] The subroutine 'open_output_file' FAILED to open a file-handle for the file \"$out_ag_ds\": $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}

print $fh_out3 join ("\t", '#{Dec_sites}\E[#{indels}]', @E_INDEL_CTS_OUT), "\n\n";

while (my $line = <$fh_in3>) { # 3rd Main outer while-loop (over input lines). #

	my ($LD) = ($line =~ m/^(\d+)\t/);

	unless (defined $LD) { next; } # Skip lines not providing the multiplication factors.

	unless ((1 <= $LD) and ($LD <= $UB_CT_SITES_OUT)) { next; } # Skip lines whose $LD's are outside of the range for the output.

	$line =~ s/\s+$//;

	my @in_flds = split /\t/, $line;
	my $row_label = shift @in_flds;

	unless (@in_flds == $ct_tlapses_cmpt) {
	    my $ct_flds = @in_flds;
	    print STDOUT "[WARNING '$program_name'] ILLEGAL INPUT: The row labelled '$row_label' has $ct_flds fields (excluding the label), while it SHOULD HAVE $ct_tlapses_cmpt fields: $! !! ... I will SKIP this row ...\n\n";
	    next;
	}

	my @out_flds = ();

	foreach my $k (@SMPL_TIME_POINTS) { 
	    my $lmfac = log ($in_flds[$k]) ;
	    push @out_flds, $lmfac;
	}

	print $fh_out3 join ("\t", $LD, @out_flds), "\n";

} # End of the 3rd Main outer while-loop (over input lines). #

close ($fh_in3);
close ($fh_out3);


print STDOUT "\n\n<<< Final, 3+1: (Rescale &) Output log-factors for case (iv)  >>>\n\n";

my $fh_in3plus1 = open_input_file2 ($infile_muP_case_iv);
unless (defined $fh_in3plus1) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'open_input_file2', FAILED to open a file-handle for the file \"$infile_muP_case_iv\": $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
 
my $fh_out3plus1 = open_output_file ($out_as_ds, 0);
unless (defined $fh_out3plus1) {
	print STDOUT "[ERROR '$program_name'] The subroutine 'open_output_file' FAILED to open a file-handle for the file \"$out_as_ds\": $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}

print $fh_out3plus1 join ("\t", '#{Anc_sites}:#{dec_sites}\E[#{indels}]', @E_INDEL_CTS_OUT), "\n\n";

while (my $line = <$fh_in3plus1>) { # (3+1)th Main outer while-loop (over input lines). #

	my ($LA, $LD) = ($line =~ m/^(\d+)\:(\d+)\t/);

	unless ((defined $LA) and (defined $LD)) { next; } # Skip lines not providing the multiplication factors.

	unless ((1 <= $LA) and ($LA <= $UB_CT_SITES_OUT)) { next; } # Skip lines whose $LA's are outside of the range for the output.
	unless ((1 <= $LD) and ($LD <= $UB_CT_SITES_OUT)) { next; } # Skip lines whose $LD's are outside of the range for the output.

	$line =~ s/\s+$//;

	my @in_flds = split /\t/, $line;
	my $row_label = shift @in_flds;

	unless (@in_flds == $ct_tlapses_cmpt) {
	    my $ct_flds = @in_flds;
	    print STDOUT "[WARNING '$program_name'] ILLEGAL INPUT: The row labelled '$row_label' has $ct_flds fields (excluding the label), while it SHOULD HAVE $ct_tlapses_cmpt fields: $! !! ... I will SKIP this row ...\n\n";
	    next;
	}

	my @out_flds = ();

	foreach my $k (@SMPL_TIME_POINTS) { 
	    my $lmfac = $k * $SIZE_SB_INTVL * $TOTRATE_INDEL * $LA + log ($in_flds[$k]) ;
	    push @out_flds, $lmfac;
	}

	print $fh_out3plus1 join ("\t", "$LA\:$LD", @out_flds), "\n";

} # End of the (3+1)th Main outer while-loop (over input lines). #

close ($fh_in3plus1);
close ($fh_out3plus1);



print STDOUT "\n\n<<< Final, 5: Deal with 'NOMINAL' Boundary Multiplication Factors  >>>\n\n";


print STDOUT "# NOTE: This version of '$program_name' DEALS WITH 'NOMINAL' Boundary Multiplication Factors\n";
print STDOUT "#       merely by creating SYMBOLIC LINKs (with the name suggesting boundary factors) to the BULK Multiplication Factors.\n";
print STDOUT "#\n";
print STDOUT "#      ... Hopefully, in the future, the boundary multiplication factors will be computed more accurately (under some specific evolution model(s)) ...\n\n\n";


			# (Added on 2020/03/02.) #

print STDOUT "* First, change the current directory to: \"$fin_outdir/\" .\n\n";

unless ( chdir ($fin_outdir) ) {
	print STDOUT "[ERROR '$program_name'] FAILED the built-in function, 'chdir', to \"$fin_outdir\": $? \n\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... the built-in function, 'chdir', Ended  ...\n\n") if ($TEST_LEVEL > 1);

print STDOUT "\n* Then, create symbolic links for the boundary factors ...\n\n";

my $log_slink = "log_symlink.txt";

foreach my $fname_pair (
	[$name_out_nosites, $name_out_bd_nosites],
	[$name_out_as_dg, $name_out_bd_as_dg],
	[$name_out_ag_ds, $name_out_bd_ag_ds],
	[$name_out_as_ds, $name_out_bd_as_ds]
			) { # Trailing foreach-loop (over source-target pairs of files to be linked). #

	my ($fname_src, $fname_tgt) = @{$fname_pair};

	unless (0 == system ( "ln -s $fname_src $fname_tgt >> $log_slink 2>&1" ) ) {
	    print STDOUT "[ERROR '$program_name'] FAILED to create a symbolic link, '$fname_tgt', to the source file, '$fname_src': $? \n\n";
	    print STDERR "[ERROR '$program_name'] $!\n";
	    die ;
	}
    (print STDOUT "\n ... Created a symbolic link, '$fname_tgt', to the source file, '$fname_src' ...\n\n") if ($TEST_LEVEL > 0);

} # End of the Trailing foreach-loop (over source-target pairs of files to be linked). #

			# End of "(Added on 2020/03/02.)" #
 




		### RESTART FROM HERE ###

	### END of "FINAL: Compute & Output the log-multiplication factors for cases (i)-(iv). ###
	### (Writing Started on 2020/03/01, Ended on 2020/03/dd.)"


print STDOUT "\n\n\n ===== ALL the PROCESSES of '$program_name' FINISHED !!!!! =====\n\n\n"; # ADDED on 2020/03/01. #



	##### DEFINITIONS of the Subroutines. #####


# create_dir_rw:
#	This subroutine receives the path to a directory,
#	examines (1) whether the directory exists.
#	Otherwise, it creates the directory.
#	Then, it examines (2) whether the directory is readable & writable.
#	Otherwise, it changes the permission mode of the directory to make it readable & writable.
#	[CREATED on 2020/01/29.]
#
sub create_dir_rw (\$) {

	my ($rf_dir_path) = @_;
	my $dir_path = ${$rf_dir_path};

	unless ( -d $dir_path ) {
	   unless (0 == system ("mkdir -p $dir_path") ) {
		print STDOUT "[ERROR '$program_name'] FAILED to CREATE the OUTPUT DIRECTORY, '$dir_path/': $? !\n\n";
		print STDERR "[ERROR '$program_name'] $!\n";
		return 0 ;
	   }
	}
	unless (( -r $dir_path ) and ( -w $dir_path )) {
	    unless (0 == system ("chmod u+rw $dir_path") ) {
		print STDOUT "[ERROR '$program_name'] FAILED to CHANGE permission MODES of the output directory, '$dir_path': $? !\n";
		print STDERR "[ERROR '$program_name'] $!\n";
		return 0 ;
	    }
	}

	return 1;

}	# End of the subroutine, 'create_dir_rw'. #


=MOVED to "../MyPerlModules_LASTPIECE/MyLASTPIECE_IO.pm" on 2020/01/27.
# print_indel_length_freqs:
#	This subroutine prints out the length frequencies of insertions/deletions,
#	both in the bulk and at the boundary.
#	It was created by modifying a block in "calc_log_mfacs_pars_pwa_dawg.pl" (at "~/Ezawa_works_KyuTech/Study/New_Alignment_Programs/OpenSource/Exported/TESTs/Test_MyANEX_Main/Test_nonrec_mlt_shifts_gapblocks/InData/").
#	[CREATED on 2020/01/26; CORRECTED on 2020/01/27; MOVED to "../MyPerlModules_LASTPIECE/MyLASTPIECE_IO.pm" on 2020/01/27]
#
sub print_indel_length_freqs  (\$$$\@\@\@\@) {

	my ($rf_ofile, $cutoff_ins_len, $cutoff_del_len, $freqs_ins, $freqs_del, $freqs_ins_bd, $freqs_del_bd) = @_;

	my $ofile = ${$rf_ofile};
	my $fh_out = open_output_file ($ofile, 0);
	unless (defined $fh_out) {
	   print STDOUT "[ERROR 'print_indel_length_freqs'] The subroutine 'open_output_file' FAILED to open a file-handle for the file \"$ofile\": $! !\n";
	   return 0;
	}

   ##print STDOUT "\n< Indel length frequencies >\n\n";

	print $fh_out join ("\t", 'Length', 'Ins', 'Del', 'Ins(bd)', 'Del(bd)'), "\n\n";

	my $cutoff = ($cutoff_ins_len > $cutoff_del_len) ? $cutoff_ins_len : $cutoff_del_len ;

   for (my $l=1; $l<= $cutoff; $l++) {
	my $f_ins = ($l > $cutoff_ins_len) ? 'undef' : $freqs_ins->[$l] ;
	my $f_del = ($l > $cutoff_del_len) ? 'undef' : $freqs_del->[$l] ;
	my $f_ins_bd = ($l > $cutoff_ins_len) ? 'undef' : $freqs_ins_bd->[$l] ;
	my $f_del_bd = ($l > $cutoff_del_len) ? 'undef' : $freqs_del_bd->[$l] ;

	print $fh_out join ("\t", $l, $f_ins, $f_del, $f_ins_bd, $f_del_bd), "\n";
   }
	print $fh_out "\n\n";

	close ($fh_out);

	return 1;
 
}	# End of the subroutine, 'print_indel_length_freqs'. #
=cut # END of "MOVED to "../MyPerlModules_LASTPIECE/MyLASTPIECE_IO.pm" on 2020/01/27."



