#!/usr/bin/perl
#
###################################################################################
#	
#	LASTPIECE_P (Local Alignment-STate Probabilities that Insertion-type and dEletion-type gaps Co-Exist, type P):
#	    A package of programs to compute the probabilities of gapped segments in each of which an insertion-type gap and a deletion-type gap co-exist, (which were referred to as case-(iv) gapped segments by Ezawa (2016a?)),
#	    under a stochastic model of sequence evolution with biologically realistic insertions/deletions.
#	   (Written almost exclusively in Perl.)
#
#	Version 0.3: Copyright (C) 2020 Kiyoshi Ezawa
#
#    This package is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This package is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License, "GNU_GPL.txt",
#    along with this package.  If not, see <http://www.gnu.org/licenses/>.
#
#    The author can be contacted by e-mailing to <kezawa dot ezawa3 at gmail dot com>
#    (replace " dot " and " at " with "." and "@", respectively).
#
###################################################################################
#
# cmpt_mu_Dcr.pl:
#	This servant script computes the multiplication factors
#	for the creation of a "D"-region:
#	mu_{D-cr} [ (x, [t_1, t_1 + Delta{}t]) ; (L, t_2) ],
#	where x is the position at which the "D"-region is created,
#	[t_1, t_1 + Delta{}t] is the time-interval within which the D-creation occurred,
#	and L is the number of sites in the "D"-region at the time-point t_2 ( > t_1).
#	The script was created by taking advantage of the main framework of another servant script,
#	'cmpt_muP0.pl' (in "../muP0/") .
#	[CREATION STARTED on 2020/02/03, ENDED on 2020/02/04; Modified on 2020/02/05]
#
#	[DEBUGGED on 2020/02/25:]
#	CORRECTED the erroneous processing of the command-line options '--RAW' and '--DRESSED'.
#


	##### (IMPORTED w/ modifications from '../muP0/cmpt_muP0.pl' on 2020/02/04) (HD & CL & CHK) #####

		### (IMPORTED w/ modifications from '../../../lastpiece.alpha.pl' on 2020/01/30) (HD) ###

use strict;
use warnings;

use MyFile_manipu qw(
	open_input_file2
	open_output_file
	list_sessions_ext
	);

use MyLASTPIECE_IO qw(
	readin_indel_params
	readin_indel_rates
	print_muP_case_ii_or_iii 
	readin_muP0s
	);
	# print_muP_case_i
	# print_muP0s
	#
	# Added 'readin_indel_params' & 'readin_indel_rates' on 2020/01/31. #
	# Added 'print_muP_case_i' & 'print_muP_case_ii_or_iii' on 2020/02/01. # => Removed on 2020/02/03. 
	# => Resurrected 'print_muP_case_ii_or_iii', in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/05).
	#
	# Added 'print_muP0s' on 2020/02/03. => Removed on 2020/02/04.
	# Added 'readin_muP0s' on 2020/02/04.

##use MyLASTPIECE_Sppl qw(
##	cal_delta_exp_del_5p2
##	);


##sub create_dir_rw (\$); # Added on 2020/01/29.


$| = 1; # Flush the buffer for standard output as soon as it receives data.

my $bin_perl = "/usr/bin/perl"; # The Perl interpreter. # (Added on 2020/01/29). #

my $program_path = $0;
my ($program_name) = ($program_path =~ m!([^/]+)$!);

	# Controlling the level of test outputs. #
	# (0: silent; 1: very quiet; 2: quiet; 3: medium; 4: verbose; 5: very verbose) 
our $TEST_LEVEL = 2;
##our $TEST_LEVEL = 5;

$MyLASTPIECE_IO::TEST_LEVEL = $TEST_LEVEL ; # Added on 2020/01/27. # RESURRECTED on 2020/02/04.


		### END of "(IMPORTED w/ modifications from '../../../lastpiece.alpha.pl' on 2020/01/30) (HD)" ###

our $EPS = 1E-07; # A "small" number to be used for checking the consistency. # (Added on 2020/02/01.) #


our $IF_DRESSED = 0; # Whether the input indel rates are raw (=0) or dressed (=1). (The default value = 0.)  # Added in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). #

	####### Take in the Command-Line Arguments. #######

		# Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04) (CL1) #
##unless (@ARGV > 7) {
##	print STDERR "\nUsage:  [perl] $program_name {initial_time (non-negative number)} {final_time (positive number, greater than initial_time)} {number_of_sub_time_intervals (positive integer)} {upper-bound_of_number_of_sites} {SIGMA_BE (= 0,1, or 2)} {input_file_for_indel_parameters} {input_file_for_raw_indel_rates} {output_file}";
##	print STDERR " .\n\n";
#	...
##}
unless (@ARGV > 8) {
	##print STDERR "\nUsage:  [perl] $program_name {initial_time (non-negative number)} {final_time (positive number, greater than initial_time)} {number_of_sub_time_intervals (positive integer)} {upper-bound_of_number_of_sites} {SIGMA_BE (MUST ALWAYS BE 2!!)} {input_file_for_indel_parameters} {input_file_for_raw_indel_rates} {input_file for muP0 (w/ SIGMA_BE=2)} {output_file} [ options ]";
	print STDERR "\nUsage:  [perl] $program_name {initial_time (non-negative number)} {final_time (positive number, greater than initial_time)} {number_of_sub_time_intervals (positive integer)} {upper-bound_of_number_of_sites} {SIGMA_BE (MUST ALWAYS BE 2!!)} {input_file_for_indel_parameters} {input_file_for_raw_indel_rates} {input_file for muP0 (w/ SIGMA_BE=2)} {output_file} "; # CORRECTED on 2020/02/06. # 
		# End of "Modified in creation of 'cmpt_muP0.pl' (on 2020/02/03) (CL1)" #
	print STDERR " [ options ] ,\n";
	print STDERR "   where the [options] can be either of the following:\n\n";
	print STDERR "            --RAW      when using the RAW indel rates;\n";
	print STDERR "            --DRESSED      when using the DRESSED indel rates.\n";
	print STDERR "		  (NOTE1: If both options are used, the '--RAW' will be prioritized.)\n"; # (The '\n";' was added in debugging on 2020/02/05.)
	print STDERR "		  (NOTE2: If no options are used, the '--RAW' will be assumed.)\n"; # (The '\n";' was added in debugging on 2020/02/05.)
	##print STDERR "           --XLEVEL=integer       for the level of MSA space exploration (from =1 (performing only multiple-\"shift\"s) to =5 (performing all programed moves) (default=$XLEVEL);\n"; # ADDED on 2019/08/01.
	print STDERR "\n";

	die "$!\n\n";
}
		# End of "Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04) (CL1)" #

		# Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04) (CL2) #
##our ($TIME_INIT, $TIME_FIN, $CT_SB_INTVLS, $UB_CT_SITES_CMPT, $SIGMA_BE) = @ARGV[0 .. 3+1];
##my ($infile_id_prms, $infile_id_rates, $ofile_muP0) = @ARGV[5 .. 7];
our ($TIME_INIT, $TIME_FIN, $CT_SB_INTVLS, $UB_CT_SITES_CMPT, $SIGMA_BE) = @ARGV[0 .. 3+1];
my ($infile_id_prms, $infile_id_rates, $infile_muP0, $ofile_mu_Dcr) = @ARGV[5 .. 8];
		# End of "Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04) (CL2)" #

		# Added in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). (CL) #
my ($if_dressed, $if_raw) = (0, 0);

for (my $k=9; $k < @ARGV; $k++) { # Preliminary for-loop (over additional command-line arguments). #

	my $clarg = $ARGV[$k];
	unless ($clarg =~ s/^(\-)+//) { next; } # Skip if the argument lacks preceding '-'s. #

	my ($option, $value) = split /\=/, $clarg;

=DISABLED in DEBUGGING (on 2020/02/25). (CL)
	unless ((defined $value) and ($value ne '')) {
	     print STDOUT "[CAUTION '$program_name'] Found an UNEXPECTED FORMAT of the commmand-line argument, ", $ARGV[$k], ": $! ! ... I will SKIP it ...\n\n";
	     next; # Skip an argument in an unexpected format.
	}
=cut # END of "DISABLED in DEBUGGING (on 2020/02/25). (CL)"

	my $org_option = $option;
	$option = uc $option; # Capitalize the option, in advance.

	if ($option eq 'RAW') {
	    $if_raw = 1;

	} elsif ($option eq 'DRESSED') {
	    $if_dressed = 1;

	} else {
	    print STDOUT "[CAUTION '$program_name'] Found an UNEXPECTED OPTION, '$org_option', within the cmmmand-line argument, ", $
ARGV[$k], ": $! ! ... I will SKIP it ...\n\n";
	    next; # Skip an argument with an unexpected option.
	}

} # End of the "Preliminary for-loop (over additional command-line arguments)." #

if ($if_dressed and !($if_raw)) {
	$IF_DRESSED = 1;

	print STDOUT "\n[INFORMATION '$program_name'] AS YOU SPECIFIED, we set \$IF_DRESSED to be 1 (true) ...\n\n";
}
		# End of "Added in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). (CL)" #

	####### END of "Take in the Command-Line Arguments." #######

	## Check the value of $SIGMA_BE ##
	## (Added in creation of 'cmpt_mu_Dcr.pl' on 2020/02/04.) ##
unless ($SIGMA_BE == 2) {
	print STDOUT "[ERROR '$program_name'] The \$SIGMA_BE (=$SIGMA_BE) is NOT 2, which this script requires: $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
	## End of "(Added in creation of 'cmpt_mu_Dcr.pl' on 2020/02/04.)" ##


	### Check the existence & permission mode of the input files. ###
	### (Added on 2020/02/03.) ###

foreach my $infile ($infile_id_prms, $infile_id_rates, $infile_muP0) {
    unless ( ( -f $infile ) and ( -r $infile ) ) {
	print STDOUT "[ERROR '$program_name'] CanNOT FIND a readable input file, \"$infile\": $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
    }
}
	### End of "(Added on 2020/02/03.)" ###

	##### END of "(IMPORTED w/ modifications from '../muP0/cmpt_muP0.pl' on 2020/02/04) (HD & CL & CHK)" #####


	### Declare the Beginning of the Script ###
	### (IMPORTED w/ modifications from '../muP0/cmpt_muP0.pl' on 2020/02/04) (DC) ###

print STDOUT "\n<<< Program name: $program_name (path: $program_path) >>>\n\n";

print STDOUT "\n<< Input Files: >>\n\n";

print STDOUT " (for indel parameters) $infile_id_prms ,\n";
print STDOUT " (for raw indel rates)  $infile_id_rates ,\n";
print STDOUT " (for muP0 under \"base\" rate operator)  $infile_muP0 .\n\n"; # Added in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). (DC1) #

print STDOUT "\n<< Output File: >>\n\n"; 

		# Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). #
##print STDOUT " (for muP0 under \"base\" rate operator)  $ofile_muP0 .\n\n";
print STDOUT " (for multiplication factors for the creation of a \"D\"-region)  $ofile_mu_Dcr .\n\n";
		# End of "Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04)." #


print STDOUT "\n<< Global Variables specifying the levels of tests: >>\n\n"; # Added on 2020/01/30. (DC2) #

print STDOUT "\n", '$TEST_LEVEL = ', $TEST_LEVEL, " .\n\n";

print STDOUT '$MyLASTPIECE_IO::TEST_LEVEL  = ', $MyLASTPIECE_IO::TEST_LEVEL , " .\n\n"; # Added on 2020/02/04.

print STDOUT "\n<< A \"small\" number to be used for checking the consistency: >>\n\n"; #
print STDOUT '$EPS = ', $EPS, " .\n\n";

		# Added in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). (DC2) #
print STDOUT "\n<< Option that specifies the type of indel rates: >>\n\n";
print STDOUT '$IF_DRESSED = ', $IF_DRESSED, " .\n\n";
		# End of "Added in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). (DC2)" #


print STDOUT "\n<< Parameters necessary for recursive computations of the multiplication factors: >>\n\n";

print STDOUT '($TIME_INIT, $TIME_FIN) = ', "($TIME_INIT, $TIME_FIN) .\n\n";
print STDOUT '#{Sub-time-intervals} = ', $CT_SB_INTVLS, " .\n\n";

print STDOUT '{Upper-bound of #{sites} for computation} = ', $UB_CT_SITES_CMPT, " .\n\n"; 

print STDOUT '$SIGMA_BE = ', $SIGMA_BE, " .\n\n"; # Added in creation of 'cmpt_muP0.pl' (on 2020/02/03). #

	### End of "(IMPORTED w/ modifications from '../muP0/cmpt_muP0.pl' on 2020/02/04) (DC)" ###



	##### (IMPORTED from '../muP_case_i2iii/cmpt_muP_case_i2iii.pl' on 2020/02/03) (Pre 00) #####
	##### (==> IMPORTED w/ modifications from '../muP0/cmpt_muP0.pl' on 2020/02/04&05) (Pre 00) #####


	## (MODIFIED on 2020/01/31) (00-IN) ##

print STDOUT "\n<<< Preliminary Process 00-IN: Reading in the quantities necessary for the computations >>>\n\n";

	## 00-IN-A: Read in the input file for indel parameters. ##

print STDOUT "\n<< (Pre 00-IN-A): Reading in the indel model parameters >>\n\n";

our ($TOTRATE_INS, $TOTRATE_DEL, $CUTOFF_INS_LEN, $CUTOFF_DEL_LEN)
   = readin_indel_params ($infile_id_prms);

unless (defined $CUTOFF_DEL_LEN) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'readin_indel_params', FAILED on the input file, \"$infile_id_prms\" $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'readin_indel_params', Ended on the input file, \"$infile_id_prms\" ...\n\n") if ($TEST_LEVEL > 1);

##my ($infile_id_prms, $infile_id_rates, $ofile_muP_case_i, $ofile_muP_case_ii, $ofile_muP_case_iii) = @ARGV[4 .. 8];

print STDOUT "\n[ Indel Model Parameters retrieved: ]\n\n";

print STDOUT '($TOTRATE_INS, $TOTRATE_DEL) = ', "($TOTRATE_INS, $TOTRATE_DEL) .\n";
print STDOUT '($CUTOFF_INS_LEN, $CUTOFF_DEL_LEN) = ', "($CUTOFF_INS_LEN, $CUTOFF_DEL_LEN) .\n\n";


	## Preliminary 00-B: Read in the input file for indel rates. ##

		# Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). (00-B-1) #
if ($IF_DRESSED) {
	print STDOUT "\n<< (Pre 00-IN-B): Reading in the DRESSED indel rates >>\n\n";
} else {
	print STDOUT "\n<< (Pre 00-IN-B): Reading in the RAW indel rates >>\n\n";
}
##print STDOUT "\n<< (Pre 00-IN-B): Reading in the raw indel rates >>\n\n";
		# End of "Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). (00-B-1)" #

our ($rf_RATES_INS, $rf_RATES_DEL, $rf_RATES_INS_BD, $rf_RATES_DEL_BD)
    = readin_indel_rates ($infile_id_rates);

unless (defined $rf_RATES_DEL_BD) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'readin_indel_rates', FAILED on the input file, \"$infile_id_rates\" $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'readin_indel_rates', Ended on the input file, \"$infile_id_rates\" ...\n\n") if ($TEST_LEVEL > 1);


if ($TEST_LEVEL > 2) {

		# Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). (00-B-2) #
  if ($IF_DRESSED) {
    print STDOUT "\n[ DRESSED Indel Rates (x \$SIZE_SB_INTVL) retrieved: ]\n\n";
  } else {
    print STDOUT "\n[ RAW Indel Rates retrieved: ]\n\n";
  }
    ##print STDOUT "\n[ Raw Indel Rates retrieved: ]\n\n";
		# End of "Modified in creation of 'cmpt_mu_Dcr.pl' (on 2020/02/04). (00-B-2)" #

    print STDOUT join ("\t", 'Length', 'Ins', 'Del', 'Ins(bd)', 'Del(bd)'), "\n\n";

    my $maxlen = 0;
    foreach my $rf_rates ($rf_RATES_INS, $rf_RATES_DEL, $rf_RATES_INS_BD, $rf_RATES_DEL_BD) {
	my $len = @{$rf_rates} - 1;
	if ($maxlen < $len) { $maxlen = $len; }
    }

    for (my $l=1; $l <= $maxlen; $l++) {
	my @out_rates = ();
	foreach my $rf_rates ($rf_RATES_INS, $rf_RATES_DEL, $rf_RATES_INS_BD, $rf_RATES_DEL_BD) {
	    my $rate = $rf_rates->[$l];
	    unless (defined $rate) { $rate = 'undef'; }
	    push @out_rates, $rate;
	}
	print STDOUT join ("\t", $l, @out_rates), "\n";
    }
    print STDOUT "\n\n";

} # End of "if ($TEST_LEVEL > 2) {...}"



	## 00-IN-C: Read in the input file for muP0s (w/ $SIGMA_BE = 2). ##
	## (Added in creation of 'cmpt_mu_Dcr.pl' on 2020/02/04&05). (00-IN-C) ##

print STDOUT "\n<< (Pre 00-IN-C): Reading in the muP0s (w/ \$SIGMA_BE = 2) >>\n\n";

my ($rf_muP0s) =  readin_muP0s ($infile_muP0, $CT_SB_INTVLS, $UB_CT_SITES_CMPT);

	# $rf_muP0s->[$LI][$jt][$l] = (w/ 0 < $L_I, l <= $ub_ct_sites, & 0 <= $jt <= $ct_sb_intvls) 
	# is the multiplication factor computed under the "base" rate operator
	# at the $jt th time-interval with $L_I sites initially and $l sites finally, both in between the PASs.


unless (defined $rf_muP0s) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'readin_muP0s', FAILED on the input file, \"$infile_muP0\" $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'readin_muP0s', Ended on the input file, \"$infile_muP0\" ...\n\n") if ($TEST_LEVEL > 1);


if ($TEST_LEVEL >= 5) {

	print STDOUT "\n\n[[ \"Base\" multiplication factors, mu_{P0} [ (L, t) | (L_I, t_I) ], retrieved (& stored in ", '@{$rf_muP0s}', "): ]]\n\n";

	print STDOUT join ("\t", '$L_I:$l\$jt', 0 .. $CT_SB_INTVLS), "\n\n";

			## IMPORTED w/ modifications from the subroutine, 'print_muP0s', in "MyLASTPIECE_IO.pm" (on 2020/02/05). ## 

	my $ct_tintvls = $CT_SB_INTVLS +1;
	my $cap_ct_sites = $UB_CT_SITES_CMPT +1;

	for (my $L_I = 1; $L_I < $cap_ct_sites; $L_I++) { # Main outer for-loop (over the initial site-count).

	    my $muP0s_fxd_L_I = $rf_muP0s->[$L_I];

	    for (my $l =1; $l < $cap_ct_sites; $l++) { # Main middle for-loop (over the final site-counts). # (MODIFIED on 2020/02/02). #

		my @out_muP0s = ();

		for (my $jt = 0; $jt < $ct_tintvls; $jt++) { # Inner for-loop (over the time-intervals). #
		    $out_muP0s[$jt] = $muP0s_fxd_L_I->[$jt][$l];
		} # End of the inner for-loop (over the time-intervals). #

		print STDOUT join ("\t", "$L_I:$l", @out_muP0s), "\n";

	    } # End of the main middle for-loop (over the final site-counts). #

	} # Main outer for-loop (over the initial site-count).

			## End of "IMPORTED w/ modifications from the subroutine, 'print_muP0s', in "MyLASTPIECE_IO.pm" (on 2020/02/05)." ## 

	print STDOUT "\n\n\n";

} # End of "if ($TEST_LEVEL >= 5) {...}"

	## End of "(Added in creation of 'cmpt_mu_Dcr.pl' on 2020/02/04&05). (00-IN-C)" ##


=OBSOLETE as of 2020/01/31. (00-IN)
	## Preliminary 00-A: Read in the input file for indel parameters. ##

print STDOUT "\n<< Indel Model Parameters retrieved: >>\n\n";

print STDOUT '$TOTRATE_INS = ', $TOTRATE_INS, " ,\n";
print STDOUT '$TOTRATE_DEL = ', $TOTRATE_DEL, " .\n\n";

	## Preliminary 00-B: Read in the input file for raw indel rates. ##

=cut # End of "OBSOLETE as of 2020/01/31. (00-IN)"

	## End of "(MODIFIED on 2020/01/31) (00-IN)" ##


	## Preliminary 00-C: Compute Other Important Parameters ##
	### (IMPORTED w/ modifications from '../../../lastpiece.alpha.pl' on 2020/01/30) (00-C) ###

print STDOUT "\n<<< Preliminary Process 00-C: Computing Other Important Parameters >>>\n\n";

our $TOTRATE_INDEL = $TOTRATE_INS + $TOTRATE_DEL;
our $SIZE_FULL_INTVL = $TIME_FIN - $TIME_INIT;
our $SIZE_SB_INTVL = $SIZE_FULL_INTVL /$CT_SB_INTVLS;

		# Modified on 2020/01/31. (00-C-1) #
our @TIME_LAPSES_CMPT = (); # Values of the time lapse (= t_F - t_I) at which the factors will be computed.
for (my $i = 0; $i <= $CT_SB_INTVLS; $i++) {
	$TIME_LAPSES_CMPT[$i] = $i * $SIZE_SB_INTVL;
} 
=OBSOLETE as of 2020/01/31. (A)
our @TIME_LAPSES = (); # Values of the time lapse (= t_F - t_I) to be examined.
for (my $i = 0; $i <= $CT_SB_INTVLS; $i++) {
	$TIME_LAPSES[$i] = $i * $SIZE_SB_INTVL;
} 
=cut # End of "OBSOLETE as of 2020/01/31. (A)"
		# End of "Modified on 2020/01/31. (00-C-1)" #

=DISABLED on 2020/01/31. (1)
our @E_INDEL_CTS = (); # Expected counts of indels. #
foreach my $tlapse (@TIME_LAPSES) { push @E_INDEL_CTS, $TOTRATE_INDEL * $tlapse; }
=cut # End of "DISABLED on 2020/01/31. (1)"

print STDOUT '$TOTRATE_INDEL = ', $TOTRATE_INDEL, " \;\n";
print STDOUT '$SIZE_FULL_INTVL = ', $SIZE_FULL_INTVL, " \;\n";
print STDOUT '$SIZE_SB_INTVL = ', $SIZE_SB_INTVL, " .\n\n";

		# Modified on 2020/01/31. (00-C-2) #
print STDOUT '@TIME_LAPSES_CMPT = (', join (', ', @TIME_LAPSES_CMPT), ") .\n\n";
##print STDOUT '@TIME_LAPSES = (', join (', ', @TIME_LAPSES), ") .\n\n";
		# End of "Modified on 2020/01/31. (00-C-2)" #

=DISABLED on 2020/01/31. (2)
print STDOUT '@E_INDEL_CTS = (', join (', ', @E_INDEL_CTS), ") .\n\n";
=cut # End of "DISABLED on 2020/01/31. (2)"

print STDOUT "\n";

	### End of "(IMPORTED w/ modifications from '../../../lastpiece.alpha.pl' on 2020/01/30) (00-C)" ###

	##### END of "(IMPORTED from '../muP_case_i2iii/cmpt_muP_case_i2iii.pl' on 2020/02/03) (Pre 00) #####
	##### (==> IMPORTED w/ modifications from '../muP0/cmpt_muP0.pl' on 2020/02/04&05) (Pre 00)" #####


print STDOUT "\n<<< Preliminary Process 1: Computing ", '@insrates_x_dt', " >>>\n\n";

my @insrates_x_dt  = (0); # $insrates_x_dt[$l] = {the rate of insertion of length $l} x dt.


if ($IF_DRESSED) { # The input rates are "dressed".

   for (my $l=1; $l <= $CUTOFF_INS_LEN; $l++) {
	$insrates_x_dt[$l] = $rf_RATES_INS->[$l];
   }

} else { # The input rates are "raw".

   for (my $l=1; $l <= $CUTOFF_INS_LEN; $l++) {
	$insrates_x_dt[$l] = $SIZE_SB_INTVL * $rf_RATES_INS->[$l];
   }
} # End of "if ($IF_DRESSED) {...} else {...}". #


	###  Main Process: Compute multiplication factors, 'mu_Dcr [(x, [t_1, t_1 +Delta{}t]); (L, t_2) ]',  for the creation of a "D"-region. ###

print STDOUT "\n<<<<< Main Process: Compute multiplication factors,  'mu_Dcr [(x, [t_1, t_1 +Delta{}t]); (L, t_2) ]',  for the creation of a \"D\"-region >>>>>\n\n\n";

my @mu_Dcr = (); # $mu_Dcr[$jt]->[$L] = mu_Dcr [ (x, [t_1, t_1 + Delta{}t]) ; ($L, t_2 = t_1 + $jt * $SIZE_SB_INTVL) ].

my $ct_tlapses = @TIME_LAPSES_CMPT;
my $cap_ct_sites = $UB_CT_SITES_CMPT +1;

		# First, Initialize @mu_Dcr. #
		# (NOTE: the mu_Dcr's at $jt =0 should always be zero.)

for (my $jt = 0; $jt < $ct_tlapses; $jt++) { # Preliminary outer for-loop (over time-lapses). #

	my @vec_mu_Dcr = ();
	for (my $L=0; $L < $cap_ct_sites; $L++) { $vec_mu_Dcr[$L] = 0; }

	$mu_Dcr[$jt] = \@vec_mu_Dcr;

}	# End of the preliminary outer for-loop (over time-lapses). #


		# Then, perform the main computation. #

my $cap_inslen_cmpt = ($CUTOFF_INS_LEN < $UB_CT_SITES_CMPT) ? $CUTOFF_INS_LEN : $UB_CT_SITES_CMPT ;
$cap_inslen_cmpt++;

# our ($rf_RATES_INS, $rf_RATES_DEL, $rf_RATES_INS_BD, $rf_RATES_DEL_BD)
# print STDOUT '($CUTOFF_INS_LEN, $CUTOFF_DEL_LEN) = ', "($CUTOFF_INS_LEN, $CUTOFF_DEL_LEN) .\n\n";

for (my $LI = 1; $LI < $cap_inslen_cmpt; $LI++) { # Main Outer for-loop (over insertion lengths). #

	my $insrate_x_dt = $insrates_x_dt[$LI];

	my $muP0s_fxd_LI = $rf_muP0s->[$LI];

	# $rf_muP0s->[$LI][$jt][$L] = (w/ 0 < $LI, L <= $ub_ct_sites, & 0 <= $jt <= $ct_sb_intvls) 
	# is the multiplication factor computed under the "base" rate operator
	# at the $jt th time-interval with $L_I sites initially and $L sites finally, both in between the PASs.

	for (my $jt=1; $jt < $ct_tlapses; $jt++) { # Main middle for-loop (over time-lapses). #

	    my $vec_muP0s = $muP0s_fxd_LI->[$jt-1]; # The time-lapse of mu_{P0} MUST BE smaller than that of mu_Dcr by 1 sub-interval (i.e., Delta{}t). #
	    my $vec_mu_Dcr = $mu_Dcr[$jt];

	    for (my $L=1; $L < $cap_ct_sites; $L++) {
		$vec_mu_Dcr->[$L] += $insrate_x_dt * $vec_muP0s->[$L];
	    }

	} # End of the main middle for-loop (over time-lapses). #

} # End of the Main Outer for-loop (over insertion lengths). #


		### RESTART FROM HERE ###


	### Finishing Process: Output the Results. ###

print STDOUT "\n<<<<< Finishing Process: Output the Results. >>>>>\n\n";

unless ( print_muP_case_ii_or_iii ($ofile_mu_Dcr, '#{sites}_Fin\Time_Lapse', @TIME_LAPSES_CMPT, @mu_Dcr) ) {
	print STDOUT "[ERROR '$program_name'] The subroutine, 'print_muP_case_ii_or_iii', FAILED to output to the file, \"$ofile_mu_Dcr\" $? !\n";
	print STDERR "[ERROR '$program_name'] $!\n";
	die ;
}
(print STDOUT "\n ... The subroutine, 'print_muP_case_ii_or_iii', Ended outputting to the file, \"$ofile_mu_Dcr\" ...\n\n") if ($TEST_LEVEL > 1);


print STDOUT "\n\n === All The Processes of '$program_name' FINISHED !! ===\n\n\n";




