package MyArrayManipu_num;
#
###################################################################################
#	DENSERM_P: A package to DEtect Negative SElection on Recurrent Mutations
#			(almost exclusively via Perl modules and scripts).
#	Version 0.3: Copyright (C) 2012 Kiyoshi Ezawa
#
#    This package is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This package is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License, "GNU_GPL.txt",
#    along with this package.  If not, see <http://www.gnu.org/licenses/>.
#
#    The author can be contacted by e-mailing to <kezawa dot ezawa3 at gmail dot com>
#    (replace " dot " and " at " with "." and "@", respectively).
###################################################################################
#
# MyArrayManipu_num.pm: a package for various array manipulations
#			(which handles numerical arrays exclusively ).
#

use strict;
use warnings;
use Exporter;
our @ISA = qw(Exporter);
#our @EXPORT = qw( diff_nmarray union_nmarray nr_nmarray sum_nmarray nr_nmarray_graft);
#our @EXPORT_OK = qw( diff_array_of_nmarrays max_min);
our @EXPORT_OK = qw(
	diff_nmarray union_nmarray nr_nmarray sum_nmarray nr_nmarray_graft
	diff_array_of_nmarrays max_min
	find_eqdiv_boundaries find_eqdiv_wgt_boundaries
	trim_numset  union_intsec_nmarrays
	rescale_confining_vector
	log_sum_exp
	binsearch_num binsearch_num_d
	);


# diff_nmarray: a subroutine which receives as arguments two references of arrays,
#		and returns the references of THREE arrays,
#		one is the intersection of the argument arrays,
#		and the other two are their non_overlapping parts.
#
sub diff_nmarray (\@\@) {

	my ($rf_a1,$rf_a2) = @_;
	my @common = my @ext_a1 = my @ext_a2 = ();

	my @sarray1 = sort { $a <=> $b} @{$rf_a1};
	my @sarray2 = sort { $a <=> $b } @{$rf_a2};

	my $item1 = shift @sarray1;
	my $item2 = shift @sarray2;

	while (1) {			# the 'infinite' while loop #

		unless (defined $item1) {
			if (defined $item2) {
				push @ext_a2, $item2;
				@sarray2 and (push @ext_a2, @sarray2); # cram the remaining elements #
			}
			last;
		}

		unless (defined $item2) {
			if (defined $item1) {
				push @ext_a1, $item1;
				@sarray1 and (push @ext_a1, @sarray1); # cram the remaining elements #
			}
			last;
		}

		if ($item1 == $item2) {		# if-elsif-else #
			push @common, $item1;
			$item1 = shift @sarray1;
			$item2 = shift @sarray2;

		} elsif ($item1 <  $item2) {
			push @ext_a1, $item1;
			$item1 = shift @sarray1;

		} else {
			push @ext_a2, $item2;
			$item2 = shift @sarray2;
		}		# end of the if_elsif_else #

	}			# end of the 'infinite' while loop #

	return (\@common,\@ext_a1,\@ext_a2);

}			# end of the module diff_nmarray #


# diff_array_of_nmarrays: a subroutine which receives as arguments
#	 two references of arrays whose elements  being array references,
#		and returns the references of FOUR arrays,
#		two are for the intersection of the input arrays
#		(each of them is allocated for 'common' part of each input array),
#		and the other two are their non_overlapping parts.
#
sub diff_array_of_nmarrays (\@\@$) {

	my ($rf_a1,$rf_a2, $indx_cmp) = @_;
	my @common1 = my @common2 = my @ext_a1 = my @ext_a2 = ();

	my @sarray1 = sort { $a->[$indx_cmp] <=> $b->[$indx_cmp]}
			 @{$rf_a1};
	my @sarray2 = sort { $a->[$indx_cmp] <=> $b->[$indx_cmp] }
			 @{$rf_a2};

	my $item1 = shift @sarray1;
	my $item2 = shift @sarray2;

	while (1) {					# the 'infinite' while loop #

		unless (defined $item1) {
			if (defined $item2) {
				push @ext_a2, $item2;
				@sarray2 and (push @ext_a2, @sarray2); # cram the remaining elements #
			}
			last;
		}

		unless (defined $item2) {
			if (defined $item1) {
				push @ext_a1, $item1;
				@sarray1 and (push @ext_a1, @sarray1); # cram the remaining elements #
			}
			last;
		}

		if ($item1->[$indx_cmp] == $item2->[$indx_cmp]) {# if-elsif-else #
			push @common1, $item1;
			push @common2, $item2;
			$item1 = shift @sarray1;
			$item2 = shift @sarray2;

		} elsif ($item1->[$indx_cmp] <  $item2->[$indx_cmp]) {
			push @ext_a1, $item1;
			$item1 = shift @sarray1;

		} else {
			push @ext_a2, $item2;
			$item2 = shift @sarray2;
		}			# end of the if_elsif_else #

	}			# end of the 'infinite' while loop #

	return (\@common1, \@common2, \@ext_a1, \@ext_a2);

}		# end of the subroutine diff_array_of_nmarrays #


# union_nmarray: a subroutine which receives as arguments two references of arrays,
#		and returns the reference of  their union.
#
sub union_nmarray (\@\@) {

	my ($rf_a1,$rf_a2) = @_;
	my @union = ();

	my @sarray1 = sort { $a <=> $b} @{$rf_a1};
	my @sarray2 = sort { $a <=> $b} @{$rf_a2};

	my $item1 = shift @sarray1;
	my $item2 = shift @sarray2;

	while (1) {					# the 'infinite' while loop #

		unless (defined $item1) {
			if (defined $item2) {
				push @union, $item2;
				@sarray2 and (push @union, @sarray2); # cram the remaining elements #
			}
			last;
		}

		unless (defined $item2) {
			if (defined $item1) {
				push @union, $item1;
				@sarray1 and (push @union, @sarray1); # cram the remaining elements #
			}
			last;
		}

		if ($item1 == $item2) {			# if-elsif-else #
			push @union, $item1;
			$item1 = shift @sarray1;
			$item2 = shift @sarray2;

		} elsif ($item1 < $item2) {
			push @union, $item1;
			$item1 = shift @sarray1;

		} else {
			push @union, $item2;
			$item2 = shift @sarray2;
		}					# end of the if_elsif_else #

	}						# end of the 'infinite' while loop #

	return \@union;

}			# end of the module union_nmarray #


# nr_nmarray: a module which removes redundancy of a list.
#
sub nr_nmarray (\@) {

	my ($rf_array) = @_;

	(@{$rf_array} < 2) and (return $rf_array);

	my @sorted_array = sort { $a <=> $b} @{$rf_array};
	my $last = shift @sorted_array;
	my @output = ($last);

	while (defined (my $pres = shift @sorted_array) ) {

		($last == $pres) and next;

		$last = $pres;
		push @output, $pres;
	}

	return \@output;
}			# end of the module nr_nmarray #


# sum_nmarray: This subroutine calculates the sum of elements of an array (1-dim).
#
sub sum_nmarray (\@) {

	my ($rf_array) = @_;
	my $total = 0;

	for (my $i=0; $i<@{$rf_array}; $i++) {

		$total += $rf_array->[$i];
	}

	return $total;
}			# end of the subroutine sum_nmarray #


# nr_nmarray_graft: This subroutine 'graft's the 2nd array onto the 1st one in a NONREDUNDANT manner,
#			and without disturbing the original order.
#
sub nr_nmarray_graft (\@\@) {

	my ($rf_array1, $rf_array2) = @_;
	my @output = @{$rf_array1};

	OUTER:
	foreach my $ele2 (@{$rf_array2}) {
		foreach my $ele1 (@{$rf_array1}) {

			($ele1 == $ele2) and next OUTER; # dismiss $ele2 if its redundant #
		}

		push @output, $ele2;
	}

	return \@output;	
}						# end of the subroutine nr_array_graft #


# max_min: This subroutine receives an array of numbers ( $indx < 0),
#		or an array of arrays which have an numerical element (specified by $indx).
#	   Then returns maximun and minimum values.
#
sub max_min (\@$) {

	my ($array, $indx) = @_;

	my $ct_ele = @{$array};
	$ct_ele or (return (undef, undef) );

	##my $max = my $min = ($indx < 0) ? $array->[0] : $array->[0][$indx]; 
	my ($max, $min);

	for (my $i =0; $i < $ct_ele; $i++) {
		my $ele = $array->[$i];
		(defined $ele) or next;
		my $num = ($indx < 0) ? $ele : $ele->[$indx];
		(defined $num) or next;

		unless (defined $max) { # Initialize #
		    $max = $min = $num;
		}
		($num > $max) and ($max = $num);
		($num < $min) and ($min = $num);
	}

	return ($max, $min);

}		# end of the subroutine max_min #


# find_eqdiv_boundaries:
#	This subroutine finds a set of boundaries
#	which are supposed to divide an array info
#	subarrays of almost equal sizes.
#
# INPUT: (\@array, $indx, $num_div), where
#	$array[$m]->[$indx] (or $array[m] if $indx = undef or < 0) gives the measure
#	whose boundaries are to be looked for.
#	$num_div is the number of subarrays after the division.
#
# OUTPUT: \@set_boundaries.
#
# PREREQUISITE: The input array @array must be pre-sorted
#	in the acsending order of the measure $array[$m]->[$indx].
#
sub find_eqdiv_boundaries (\@$$) {

	my ($array, $indx, $num_div) = @_;

	my $fl_compo = ( (defined $indx) and ($indx >= 0) ) ? 1 : 0; 

	my $size_array = @{$array};

	if ($size_array < $num_div) {
	    print STDERR "[CAUTION: 'find_eqdiv_boundaries'] The array size $size_array is smaller than the number of sub-arrays $num_div.\n";
	}
	my @set_bds = ();

	for (my $k=1; $k<$num_div; $k++) { # the outermost for-loop #

	    my $bd = $k* ($size_array -1) /$num_div;

	    my $int_bd = int ($bd);
	    my $frac_bd = $bd - $int_bd;

	    my $b1 = $fl_compo ? $array->[$int_bd][$indx] : $array->[$int_bd];
	    my $b2 = $fl_compo ? $array->[$int_bd+1][$indx] : $array->[$int_bd+1];
	    my $boundary = $b1 + $frac_bd * ($b2 - $b1);

	    push @set_bds, $boundary;

	}			# end of the outermost for-loop #

	return \@set_bds;

}		# end of the subroutine 'find_eqdiv_boundaries' #


# find_eqdiv_wgt_boundaries:
#	This revised version of the subroutine 'find_eqdiv_boundaries'
#	finds a set of boundaries that divide an array into subarrays
#	of almost equal total weights.
#
# find_eqdiv_boundaries:
#	This subroutine finds a set of boundaries
#	which are supposed to divide an array info
#	subarrays of almost equal sizes.
#
# INPUT: (\@array, $indx_meas, $indx_wgt, $num_div, $lb_meas), where
#	$array[$m]->[$indx_meas] gives the measure
#	whose boundaries are to be looked for,
#	and $array[$m]->[$indx_wgt] gives the weight
#	that are to be equi-distributed;
#	$num_div is the number of subarrays after the division;
#	$lb_meas specifies the lower bound of the measure
#	(concerning this set).
#
# OUTPUT: ($total_wgt, \@set_boundaries), where
#	$total_wgt is the total summation of weights;
#	@set_boundaries stores the ($num_div - 1) boundaries of the measure
#	that equidivides $total_wgt into $num_div pieces.
#
# PREREQUISITE: The input array @array must be pre-sorted
#	in the acsending order of the measure $array[$m]->[$indx_meas].
#	And the inequality ($lb_meas <= $array[0]->[$indx_meas]) has to hold. 
#
sub find_eqdiv_wgt_boundaries (\@$$$$) {

	my ($array, $indx_meas, $indx_wgt, $num_div, $lb_meas) = @_;

	##my $fl_compo = ( (defined $indx) and ($indx >= 0) ) ? 1 : 0; 
	my $size_array = @{$array};

	unless ($lb_meas <= $array->[0][$indx_meas]) {
	    print STDERR "[ERROR 'find_eqdiv_wgt_boundaries'] The 0th measure (", $array->[0][$indx_meas], ") is NOT MORE THAN OR EQUAL TO the lower bound ($lb_meas), while it MUST BE: $! !\n";
	    return (undef, undef);
	}
	if ($size_array < $num_div) {
	    print STDERR "[CAUTION: 'find_eqdiv_wgt_boundaries'] The array size $size_array is smaller than the number of sub-arrays $num_div.\n";
	}
	my $total_wgt = 0;

		## A BUG FIXED on March 21 (Sat), 2009 ##
	for (my $i=0; $i<$size_array; $i++) { $total_wgt += $array->[$i][$indx_wgt]; } 
	##for (my $i=0; $i<$size_array; $i++) { $total_wgt += $array->[$i][$indx_wgt]++; } 
	my @set_bds = ();

=for DEBUGGING the subroutine
	print STDERR "\n< Subroutine 'find_eqdiv_wgt_boundaries' >\n\n";
	print STDERR "\$num_div = $num_div,  \$total_wgt = $total_wgt, \$size_array = $size_array, \$lb_meas = $lb_meas .\n\n";
=cut

	my $last_int_bd = 0;
	my $sum_wgt = 0;
	for (my $k=1; $k<$num_div; $k++) { # the outermost for-loop #

	    ##my $bd = $k* ($size_array -1) /$num_div;
	    ##my $int_bd = int ($bd);
	    ##my $frac_bd = $bd - $int_bd;
	    my $bd_wgt = $k * $total_wgt / $num_div;

	    ##my $b1 = $fl_compo ? $array->[$int_bd][$indx] : $array->[$int_bd];
	    ##my $b2 = $fl_compo ? $array->[$int_bd+1][$indx] : $array->[$int_bd+1];
	    ##my $boundary = $b1 + $frac_bd * ($b2 - $b1);

=for DEBUGGING the subroutine
	    print STDERR "[ The $k th round ]\n\n";
	    print STDERR "(before the inner for-loop)\n";
	    print STDERR "(\$indx_meas, \$indx_wgt) = ($indx_meas, $indx_wgt) ,\n";
	    print STDERR "\$bd_wgt = $bd_wgt,\n";
	    print STDERR "\$sum_wgt = $sum_wgt,\n";
	    print STDERR "\$last_int_bd = $last_int_bd .\n\n";
=cut

	    my $i; my $wgt;
	    for ($i=$last_int_bd; $i < $size_array; $i++) { # The inner for-oop #
		$wgt = $array->[$i][$indx_wgt];

=for DEBUGGING the subroutine
		print STDERR "The $i th inner iteration:  \$sum_wgt = $sum_wgt, \$wgt = $wgt .\n";
=cut

		($sum_wgt + $wgt > $bd_wgt) and last;
		$sum_wgt += $wgt;
	    }				# End of the inner for-loop #

	    my $meas = $array->[$i][$indx_meas];
	    my $last_meas = ($i==0) ? $lb_meas : $array->[$i-1][$indx_meas];

	    my $boundary = $last_meas + ($bd_wgt - $sum_wgt)/$wgt * ($meas - $last_meas) ;
	    push @set_bds, $boundary;

	    $last_int_bd = $i;

=for DEBUGGING the subroutine
	    print STDERR "(At the end of the outer for-loop)\n";
	    print STDERR "(\$indx_meas, \$indx_wgt) = ($indx_meas, $indx_wgt) ,\n";
	    print STDERR "(\$last_meas, \$meas) = ($last_meas, $meas),\n";
	    print STDERR "\$sum_wgt = $sum_wgt, \$wgt = $wgt,\n";
	    print STDERR "\$boundary = $boundary,\n";
	    print STDERR "\$last_int_bd = \$i = $last_int_bd .\n\n";
=cut
	}			# end of the outermost for-loop #

	return ($total_wgt, \@set_bds);

}		# end of the subroutine 'find_eqdiv_wgt_boundaries' #


# trim_numset:
#	This subroutine receives a (sorted) set of numbers,
#	and returns a trimmed set made by removing numbers
#	flanked by numbers not differing from it by $thrsh_diff.
#
# INPUT: (\@set_num, $thrsh_diff), where
#	@set_num is a list of numbers.
#
# OUTPUT: \@trimmed_nmset.
#
sub trim_numset (\@$) {

	my ($set_num, $thrsh_diff) = @_;

	my $ct_nums = @{$set_num};
	($ct_nums>0) or (return []);

	my @clusters = ( [$set_num->[0] ]);
	for (my $k=1; $k<$ct_nums; $k++) { # The 1st for-loop #
	    my $num = $set_num->[$k];
	    my $last_cluster = $clusters[$#clusters];
	    if ($num < $last_cluster->[$#{$last_cluster}] + $thrsh_diff) {
		push @{$last_cluster}, $num;
	    } else {
		push @clusters, [$num];
	    }
	}			# End of the 1st for-loop # 

	my @trimmed_nmset = ();
	foreach my $cluster (@clusters) { # The 2nd foreach-loop #
	    my $indx_last = $#{$cluster};
	    if ($indx_last>0) { # The cluster contains more than a number
		push @trimmed_nmset, @{$cluster}[0,$indx_last];

	    } else { # The cluster consists of a single number
		push @trimmed_nmset, $cluster->[0];
	    }
	}			# End of the 2nd foreach-loop #

	return \@trimmed_nmset;

}		# End of the subroutine 'trim_numset' #


# union_intsec_nmarrays:
#	This subroutine makes the union and the intersection of a set of numerical arrays.
#
# INPUT: (\@set_nmarrays, $indx), where
#	the $k th numerical array is @{$set_nmarrays[$k]->[$indx]} when $indx >=0,
#	@{$set_nmarrays[$k]} when $indx < 0.
#
# OUTPUT: (\@union_nmarrays, \@intsec_nmararys), where
#	@union_numarays and @intsec_nmarrays are the uion and the intersection
#	of the numerical arrays, respectively.
#
sub union_intsec_nmarrays (\@$) {

	my ($set_nmarrays, $indx) = @_;

	my $ct_arrays = @{$set_nmarrays};
	($ct_arrays>0) or (return ([], []));

		# Initialize the output #

	my @init_nmarray = ($indx<0) ?
		@{$set_nmarrays->[0]} : @{$set_nmarrays->[0][$indx]} ;
	my @cp_init_nmarray = @init_nmarray;
	my $union_nmarrays = \@init_nmarray;
	my $intsec_nmarrays = \@cp_init_nmarray;

	for (my $k=1; $k<$ct_arrays; $k++) { # The main foreach-loop #

	    my $element = $set_nmarrays->[$k];
	    my $nmarray = ($indx< 0) ? $element : $element->[$indx];
	    $union_nmarrays = union_nmarray (@{$union_nmarrays}, @{$nmarray});
	    ($intsec_nmarrays) = diff_nmarray (@{$intsec_nmarrays}, @{$nmarray});

	}			# End of the main foreach-loop #

	return ($union_nmarrays, $intsec_nmarrays);

}		# End of the subroutine 'union_intsec_nmarrays' #

# rescale_confining_vector:
#	This subroutine examines whether an input vector
#	neatly fits within a given set of boundaries, forming a hyperrectangle.
#	If not, it calculates the largest rescaling factor that makes the vector fit in.
#
# INPUT: (\@vec, \@base, \@lower, \@upper), where
#	@vec is the vector to be examined;
#	@base is the base point of the vector, which should be fixed even when rescaling;
#	$lower[$i] and $upper[$i] give the upper and the lower bounds for $base[$i] + $x * $vec[$i].
#
#	REQUISITE1: All the input vectors must have the same size.
#	REQUISITE2: @base must sit within the boundaries.
#
# OUTPUT: $x,
#	which is the rescaling  factor.
#	It is 1 if the input vector neatly fits within the boundaries.
#	Otherwise, it is the largest positive value that makes the vector fit in.
#
#	The subroutine returns undef if it failed.
#
sub rescale_confining_vector (\@\@\@\@) {

	my ($vec, $base, $lower, $upper) = @_;

	my $dim = @{$vec};

		# Check the input vectors (I) #
	unless (($dim == @{$base}) and ($dim == @{$lower}) and ($dim == @{$upper})) {
	    print STDERR "[ERROR 'rescale_confining_vector'] ILLEGAL DIMENSIONALITIES of the INPUT VECTORS: $! \n";
	    print STDERR '    (|@$vec|, |@$base|, |@$lower|, |@$upper|) = (';
	    print STDERR join (', ', $dim, scalar (@{$base}), scalar (@{$lower}), scalar (@{$upper}) ), ") .\n\n";
	    return undef;
	}

		# Check the input vectors (II) #
	for (my $k=0; $k<$dim; $k++) { # 1st for-loop #
	    unless (($lower->[$k] <= $base->[$k]) and ($base->[$k] <= $upper->[$k])) {
		print STDERR  "[ERROR 'rescale_confining_vector'] The $k th component of the \@\$base (", $base->[$k], ") is NOT within the specified interval \[", $lower->[$k], ', ', $upper->[$k], "]: $! !\n\n";
		return undef;
	    }
	}			# End of the 1st for-loop #

	     # Main Process #

	my $rescale = 1;
	for (my $k=0; $k<$dim; $k++) { # 2nd for-loop #

	    my $cmp = $vec->[$k];
	    if ($cmp > 0) {
		my $tmp = ($upper->[$k] - $base->[$k]) / $cmp;
		($rescale > $tmp) and ($rescale = $tmp);

	    } elsif ($cmp < 0) {
		my $tmp = ($lower->[$k] - $base->[$k]) / $cmp;
		($rescale > $tmp) and ($rescale = $tmp); 
	    }
		# Do nothing if $cmp == 0 #

	}			# End of the 2nd for-loop #

	return $rescale;

}	# End of the subroutine 'rescale_confining_vector' #


# log_sum_exp:
#	This subroutine calculates the logarithm
#	of the summation of the exponentials of the input numbers,
#	in a manner that does not ruin the benefit of the logarithmic representation.
#	The subroutine uses the identity:
#
#	log [Sum_{$i = 0, ..., $n} exp ($numbers[$i]) ]
#	= $n_max + log [Sum_{$i = 0, ..., $n} exp ($numbers[$i] - $n_max) ],
#	where $n is the count of input numbers, and $n_max = Max (@numbers).
#
# INPUT: \@numbers, which lists the input numbers.
#
# OUTPUT: $log_sum_exp, which equals
#	log [Sum_{$i = 0, ..., $n} exp ($numbers[$i]) ].
#
sub log_sum_exp (\@) {

	my ($numbers) = @_;

	(@{$numbers}>0) or (return undef);

	my $n_max ;
	foreach my $x (@{$numbers}) { # 1st foreach-loop # 
	    ((defined $n_max) and  ($n_max >= $x)) or ($n_max = $x);
	}			# End of the 1st foreach-loop #

	my $sum_small_exp = 0;
	foreach my $x (@{$numbers}) { # 2nd foreach-loop #
		$sum_small_exp += exp ($x - $n_max);
	}			# End of the 2nd foreach-loop #

	my $log_sum_exp = $n_max + log ($sum_small_exp) ;

	return $log_sum_exp;

}	# End of the subroutine 'log_sum_exp' #


# binsearch_num:
#	This subroutine conducts a binary search in a 'target' set of numbers
#	for a 'query' number.
#	The target set is assumed to be sorted in numerically ascending order.
#	(CREATED on Nov 5 (Sat), 2011)
#	
# binsearch_array:
#	This subroutine searches a target set of arrays (@set_arrays)
#	for a query array (@query) using a binary search algorithm.
#
# INPUT: ($query, \@set_nums, $indx_num, \@indices_relv), where
#	$query is the number to be searched for;
#	$set_nums[$k]->[$indx_num] if $indx_num is defined,
#	otherwise $set_nums[$k] itself, is the $k th number to be compared with the query;
#	@indices_rlv gives indices in a limited set to which the search will be narrowd down;
#
#	PREREQUISITE: Elements in @set_nums must be pre-sorted in numerically ascendig order.
#	and @indices_relv must also be sorted in numerically ascending order.
#
# OUTPUT: ($fl_exist, $indx2_qu, $indx_qu), where
#	$fl_exist = 1 if the query exists in the set,
#		  = 0 otherwise;
#	$indx_qu = $indices_relv[$indx2_qu] gives the index of the query
#	in @set_arrays (if $fl_exist = 1),
#	the index of the left neighbor of the query in the relevant subset of @set_array (if $fl_exist = 0)
#	or -1 if the query is 'smaller' than any arrays in @set_array[@indices_relv].
#
#	THe subroutine returns (undef, undef, undef) if the process failed.
#
sub binsearch_num ($\@$\@) {

	my ($query, $set_nums, $indx_num, $indices_relv) = @_;
	my $ct_nums = @{$set_nums};
	unless ($ct_nums>0) {
	    print STDERR "[ERROR 'binsearch_num'] ILLEGAL INPUT: NO NUMBERS in the set to be searched: $! !\n";
	    return (undef, undef, undef);
	}
	my $ct_rlv = @{$indices_relv};
	unless ($ct_rlv <= $ct_nums) {
	    print STDERR "[ERROR 'binsearch_num'] ILLEGAL INPUT: The number of relevant entries ($ct_rlv) EXCEEDS the number of numbers ($ct_nums): $! !\n";
	    return (undef, undef, undef);
	}

		# Compare the leftmost entry with the query #
	my $indx2_left = 0;
	my $indx_left = $indices_relv->[$indx2_left];
	my $entry_left = $set_nums->[$indx_left];
	my $num_left = (defined $indx_num) ? $entry_left->[$indx_num] : $entry_left;
	my $sgn_left = $query <=> $num_left;
		# compare_arrays (@{$query}, @{$array_left}, @{$indices_cmp}, @{$methods_cmp});
	unless (defined $sgn_left) {
	   print STDERR "[ERROR 'binsearch_num'] The operator '<=>' FAILED to compare the query with the $indx_left th number in the set: $! !\n";
	    return (undef, undef, undef);
	}
	if ($sgn_left < 0) { # The query is smaller than the smallest relevant number #
		return (0, -1, -1);
	} elsif ($sgn_left > 0) { # The query is larger than the smallest entry #
	    if ($ct_rlv == 1) { # Only the smallest entry is in the set of relevant numbers
		return (0, $indx2_left, $indx_left);
	    }
	} else { # The query equals the smallest number #
	    return (1, $indx2_left, $indx_left);
	}

		# Compare the rightmost entry with the query #
	my $indx2_right = $ct_rlv - 1;
	my $indx_right = $indices_relv->[$indx2_right];
	my $entry_right = $set_nums->[$indx_right];
	my $num_right = (defined $indx_num) ? $entry_right->[$indx_num] : $entry_right;
	my $sgn_right = $query <=> $num_right;
		# compare_arrays (@{$query}, @{$array_right}, @{$indices_cmp}, @{$methods_cmp});
	unless (defined $sgn_right) {
	    print STDERR "[ERROR 'binsearch_num'] The operator '<=>' FAILED to compare the query with the $indx_right th number in the target set: $! !\n";
	    return (undef, undef, undef);
	}
	if ($sgn_right > 0) { # The query is larger than the largest relevant number #
	    return (0, $indx2_right, $indx_right);

	} elsif ($sgn_right == 0) { # The query equals the largest number #
	    return (1, $indx2_right, $indx_right);
	}

		# Now, the query should be between the smallest and the largest relevant numbers #

		## Main Process:  Binary search ##

	while ($indx2_left + 1 < $indx2_right) { # Main while-loop # continues as long as there are indices in between $indx2_left and $indx2_right

		# Compare the middle entry with the query #

	    my $indx2_mid = int (($indx2_left + $indx2_right)/2);
	    my $indx_mid = $indices_relv->[$indx2_mid];
	    my $entry_mid = $set_nums->[$indx_mid];
	    my $num_mid = (defined $indx_num) ? $entry_mid->[$indx_num] : $entry_mid;
	    my $sgn_mid =  $query <=> $num_mid;
		# compare_arrays (@{$query}, @{$array_mid}, @{$indices_cmp}, @{$methods_cmp});
	    unless (defined $sgn_mid) {
		print STDERR "[ERROR 'binsearch_num'] The operator '<=>' FAILED to compare the query with the $indx_mid th number in the terget set: $! !\n";
		return (undef, undef, undef);
	    }

	    if ($sgn_mid < 0) { # The query is smaller than the middle number #
		$indx2_right = $indx2_mid; # Narrow the search down to the left half

	    } elsif ($sgn_mid > 0) { # The query is larger than the middle number # 
		$indx2_left = $indx2_mid; # Narraw the search down to the right half

	    } else { # The query equals the middle number #
		return (1, $indx2_mid, $indx_mid);
	    }
	}			# End of the main while-loop #

	$indx_left = $indices_relv->[$indx2_left];

	return (0, $indx2_left, $indx_left); # Found no relevant number that is equal to the query #

}	# End of the subroutine 'binsearch_num' #


# binsearch_num_d:
#	This slightly modified version of 'binsearch_num' conducts a binary search
#	in a 'target' set that are sorted in numerically DESCENDING order,
#	unlike the predecessor 'binsearch_num'.
#	(CREATED on Jan 29 (Sun), 2012)
#
# binsearch_num:
#	This subroutine conducts a binary search in a 'target' set of numbers
#	for a 'query' number.
#	The target set is assumed to be sorted in numerically ascending order.
#	(CREATED on Nov 5 (Sat), 2011)
#	
# binsearch_array:
#	This subroutine searches a target set of arrays (@set_arrays)
#	for a query array (@query) using a binary search algorithm.
#
# INPUT: ($query, \@set_nums, $indx_num, \@indices_relv), where
#	$query is the number to be searched for;
#	$set_nums[$k]->[$indx_num] if $indx_num is defined,
#	otherwise $set_nums[$k] itself, is the $k th number to be compared with the query;
#	@indices_rlv gives indices in a limited set to which the search will be narrowd down;
#
#	PREREQUISITE: Elements in @set_nums must be pre-sorted in numerically DESCENDING order.
#	and @indices_relv must be sorted in numerically ascending order.
#
# OUTPUT: ($fl_exist, $indx2_qu, $indx_qu), where
#	$fl_exist = 1 if the query exists in the set,
#		  = 0 otherwise;
#	$indx_qu = $indices_relv[$indx2_qu] gives the index of the query
#	in @set_arrays (if $fl_exist = 1),
#	the index of the left neighbor of the query in the relevant subset of @set_array (if $fl_exist = 0)
#	or -1 if the query is 'larger' than any arrays in @set_array[@indices_relv].
#
#	THe subroutine returns (undef, undef, undef) if the process failed.
#
sub binsearch_num_d ($\@$\@) {

	my ($query, $set_nums, $indx_num, $indices_relv) = @_;
	my $ct_nums = @{$set_nums};
	unless ($ct_nums>0) {
	    print STDERR "[ERROR 'binsearch_num_d'] ILLEGAL INPUT: NO NUMBERS in the set to be searched: $! !\n";
	    return (undef, undef, undef);
	}
	my $ct_rlv = @{$indices_relv};
	unless ($ct_rlv <= $ct_nums) {
	    print STDERR "[ERROR 'binsearch_num_d'] ILLEGAL INPUT: The number of relevant entries ($ct_rlv) EXCEEDS the number of numbers ($ct_nums): $! !\n";
	    return (undef, undef, undef);
	}

		# Compare the leftmost entry with the query #
	my $indx2_left = 0;
	my $indx_left = $indices_relv->[$indx2_left];
	my $entry_left = $set_nums->[$indx_left];
	my $num_left = (defined $indx_num) ? $entry_left->[$indx_num] : $entry_left;
	my $sgn_left = $query <=> $num_left;
		# compare_arrays (@{$query}, @{$array_left}, @{$indices_cmp}, @{$methods_cmp});
	unless (defined $sgn_left) {
	   print STDERR "[ERROR 'binsearch_num_d'] The operator '<=>' FAILED to compare the query with the $indx_left th number in the set: $! !\n";
	    return (undef, undef, undef);
	}
	if ($sgn_left > 0) { # The query is larger than the largest relevant number #
	##if ($sgn_left < 0) { # The query is smaller than the smallest relevant number #
		return (0, -1, -1);
	} elsif ($sgn_left < 0) { # The query is smaller than the largest entry #
	##} elsif ($sgn_left > 0) { # The query is larger than the smallest entry #
	    if ($ct_rlv == 1) { # Only the largest entry is in the set of relevant numbers
		return (0, $indx2_left, $indx_left);
	    }
	} else { # The query equals the largest number #
	    return (1, $indx2_left, $indx_left);
	}

		# Compare the rightmost entry with the query #
	my $indx2_right = $ct_rlv - 1;
	my $indx_right = $indices_relv->[$indx2_right];
	my $entry_right = $set_nums->[$indx_right];
	my $num_right = (defined $indx_num) ? $entry_right->[$indx_num] : $entry_right;
	my $sgn_right = $query <=> $num_right;
		# compare_arrays (@{$query}, @{$array_right}, @{$indices_cmp}, @{$methods_cmp});
	unless (defined $sgn_right) {
	    print STDERR "[ERROR 'binsearch_num_d'] The operator '<=>' FAILED to compare the query with the $indx_right th number in the target set: $! !\n";
	    return (undef, undef, undef);
	}
	if ($sgn_right < 0) { # The query is smaller than the smallest relevant number #
	##if ($sgn_right > 0) { # The query is larger than the largest relevant number #
	    return (0, $indx2_right, $indx_right);

	} elsif ($sgn_right == 0) { # The query equals the smallest number #
	    return (1, $indx2_right, $indx_right);
	}

		# Now, the query should be between the smallest and the largest relevant numbers #

		## Main Process:  Binary search ##

	while ($indx2_left + 1 < $indx2_right) { # Main while-loop # continues as long as there are indices in between $indx2_left and $indx2_right

		# Compare the middle entry with the query #

	    my $indx2_mid = int (($indx2_left + $indx2_right)/2);
	    my $indx_mid = $indices_relv->[$indx2_mid];
	    my $entry_mid = $set_nums->[$indx_mid];
	    my $num_mid = (defined $indx_num) ? $entry_mid->[$indx_num] : $entry_mid;
	    my $sgn_mid =  $query <=> $num_mid;
		# compare_arrays (@{$query}, @{$array_mid}, @{$indices_cmp}, @{$methods_cmp});
	    unless (defined $sgn_mid) {
		print STDERR "[ERROR 'binsearch_num_d'] The operator '<=>' FAILED to compare the query with the $indx_mid th number in the terget set: $! !\n";
		return (undef, undef, undef);
	    }

	    if ($sgn_mid > 0) { # The query is larger than the middle number #
	    ##if ($sgn_mid < 0) { # The query is smaller than the middle number #
		$indx2_right = $indx2_mid; # Narrow the search down to the left half

	    } elsif ($sgn_mid < 0) { # The query is smaller than the middle number # 
	    ##} elsif ($sgn_mid > 0) { # The query is larger than the middle number # 
		$indx2_left = $indx2_mid; # Narrow the search down to the right half

	    } else { # The query equals the middle number #
		return (1, $indx2_mid, $indx_mid);
	    }
	}			# End of the main while-loop #

	$indx_left = $indices_relv->[$indx2_left];

	return (0, $indx2_left, $indx_left); # Found no relevant number that is equal to the query #

}	# End of the subroutine 'binsearch_num_d' #


1;		# a 'spell' which makes the module 'use'-able and 'require'-able #
