package MyLASTPIECE_Sppl;
#
###################################################################################
#	
#	LASTPIECE_P (Local Alignment-STate Probabilities that Insertion-type and dEletion-type gaps Co-Exist, type P):
#	    A package of programs to compute the probabilities of gapped segments in each of which an insertion-type gap and a deletion-type gap co-exist, (which were referred to as case-(iv) gapped segments by Ezawa (2016a?)),
#	    under a stochastic model of sequence evolution with biologically realistic insertions/deletions.
#	   (Written almost exclusively in Perl.)
#
#	Version 0.3: Copyright (C) 2020 Kiyoshi Ezawa
#
#    This package is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This package is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License, "GNU_GPL.txt",
#    along with this package.  If not, see <http://www.gnu.org/licenses/>.
#
#    The author can be contacted by e-mailing to <kezawa dot ezawa3 at gmail dot com>
#    (replace " dot " and " at " with "." and "@", respectively).
#
###################################################################################
#
# MyLASTPIECE_Sppl.pm:
#	This Perl package consists of some subroutines that supplement the main functions of the
#	main (master) and servant scripts of the package, 'LASTPIECE_P'.
#	[CREATION STARTED on 2020/01/27, ENDED on 2020/mm/dd.]

use strict;
use warnings;

use MyFile_manipu qw(
	open_input_file2
	open_output_file
	); # ADDED 'open_output_file' in creation of 'detect_purge_cands.ver0.5.pl' (on 2019/06/17). #
	# list_sessions_ext
	# tfm_labels2indices


use Exporter;

our @ISA =qw(Exporter);

our @EXPORT_OK = qw( 
	cal_delta_exp_del_5p2
	);

sub cal_delta_exp_del_5p2 (\@$) ; # IMPORTED from "ANEX_P.ver0.5/MyPerlModules_LOLIPOG/MyTreeMap_indels_ML_hs.pm" on 2020/01/27.

		### DEFINITIONS of the Subroutines. ###

# cal_delta_exp_del_5p2:
#	This subroutine was created by modifying its predecessor, 'cal_delta_exp_del_5p',
#	so that it can calculate the factor:
#
#	$delta_del_5p = Sum_{$l=1}^{$cutoff_5p} ($l-1) * distri_del ($l),
#	where distri_del ($l) is the relative frequency of deletions with length $l,
#
#	from the user-specified set of deletion frequencies.
#
#	(The suffix, '_5p', is a trace of the fact that it was originally interpreted as
#	the factor at the 5'-end of the PWA (as in the method of Miklos et al. (2004)).
#	Actually, it could have a more general interpretation,
#	which is not necessarily limited to the 5'-end.)
#
#	[CREATED on May 16 (Sat), 2015.]
#	[IMPORTED from "ANEX_P.ver0.5/MyPerlModules_LOLIPOG/MyTreeMap_indels_ML_hs.pm" on 2020/01/27.]
#
# INPUT: (\@freqs_del, $cutoff_del), where
#
#	$freqs_del[$l] ($l=1, ..., $cutoff_del) is the frequency of $l-residue deletions;
#	$cutoff_del is the cutoff-value for deletion lengths.
#
#	NOTE: Even if @freqs_del is not normalized to sum up to unity,
#	     the subroutine uses the input frequencies as they are.
#
# OUTPUT: $delta_del_5p, which is defined above.
#
sub cal_delta_exp_del_5p2 (\@$) {

	my ($freqs_del, $cutoff_del) = @_;

	my $delta_del_5p = 0;

	for (my $l=1; $l<$cutoff_del; $l++) { # Main for-loop (over deletion lengths). #

	    $delta_del_5p += ($l-1) * $freqs_del->[$l];

	}			# End of the main for-loop (over deletion lengths). #

	return $delta_del_5p ;

}	# End of the subroutine 'cal_delta_exp_del_5p2'. #




1;		# a "spell" which makes the module "use"-able and/or "require"-able #


